@@ -32,6 +32,8 @@
 #include <QObject>
 #include <QTimer>
 
+#include "base/path.h"
+
 namespace Log
 {
     struct Msg;
@@ -50,10 +52,10 @@ public:
         YEARS
     };
 
-    FileLogger(const QString &path, bool backup, int maxSize, bool deleteOld, int age, FileLogAgeType ageType);
+    FileLogger(const Path &path, bool backup, int maxSize, bool deleteOld, int age, FileLogAgeType ageType);
     ~FileLogger();
 
-    void changePath(const QString &newPath);
+    void changePath(const Path &newPath);
     void deleteOld(int age, FileLogAgeType ageType);
     void setBackup(bool value);
     void setMaxSize(int value);
@@ -66,7 +68,7 @@ private:
     void openLogFile();
     void closeLogFile();
 
-    QString m_path;
+    Path m_path;
     bool m_backup;
     int m_maxSize;
     QFile m_logFile;
