@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2021  Vladimir Golovnev <glassez@yandex.ru>
+ * Copyright (C) 2021-2022  Vladimir Golovnev <glassez@yandex.ru>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,12 +43,14 @@
 #include <QSqlDatabase>
 #include <QSqlError>
 #include <QSqlQuery>
+#include <QSqlRecord>
 #include <QThread>
 #include <QVector>
 
 #include "base/exceptions.h"
 #include "base/global.h"
 #include "base/logger.h"
+#include "base/path.h"
 #include "base/profile.h"
 #include "base/utils/fs.h"
 #include "base/utils/string.h"
@@ -57,14 +59,14 @@
 
 namespace
 {
-    const char DB_CONNECTION_NAME[] = "ResumeDataStorage";
+    const QString DB_CONNECTION_NAME = u"ResumeDataStorage"_qs;
 
-    const int DB_VERSION = 2;
+    const int DB_VERSION = 3;
 
-    const char DB_TABLE_META[] = "meta";
-    const char DB_TABLE_TORRENTS[] = "torrents";
+    const QString DB_TABLE_META = u"meta"_qs;
+    const QString DB_TABLE_TORRENTS = u"torrents"_qs;
 
-    const char META_VERSION[] = "version";
+    const QString META_VERSION = u"version"_qs;
 
     struct Column
     {
@@ -74,7 +76,7 @@ namespace
 
     Column makeColumn(const char *columnName)
     {
-        return {QLatin1String(columnName), (QLatin1Char(':') + QLatin1String(columnName))};
+        return {QString::fromLatin1(columnName), (u':' + QString::fromLatin1(columnName))};
     }
 
     const Column DB_COLUMN_ID = makeColumn("id");
@@ -92,6 +94,7 @@ namespace
     const Column DB_COLUMN_HAS_SEED_STATUS = makeColumn("has_seed_status");
     const Column DB_COLUMN_OPERATING_MODE = makeColumn("operating_mode");
     const Column DB_COLUMN_STOPPED = makeColumn("stopped");
+    const Column DB_COLUMN_STOP_CONDITION = makeColumn("stop_condition");
     const Column DB_COLUMN_RESUMEDATA = makeColumn("libtorrent_resume_data");
     const Column DB_COLUMN_METADATA = makeColumn("metadata");
     const Column DB_COLUMN_VALUE = makeColumn("value");
@@ -104,14 +107,14 @@ namespace
 
     QString quoted(const QString &name)
     {
-        const QLatin1Char quote {'`'};
+        const QChar quote = u'`';
 
         return (quote + name + quote);
     }
 
     QString makeCreateTableStatement(const QString &tableName, const QStringList &items)
     {
-        return QString::fromLatin1("CREATE TABLE %1 (%2)").arg(quoted(tableName), items.join(QLatin1Char(',')));
+        return u"CREATE TABLE %1 (%2)"_qs.arg(quoted(tableName), items.join(u','));
     }
 
     std::pair<QString, QString> joinColumns(const QVector<Column> &columns)
@@ -130,8 +133,8 @@ namespace
         values.reserve(valuesSize);
         for (const Column &column : columns)
         {
-            names.append(quoted(column.name) + QLatin1Char(','));
-            values.append(column.placeholder + QLatin1Char(','));
+            names.append(quoted(column.name) + u',');
+            values.append(column.placeholder + u',');
         }
         names.chop(1);
         values.chop(1);
@@ -142,27 +145,27 @@ namespace
     QString makeInsertStatement(const QString &tableName, const QVector<Column> &columns)
     {
         const auto [names, values] = joinColumns(columns);
-        return QString::fromLatin1("INSERT INTO %1 (%2) VALUES (%3)")
+        return u"INSERT INTO %1 (%2) VALUES (%3)"_qs
                 .arg(quoted(tableName), names, values);
     }
 
     QString makeUpdateStatement(const QString &tableName, const QVector<Column> &columns)
     {
         const auto [names, values] = joinColumns(columns);
-        return QString::fromLatin1("UPDATE %1 SET (%2) = (%3)")
+        return u"UPDATE %1 SET (%2) = (%3)"_qs
                 .arg(quoted(tableName), names, values);
     }
 
     QString makeOnConflictUpdateStatement(const Column &constraint, const QVector<Column> &columns)
     {
         const auto [names, values] = joinColumns(columns);
-        return QString::fromLatin1(" ON CONFLICT (%1) DO UPDATE SET (%2) = (%3)")
+        return u" ON CONFLICT (%1) DO UPDATE SET (%2) = (%3)"_qs
                 .arg(quoted(constraint.name), names, values);
     }
 
     QString makeColumnDefinition(const Column &column, const char *definition)
     {
-        return QString::fromLatin1("%1 %2").arg(quoted(column.name), QLatin1String(definition));
+        return u"%1 %2"_qs.arg(quoted(column.name), QString::fromLatin1(definition));
     }
 }
 
@@ -173,7 +176,7 @@ namespace BitTorrent
         Q_DISABLE_COPY_MOVE(Worker)
 
     public:
-        Worker(const QString &dbPath, const QString &dbConnectionName);
+        Worker(const Path &dbPath, const QString &dbConnectionName, QReadWriteLock &dbLock);
 
         void openDatabase() const;
         void closeDatabase() const;
@@ -183,19 +186,82 @@ namespace BitTorrent
         void storeQueue(const QVector<TorrentID> &queue) const;
 
     private:
-        const QString m_path;
+        const Path m_path;
         const QString m_connectionName;
+        QReadWriteLock &m_dbLock;
     };
+
+    namespace
+    {
+        LoadTorrentParams parseQueryResultRow(const QSqlQuery &query)
+        {
+            LoadTorrentParams resumeData;
+            resumeData.name = query.value(DB_COLUMN_NAME.name).toString();
+            resumeData.category = query.value(DB_COLUMN_CATEGORY.name).toString();
+            const QString tagsData = query.value(DB_COLUMN_TAGS.name).toString();
+            if (!tagsData.isEmpty())
+            {
+                const QStringList tagList = tagsData.split(u',');
+                resumeData.tags.insert(tagList.cbegin(), tagList.cend());
+            }
+            resumeData.hasSeedStatus = query.value(DB_COLUMN_HAS_SEED_STATUS.name).toBool();
+            resumeData.firstLastPiecePriority = query.value(DB_COLUMN_HAS_OUTER_PIECES_PRIORITY.name).toBool();
+            resumeData.ratioLimit = query.value(DB_COLUMN_RATIO_LIMIT.name).toInt() / 1000.0;
+            resumeData.seedingTimeLimit = query.value(DB_COLUMN_SEEDING_TIME_LIMIT.name).toInt();
+            resumeData.contentLayout = Utils::String::toEnum<TorrentContentLayout>(
+                        query.value(DB_COLUMN_CONTENT_LAYOUT.name).toString(), TorrentContentLayout::Original);
+            resumeData.operatingMode = Utils::String::toEnum<TorrentOperatingMode>(
+                        query.value(DB_COLUMN_OPERATING_MODE.name).toString(), TorrentOperatingMode::AutoManaged);
+            resumeData.stopped = query.value(DB_COLUMN_STOPPED.name).toBool();
+            resumeData.stopCondition = Utils::String::toEnum(
+                        query.value(DB_COLUMN_STOP_CONDITION.name).toString(), Torrent::StopCondition::None);
+
+            resumeData.savePath = Profile::instance()->fromPortablePath(
+                        Path(query.value(DB_COLUMN_TARGET_SAVE_PATH.name).toString()));
+            resumeData.useAutoTMM = resumeData.savePath.isEmpty();
+            if (!resumeData.useAutoTMM)
+            {
+                resumeData.downloadPath = Profile::instance()->fromPortablePath(
+                            Path(query.value(DB_COLUMN_DOWNLOAD_PATH.name).toString()));
+            }
+
+            const QByteArray bencodedResumeData = query.value(DB_COLUMN_RESUMEDATA.name).toByteArray();
+
+            lt::error_code ec;
+            const lt::bdecode_node resumeDataRoot = lt::bdecode(bencodedResumeData, ec);
+
+            lt::add_torrent_params &p = resumeData.ltAddTorrentParams;
+
+            p = lt::read_resume_data(resumeDataRoot, ec);
+
+            if (const QByteArray bencodedMetadata = query.value(DB_COLUMN_METADATA.name).toByteArray(); !bencodedMetadata.isEmpty())
+            {
+                const lt::bdecode_node torentInfoRoot = lt::bdecode(bencodedMetadata, ec);
+                p.ti = std::make_shared<lt::torrent_info>(torentInfoRoot, ec);
+            }
+
+            p.save_path = Profile::instance()->fromPortablePath(Path(fromLTString(p.save_path)))
+                    .toString().toStdString();
+
+            if (p.flags & lt::torrent_flags::stop_when_ready)
+            {
+                p.flags &= ~lt::torrent_flags::stop_when_ready;
+                resumeData.stopCondition = Torrent::StopCondition::FilesChecked;
+            }
+
+            return resumeData;
+        }
+    }
 }
 
-BitTorrent::DBResumeDataStorage::DBResumeDataStorage(const QString &dbPath, QObject *parent)
-    : ResumeDataStorage {parent}
-    , m_ioThread {new QThread(this)}
+BitTorrent::DBResumeDataStorage::DBResumeDataStorage(const Path &dbPath, QObject *parent)
+    : ResumeDataStorage(dbPath, parent)
+    , m_ioThread {new QThread}
 {
-    const bool needCreateDB = !QFile::exists(dbPath);
+    const bool needCreateDB = !dbPath.exists();
 
-    auto db = QSqlDatabase::addDatabase(QLatin1String("QSQLITE"), DB_CONNECTION_NAME);
-    db.setDatabaseName(dbPath);
+    auto db = QSqlDatabase::addDatabase(u"QSQLITE"_qs, DB_CONNECTION_NAME);
+    db.setDatabaseName(dbPath.data());
     if (!db.open())
         throw RuntimeError(db.lastError().text());
 
@@ -205,14 +271,14 @@ BitTorrent::DBResumeDataStorage::DBResumeDataStorage(const QString &dbPath, QObj
     }
     else
     {
-        const int dbVersion = currentDBVersion();
-        if (dbVersion == 1)
-            updateDBFromVersion1();
+        const int dbVersion = (!db.record(DB_TABLE_TORRENTS).contains(DB_COLUMN_DOWNLOAD_PATH.name) ? 1 : currentDBVersion());
+        if (dbVersion != DB_VERSION)
+            updateDB(dbVersion);
     }
 
-    m_asyncWorker = new Worker(dbPath, QLatin1String("ResumeDataStorageWorker"));
-    m_asyncWorker->moveToThread(m_ioThread);
-    connect(m_ioThread, &QThread::finished, m_asyncWorker, &QObject::deleteLater);
+    m_asyncWorker = new Worker(dbPath, u"ResumeDataStorageWorker"_qs, m_dbLock);
+    m_asyncWorker->moveToThread(m_ioThread.get());
+    connect(m_ioThread.get(), &QThread::finished, m_asyncWorker, &QObject::deleteLater);
     m_ioThread->start();
 
     RuntimeError *errPtr = nullptr;
@@ -236,14 +302,11 @@ BitTorrent::DBResumeDataStorage::~DBResumeDataStorage()
 {
     QMetaObject::invokeMethod(m_asyncWorker, &Worker::closeDatabase);
     QSqlDatabase::removeDatabase(DB_CONNECTION_NAME);
-
-    m_ioThread->quit();
-    m_ioThread->wait();
 }
 
 QVector<BitTorrent::TorrentID> BitTorrent::DBResumeDataStorage::registeredTorrents() const
 {
-    const auto selectTorrentIDStatement = QString::fromLatin1("SELECT %1 FROM %2 ORDER BY %3;")
+    const auto selectTorrentIDStatement = u"SELECT %1 FROM %2 ORDER BY %3;"_qs
             .arg(quoted(DB_COLUMN_TORRENT_ID.name), quoted(DB_TABLE_TORRENTS), quoted(DB_COLUMN_QUEUE_POSITION.name));
 
     auto db = QSqlDatabase::database(DB_CONNECTION_NAME);
@@ -260,11 +323,10 @@ QVector<BitTorrent::TorrentID> BitTorrent::DBResumeDataStorage::registeredTorren
     return registeredTorrents;
 }
 
-std::optional<BitTorrent::LoadTorrentParams> BitTorrent::DBResumeDataStorage::load(const TorrentID &id) const
+BitTorrent::LoadResumeDataResult BitTorrent::DBResumeDataStorage::load(const TorrentID &id) const
 {
-    const QString selectTorrentStatement =
-            QString(QLatin1String("SELECT * FROM %1 WHERE %2 = %3;"))
-            .arg(quoted(DB_TABLE_TORRENTS), quoted(DB_COLUMN_TORRENT_ID.name), DB_COLUMN_TORRENT_ID.placeholder);
+    const QString selectTorrentStatement = u"SELECT * FROM %1 WHERE %2 = %3;"_qs
+        .arg(quoted(DB_TABLE_TORRENTS), quoted(DB_COLUMN_TORRENT_ID.name), DB_COLUMN_TORRENT_ID.placeholder);
 
     auto db = QSqlDatabase::database(DB_CONNECTION_NAME);
     QSqlQuery query {db};
@@ -282,55 +344,11 @@ std::optional<BitTorrent::LoadTorrentParams> BitTorrent::DBResumeDataStorage::lo
     }
     catch (const RuntimeError &err)
     {
-        LogMsg(tr("Couldn't load resume data of torrent '%1'. Error: %2")
-            .arg(id.toString(), err.message()), Log::CRITICAL);
-        return std::nullopt;
+        return nonstd::make_unexpected(tr("Couldn't load resume data of torrent '%1'. Error: %2")
+            .arg(id.toString(), err.message()));
     }
 
-    LoadTorrentParams resumeData;
-    resumeData.restored = true;
-    resumeData.name = query.value(DB_COLUMN_NAME.name).toString();
-    resumeData.category = query.value(DB_COLUMN_CATEGORY.name).toString();
-    const QString tagsData = query.value(DB_COLUMN_TAGS.name).toString();
-    if (!tagsData.isEmpty())
-    {
-        const QStringList tagList = tagsData.split(QLatin1Char(','));
-        resumeData.tags.insert(tagList.cbegin(), tagList.cend());
-    }
-    resumeData.hasSeedStatus = query.value(DB_COLUMN_HAS_SEED_STATUS.name).toBool();
-    resumeData.firstLastPiecePriority = query.value(DB_COLUMN_HAS_OUTER_PIECES_PRIORITY.name).toBool();
-    resumeData.ratioLimit = query.value(DB_COLUMN_RATIO_LIMIT.name).toInt() / 1000.0;
-    resumeData.seedingTimeLimit = query.value(DB_COLUMN_SEEDING_TIME_LIMIT.name).toInt();
-    resumeData.contentLayout = Utils::String::toEnum<TorrentContentLayout>(
-                query.value(DB_COLUMN_CONTENT_LAYOUT.name).toString(), TorrentContentLayout::Original);
-    resumeData.operatingMode = Utils::String::toEnum<TorrentOperatingMode>(
-                query.value(DB_COLUMN_OPERATING_MODE.name).toString(), TorrentOperatingMode::AutoManaged);
-    resumeData.stopped = query.value(DB_COLUMN_STOPPED.name).toBool();
-
-    resumeData.savePath = Profile::instance()->fromPortablePath(
-                Utils::Fs::toUniformPath(query.value(DB_COLUMN_TARGET_SAVE_PATH.name).toString()));
-    resumeData.useAutoTMM = resumeData.savePath.isEmpty();
-    if (!resumeData.useAutoTMM)
-    {
-        resumeData.downloadPath = Profile::instance()->fromPortablePath(
-                    Utils::Fs::toUniformPath(query.value(DB_COLUMN_DOWNLOAD_PATH.name).toString()));
-    }
-
-    const QByteArray bencodedResumeData = query.value(DB_COLUMN_RESUMEDATA.name).toByteArray();
-    const QByteArray bencodedMetadata = query.value(DB_COLUMN_METADATA.name).toByteArray();
-    const QByteArray allData = ((bencodedMetadata.isEmpty() || bencodedResumeData.isEmpty())
-                                ? bencodedResumeData
-                                : (bencodedResumeData.chopped(1) + bencodedMetadata.mid(1)));
-
-    lt::error_code ec;
-    const lt::bdecode_node root = lt::bdecode(allData, ec);
-
-    lt::add_torrent_params &p = resumeData.ltAddTorrentParams;
-
-    p = lt::read_resume_data(root, ec);
-    p.save_path = Profile::instance()->fromPortablePath(fromLTString(p.save_path)).toStdString();
-
-    return resumeData;
+    return parseQueryResultRow(query);
 }
 
 void BitTorrent::DBResumeDataStorage::store(const TorrentID &id, const LoadTorrentParams &resumeData) const
@@ -357,9 +375,52 @@ void BitTorrent::DBResumeDataStorage::storeQueue(const QVector<TorrentID> &queue
     });
 }
 
+void BitTorrent::DBResumeDataStorage::doLoadAll() const
+{
+    const QString connectionName = u"ResumeDataStorageLoadAll"_qs;
+
+    {
+        auto db = QSqlDatabase::addDatabase(u"QSQLITE"_qs, connectionName);
+        db.setDatabaseName(path().data());
+        if (!db.open())
+            throw RuntimeError(db.lastError().text());
+
+        QSqlQuery query {db};
+
+        const auto selectTorrentIDStatement = u"SELECT %1 FROM %2 ORDER BY %3;"_qs
+                .arg(quoted(DB_COLUMN_TORRENT_ID.name), quoted(DB_TABLE_TORRENTS), quoted(DB_COLUMN_QUEUE_POSITION.name));
+
+        const QReadLocker locker {&m_dbLock};
+
+        if (!query.exec(selectTorrentIDStatement))
+            throw RuntimeError(query.lastError().text());
+
+        QVector<TorrentID> registeredTorrents;
+        registeredTorrents.reserve(query.size());
+        while (query.next())
+            registeredTorrents.append(TorrentID::fromString(query.value(0).toString()));
+
+        emit const_cast<DBResumeDataStorage *>(this)->loadStarted(registeredTorrents);
+
+        const auto selectStatement = u"SELECT * FROM %1 ORDER BY %2;"_qs.arg(quoted(DB_TABLE_TORRENTS), quoted(DB_COLUMN_QUEUE_POSITION.name));
+        if (!query.exec(selectStatement))
+            throw RuntimeError(query.lastError().text());
+
+        while (query.next())
+        {
+            const auto torrentID = TorrentID::fromString(query.value(DB_COLUMN_TORRENT_ID.name).toString());
+            onResumeDataLoaded(torrentID, parseQueryResultRow(query));
+        }
+    }
+
+    emit const_cast<DBResumeDataStorage *>(this)->loadFinished();
+
+    QSqlDatabase::removeDatabase(connectionName);
+}
+
 int BitTorrent::DBResumeDataStorage::currentDBVersion() const
 {
-    const auto selectDBVersionStatement = QString::fromLatin1("SELECT %1 FROM %2 WHERE %3 = %4;")
+    const auto selectDBVersionStatement = u"SELECT %1 FROM %2 WHERE %3 = %4;"_qs
             .arg(quoted(DB_COLUMN_VALUE.name), quoted(DB_TABLE_META), quoted(DB_COLUMN_NAME.name), DB_COLUMN_NAME.placeholder);
 
     auto db = QSqlDatabase::database(DB_CONNECTION_NAME);
@@ -368,7 +429,9 @@ int BitTorrent::DBResumeDataStorage::currentDBVersion() const
     if (!query.prepare(selectDBVersionStatement))
         throw RuntimeError(query.lastError().text());
 
-    query.bindValue(DB_COLUMN_NAME.placeholder, QString::fromLatin1(META_VERSION));
+    query.bindValue(DB_COLUMN_NAME.placeholder, META_VERSION);
+
+    const QReadLocker locker {&m_dbLock};
 
     if (!query.exec())
         throw RuntimeError(query.lastError().text());
@@ -388,6 +451,8 @@ void BitTorrent::DBResumeDataStorage::createDB() const
 {
     auto db = QSqlDatabase::database(DB_CONNECTION_NAME);
 
+    const QWriteLocker locker {&m_dbLock};
+
     if (!db.transaction())
         throw RuntimeError(db.lastError().text());
 
@@ -408,7 +473,7 @@ void BitTorrent::DBResumeDataStorage::createDB() const
         if (!query.prepare(insertMetaVersionQuery))
             throw RuntimeError(query.lastError().text());
 
-        query.bindValue(DB_COLUMN_NAME.placeholder, QString::fromLatin1(META_VERSION));
+        query.bindValue(DB_COLUMN_NAME.placeholder, META_VERSION);
         query.bindValue(DB_COLUMN_VALUE.placeholder, DB_VERSION);
 
         if (!query.exec())
@@ -422,6 +487,7 @@ void BitTorrent::DBResumeDataStorage::createDB() const
             makeColumnDefinition(DB_COLUMN_CATEGORY, "TEXT"),
             makeColumnDefinition(DB_COLUMN_TAGS, "TEXT"),
             makeColumnDefinition(DB_COLUMN_TARGET_SAVE_PATH, "TEXT"),
+            makeColumnDefinition(DB_COLUMN_DOWNLOAD_PATH, "TEXT"),
             makeColumnDefinition(DB_COLUMN_CONTENT_LAYOUT, "TEXT NOT NULL"),
             makeColumnDefinition(DB_COLUMN_RATIO_LIMIT, "INTEGER NOT NULL"),
             makeColumnDefinition(DB_COLUMN_SEEDING_TIME_LIMIT, "INTEGER NOT NULL"),
@@ -429,6 +495,7 @@ void BitTorrent::DBResumeDataStorage::createDB() const
             makeColumnDefinition(DB_COLUMN_HAS_SEED_STATUS, "INTEGER NOT NULL"),
             makeColumnDefinition(DB_COLUMN_OPERATING_MODE, "TEXT NOT NULL"),
             makeColumnDefinition(DB_COLUMN_STOPPED, "INTEGER NOT NULL"),
+            makeColumnDefinition(DB_COLUMN_STOP_CONDITION, "TEXT NOT NULL DEFAULT `None`"),
             makeColumnDefinition(DB_COLUMN_RESUMEDATA, "BLOB NOT NULL"),
             makeColumnDefinition(DB_COLUMN_METADATA, "BLOB")
         };
@@ -446,10 +513,15 @@ void BitTorrent::DBResumeDataStorage::createDB() const
     }
 }
 
-void BitTorrent::DBResumeDataStorage::updateDBFromVersion1() const
+void BitTorrent::DBResumeDataStorage::updateDB(const int fromVersion) const
 {
+    Q_ASSERT(fromVersion > 0);
+    Q_ASSERT(fromVersion != DB_VERSION);
+
     auto db = QSqlDatabase::database(DB_CONNECTION_NAME);
 
+    const QWriteLocker locker {&m_dbLock};
+
     if (!db.transaction())
         throw RuntimeError(db.lastError().text());
 
@@ -457,16 +529,27 @@ void BitTorrent::DBResumeDataStorage::updateDBFromVersion1() const
 
     try
     {
-        const auto alterTableTorrentsQuery = QString::fromLatin1("ALTER TABLE %1 ADD %2")
-                .arg(quoted(DB_TABLE_TORRENTS), makeColumnDefinition(DB_COLUMN_DOWNLOAD_PATH, "TEXT"));
-        if (!query.exec(alterTableTorrentsQuery))
-            throw RuntimeError(query.lastError().text());
+        if (fromVersion == 1)
+        {
+            const auto alterTableTorrentsQuery = u"ALTER TABLE %1 ADD %2"_qs
+                    .arg(quoted(DB_TABLE_TORRENTS), makeColumnDefinition(DB_COLUMN_DOWNLOAD_PATH, "TEXT"));
+            if (!query.exec(alterTableTorrentsQuery))
+                throw RuntimeError(query.lastError().text());
+        }
+
+        if (fromVersion <= 2)
+        {
+            const auto alterTableTorrentsQuery = u"ALTER TABLE %1 ADD %2"_qs
+                    .arg(quoted(DB_TABLE_TORRENTS), makeColumnDefinition(DB_COLUMN_STOP_CONDITION, "TEXT NOT NULL DEFAULT `None`"));
+            if (!query.exec(alterTableTorrentsQuery))
+                throw RuntimeError(query.lastError().text());
+        }
 
         const QString updateMetaVersionQuery = makeUpdateStatement(DB_TABLE_META, {DB_COLUMN_NAME, DB_COLUMN_VALUE});
         if (!query.prepare(updateMetaVersionQuery))
             throw RuntimeError(query.lastError().text());
 
-        query.bindValue(DB_COLUMN_NAME.placeholder, QString::fromLatin1(META_VERSION));
+        query.bindValue(DB_COLUMN_NAME.placeholder, META_VERSION);
         query.bindValue(DB_COLUMN_VALUE.placeholder, DB_VERSION);
 
         if (!query.exec())
@@ -482,16 +565,17 @@ void BitTorrent::DBResumeDataStorage::updateDBFromVersion1() const
     }
 }
 
-BitTorrent::DBResumeDataStorage::Worker::Worker(const QString &dbPath, const QString &dbConnectionName)
+BitTorrent::DBResumeDataStorage::Worker::Worker(const Path &dbPath, const QString &dbConnectionName, QReadWriteLock &dbLock)
     : m_path {dbPath}
     , m_connectionName {dbConnectionName}
+    , m_dbLock {dbLock}
 {
 }
 
 void BitTorrent::DBResumeDataStorage::Worker::openDatabase() const
 {
-    auto db = QSqlDatabase::addDatabase(QLatin1String("QSQLITE"), m_connectionName);
-    db.setDatabaseName(m_path);
+    auto db = QSqlDatabase::addDatabase(u"QSQLITE"_qs, m_connectionName);
+    db.setDatabaseName(m_path.data());
     if (!db.open())
         throw RuntimeError(db.lastError().text());
 }
@@ -505,7 +589,8 @@ void BitTorrent::DBResumeDataStorage::Worker::store(const TorrentID &id, const L
 {
     // We need to adjust native libtorrent resume data
     lt::add_torrent_params p = resumeData.ltAddTorrentParams;
-    p.save_path = Profile::instance()->toPortablePath(QString::fromStdString(p.save_path)).toStdString();
+    p.save_path = Profile::instance()->toPortablePath(Path(p.save_path))
+            .toString().toStdString();
     if (resumeData.stopped)
     {
         p.flags |= lt::torrent_flags::paused;
@@ -539,6 +624,7 @@ void BitTorrent::DBResumeDataStorage::Worker::store(const TorrentID &id, const L
         DB_COLUMN_HAS_SEED_STATUS,
         DB_COLUMN_OPERATING_MODE,
         DB_COLUMN_STOPPED,
+        DB_COLUMN_STOP_CONDITION,
         DB_COLUMN_RESUMEDATA
     };
 
@@ -589,7 +675,7 @@ void BitTorrent::DBResumeDataStorage::Worker::store(const TorrentID &id, const L
         query.bindValue(DB_COLUMN_NAME.placeholder, resumeData.name);
         query.bindValue(DB_COLUMN_CATEGORY.placeholder, resumeData.category);
         query.bindValue(DB_COLUMN_TAGS.placeholder, (resumeData.tags.isEmpty()
-            ? QVariant(QVariant::String) : resumeData.tags.join(QLatin1String(","))));
+            ? QVariant(QVariant::String) : resumeData.tags.join(u","_qs)));
         query.bindValue(DB_COLUMN_CONTENT_LAYOUT.placeholder, Utils::String::fromEnum(resumeData.contentLayout));
         query.bindValue(DB_COLUMN_RATIO_LIMIT.placeholder, static_cast<int>(resumeData.ratioLimit * 1000));
         query.bindValue(DB_COLUMN_SEEDING_TIME_LIMIT.placeholder, resumeData.seedingTimeLimit);
@@ -597,17 +683,19 @@ void BitTorrent::DBResumeDataStorage::Worker::store(const TorrentID &id, const L
         query.bindValue(DB_COLUMN_HAS_SEED_STATUS.placeholder, resumeData.hasSeedStatus);
         query.bindValue(DB_COLUMN_OPERATING_MODE.placeholder, Utils::String::fromEnum(resumeData.operatingMode));
         query.bindValue(DB_COLUMN_STOPPED.placeholder, resumeData.stopped);
+        query.bindValue(DB_COLUMN_STOP_CONDITION.placeholder, Utils::String::fromEnum(resumeData.stopCondition));
 
         if (!resumeData.useAutoTMM)
         {
-            query.bindValue(DB_COLUMN_TARGET_SAVE_PATH.placeholder, Profile::instance()->toPortablePath(resumeData.savePath));
-            query.bindValue(DB_COLUMN_DOWNLOAD_PATH.placeholder, Profile::instance()->toPortablePath(resumeData.downloadPath));
+            query.bindValue(DB_COLUMN_TARGET_SAVE_PATH.placeholder, Profile::instance()->toPortablePath(resumeData.savePath).data());
+            query.bindValue(DB_COLUMN_DOWNLOAD_PATH.placeholder, Profile::instance()->toPortablePath(resumeData.downloadPath).data());
         }
 
         query.bindValue(DB_COLUMN_RESUMEDATA.placeholder, bencodedResumeData);
         if (!bencodedMetadata.isEmpty())
             query.bindValue(DB_COLUMN_METADATA.placeholder, bencodedMetadata);
 
+        const QWriteLocker locker {&m_dbLock};
         if (!query.exec())
             throw RuntimeError(query.lastError().text());
     }
@@ -620,7 +708,7 @@ void BitTorrent::DBResumeDataStorage::Worker::store(const TorrentID &id, const L
 
 void BitTorrent::DBResumeDataStorage::Worker::remove(const TorrentID &id) const
 {
-    const auto deleteTorrentStatement = QString::fromLatin1("DELETE FROM %1 WHERE %2 = %3;")
+    const auto deleteTorrentStatement = u"DELETE FROM %1 WHERE %2 = %3;"_qs
             .arg(quoted(DB_TABLE_TORRENTS), quoted(DB_COLUMN_TORRENT_ID.name), DB_COLUMN_TORRENT_ID.placeholder);
 
     auto db = QSqlDatabase::database(m_connectionName);
@@ -632,6 +720,8 @@ void BitTorrent::DBResumeDataStorage::Worker::remove(const TorrentID &id) const
             throw RuntimeError(query.lastError().text());
 
         query.bindValue(DB_COLUMN_TORRENT_ID.placeholder, id.toString());
+
+        const QWriteLocker locker {&m_dbLock};
         if (!query.exec())
             throw RuntimeError(query.lastError().text());
     }
@@ -644,7 +734,7 @@ void BitTorrent::DBResumeDataStorage::Worker::remove(const TorrentID &id) const
 
 void BitTorrent::DBResumeDataStorage::Worker::storeQueue(const QVector<TorrentID> &queue) const
 {
-    const auto updateQueuePosStatement = QString::fromLatin1("UPDATE %1 SET %2 = %3 WHERE %4 = %5;")
+    const auto updateQueuePosStatement = u"UPDATE %1 SET %2 = %3 WHERE %4 = %5;"_qs
             .arg(quoted(DB_TABLE_TORRENTS), quoted(DB_COLUMN_QUEUE_POSITION.name), DB_COLUMN_QUEUE_POSITION.placeholder
                  , quoted(DB_COLUMN_TORRENT_ID.name), DB_COLUMN_TORRENT_ID.placeholder);
 
@@ -652,6 +742,8 @@ void BitTorrent::DBResumeDataStorage::Worker::storeQueue(const QVector<TorrentID
 
     try
     {
+        const QWriteLocker locker {&m_dbLock};
+
         if (!db.transaction())
             throw RuntimeError(db.lastError().text());
 
