@@ -30,6 +30,7 @@
 
 #include <QBitArray>
 
+#include "base/bittorrent/ltqbitarray.h"
 #include "base/bittorrent/torrent.h"
 #include "base/net/geoipmanager.h"
 #include "base/unicodestrings.h"
@@ -39,8 +40,8 @@ using namespace BitTorrent;
 
 PeerInfo::PeerInfo(const Torrent *torrent, const lt::peer_info &nativeInfo)
     : m_nativeInfo(nativeInfo)
+    , m_relevance(calcRelevance(torrent))
 {
-    calcRelevance(torrent);
     determineFlags();
 }
 
@@ -180,6 +181,31 @@ QString PeerInfo::client() const
     return QString::fromStdString(m_nativeInfo.client);
 }
 
+QString PeerInfo::peerIdClient() const
+{
+    // when peer ID is not known yet it contains only zero bytes,
+    // do not create string in such case, return empty string instead
+    if (m_nativeInfo.pid.is_all_zeros())
+        return {};
+
+    QString result;
+
+    // interesting part of a typical peer ID is first 8 chars
+    for (int i = 0; i < 8; ++i)
+    {
+        const std::uint8_t c = m_nativeInfo.pid[i];
+
+        // ensure that the peer ID slice consists only of printable ASCII characters,
+        // this should filter out most of the improper IDs
+        if ((c < 32) || (c > 126))
+            return tr("Unknown");
+
+        result += QChar::fromLatin1(c);
+    }
+
+    return result;
+}
+
 qreal PeerInfo::progress() const
 {
     return m_nativeInfo.progress;
@@ -207,47 +233,29 @@ qlonglong PeerInfo::totalDownload() const
 
 QBitArray PeerInfo::pieces() const
 {
-    QBitArray result(m_nativeInfo.pieces.size());
-    for (int i = 0; i < result.size(); ++i)
-    {
-        if (m_nativeInfo.pieces[lt::piece_index_t {i}])
-            result.setBit(i, true);
-    }
-    return result;
+    return LT::toQBitArray(m_nativeInfo.pieces);
 }
 
 QString PeerInfo::connectionType() const
 {
     if (m_nativeInfo.flags & lt::peer_info::utp_socket)
-        return QString::fromUtf8(C_UTP);
+        return C_UTP;
 
     return (m_nativeInfo.connection_type == lt::peer_info::standard_bittorrent)
-        ? QLatin1String {"BT"}
-        : QLatin1String {"Web"};
+        ? u"BT"_qs
+        : u"Web"_qs;
 }
 
-void PeerInfo::calcRelevance(const Torrent *torrent)
+qreal PeerInfo::calcRelevance(const Torrent *torrent) const
 {
     const QBitArray allPieces = torrent->pieces();
+    const int localMissing = allPieces.count(false);
+    if (localMissing <= 0)
+        return 0;
+
     const QBitArray peerPieces = pieces();
-
-    int localMissing = 0;
-    int remoteHaves = 0;
-
-    for (int i = 0; i < allPieces.size(); ++i)
-    {
-        if (!allPieces[i])
-        {
-            ++localMissing;
-            if (peerPieces[i])
-                ++remoteHaves;
-        }
-    }
-
-    if (localMissing == 0)
-        m_relevance = 0.0;
-    else
-        m_relevance = static_cast<qreal>(remoteHaves) / localMissing;
+    const int remoteHaves = (peerPieces & (~allPieces)).count(true);
+    return static_cast<qreal>(remoteHaves) / localMissing;
 }
 
 qreal PeerInfo::relevance() const
@@ -257,123 +265,86 @@ qreal PeerInfo::relevance() const
 
 void PeerInfo::determineFlags()
 {
+    const auto updateFlags = [this](const QChar specifier, const QString &explanation)
+    {
+        m_flags += (specifier + u' ');
+        m_flagsDescription += u"%1 = %2\n"_qs.arg(specifier, explanation);
+    };
+
     if (isInteresting())
     {
-        // d = Your client wants to download, but peer doesn't want to send (interested and choked)
         if (isRemoteChocked())
         {
-            m_flags += "d ";
-            m_flagsDescription += ("d = "
-                + tr("Interested(local) and Choked(peer)") + '\n');
+            // d = Your client wants to download, but peer doesn't want to send (interested and choked)
+            updateFlags(u'd', tr("Interested (local) and choked (peer)"));
         }
         else
         {
             // D = Currently downloading (interested and not choked)
-            m_flags += "D ";
-            m_flagsDescription += ("D = "
-                + tr("interested(local) and unchoked(peer)") + '\n');
+            updateFlags(u'D', tr("Interested (local) and unchoked (peer)"));
         }
     }
 
     if (isRemoteInterested())
     {
-        // u = Peer wants your client to upload, but your client doesn't want to (interested and choked)
         if (isChocked())
         {
-            m_flags += "u ";
-            m_flagsDescription += ("u = "
-                + tr("interested(peer) and choked(local)") + '\n');
+            // u = Peer wants your client to upload, but your client doesn't want to (interested and choked)
+            updateFlags(u'u', tr("Interested (peer) and choked (local)"));
         }
         else
         {
             // U = Currently uploading (interested and not choked)
-            m_flags += "U ";
-            m_flagsDescription += ("U = "
-                + tr("interested(peer) and unchoked(local)") + '\n');
+            updateFlags(u'U', tr("Interested (peer) and unchoked (local)"));
         }
     }
 
-    // O = Optimistic unchoke
-    if (optimisticUnchoke())
-    {
-        m_flags += "O ";
-        m_flagsDescription += ("O = " + tr("optimistic unchoke") + '\n');
-    }
-
-    // S = Peer is snubbed
-    if (isSnubbed())
-    {
-        m_flags += "S ";
-        m_flagsDescription += ("S = " + tr("peer snubbed") + '\n');
-    }
-
-    // I = Peer is an incoming connection
-    if (!isLocalConnection())
-    {
-        m_flags += "I ";
-        m_flagsDescription += ("I = " + tr("incoming connection") + '\n');
-    }
-
     // K = Peer is unchoking your client, but your client is not interested
     if (!isRemoteChocked() && !isInteresting())
-    {
-        m_flags += "K ";
-        m_flagsDescription += ("K = "
-            + tr("not interested(local) and unchoked(peer)") + '\n');
-    }
+        updateFlags(u'K', tr("Not interested (local) and unchoked (peer)"));
 
     // ? = Your client unchoked the peer but the peer is not interested
     if (!isChocked() && !isRemoteInterested())
-    {
-        m_flags += "? ";
-        m_flagsDescription += ("? = "
-            + tr("not interested(peer) and unchoked(local)") + '\n');
-    }
+        updateFlags(u'?', tr("Not interested (peer) and unchoked (local)"));
 
-    // X = Peer was included in peerlists obtained through Peer Exchange (PEX)
-    if (fromPeX())
-    {
-        m_flags += "X ";
-        m_flagsDescription += ("X = " + tr("peer from PEX") + '\n');
-    }
+    // O = Optimistic unchoke
+    if (optimisticUnchoke())
+        updateFlags(u'O', tr("Optimistic unchoke"));
+
+    // S = Peer is snubbed
+    if (isSnubbed())
+        updateFlags(u'S', tr("Peer snubbed"));
+
+    // I = Peer is an incoming connection
+    if (!isLocalConnection())
+        updateFlags(u'I', tr("Incoming connection"));
 
     // H = Peer was obtained through DHT
     if (fromDHT())
-    {
-        m_flags += "H ";
-        m_flagsDescription += ("H = " + tr("peer from DHT") + '\n');
-    }
+        updateFlags(u'H', tr("Peer from DHT"));
 
-    // E = Peer is using Protocol Encryption (all traffic)
-    if (isRC4Encrypted())
-    {
-        m_flags += "E ";
-        m_flagsDescription += ("E = " + tr("encrypted traffic")  + '\n');
-    }
-
-    // e = Peer is using Protocol Encryption (handshake)
-    if (isPlaintextEncrypted())
-    {
-        m_flags += "e ";
-        m_flagsDescription += ("e = " + tr("encrypted handshake") + '\n');
-    }
-
-    // P = Peer is using uTorrent uTP
-    if (useUTPSocket())
-    {
-        m_flags += "P ";
-        m_flagsDescription += ("P = " + QString::fromUtf8(C_UTP) + '\n');
-    }
+    // X = Peer was included in peerlists obtained through Peer Exchange (PEX)
+    if (fromPeX())
+        updateFlags(u'X', tr("Peer from PEX"));
 
     // L = Peer is local
     if (fromLSD())
-    {
-        m_flags += "L ";
-        m_flagsDescription += ("L = " + tr("peer from LSD") + '\n');
-    }
+        updateFlags(u'L', tr("Peer from LSD"));
 
-    m_flags = m_flags.trimmed();
-    m_flagsDescription = m_flagsDescription.trimmed();
+    // E = Peer is using Protocol Encryption (all traffic)
+    if (isRC4Encrypted())
+        updateFlags(u'E', tr("Encrypted traffic"));
+
+    // e = Peer is using Protocol Encryption (handshake)
+    if (isPlaintextEncrypted())
+        updateFlags(u'e', tr("Encrypted handshake"));
+
+    // P = Peer is using uTorrent uTP
+    if (useUTPSocket())
+        updateFlags(u'P', C_UTP);
+
+    m_flags.chop(1);
+    m_flagsDescription.chop(1);
 }
 
 QString PeerInfo::flags() const
