@@ -53,23 +53,23 @@ namespace
     // constants
     const int PEER_ID_SIZE = 20;
 
-    const char ANNOUNCE_REQUEST_PATH[] = "/announce";
+    const QString ANNOUNCE_REQUEST_PATH = u"/announce"_qs;
 
-    const char ANNOUNCE_REQUEST_COMPACT[] = "compact";
-    const char ANNOUNCE_REQUEST_INFO_HASH[] = "info_hash";
-    const char ANNOUNCE_REQUEST_IP[] = "ip";
-    const char ANNOUNCE_REQUEST_LEFT[] = "left";
-    const char ANNOUNCE_REQUEST_NO_PEER_ID[] = "no_peer_id";
-    const char ANNOUNCE_REQUEST_NUM_WANT[] = "numwant";
-    const char ANNOUNCE_REQUEST_PEER_ID[] = "peer_id";
-    const char ANNOUNCE_REQUEST_PORT[] = "port";
+    const QString ANNOUNCE_REQUEST_COMPACT = u"compact"_qs;
+    const QString ANNOUNCE_REQUEST_INFO_HASH = u"info_hash"_qs;
+    const QString ANNOUNCE_REQUEST_IP = u"ip"_qs;
+    const QString ANNOUNCE_REQUEST_LEFT = u"left"_qs;
+    const QString ANNOUNCE_REQUEST_NO_PEER_ID = u"no_peer_id"_qs;
+    const QString ANNOUNCE_REQUEST_NUM_WANT = u"numwant"_qs;
+    const QString ANNOUNCE_REQUEST_PEER_ID = u"peer_id"_qs;
+    const QString ANNOUNCE_REQUEST_PORT = u"port"_qs;
 
-    const char ANNOUNCE_REQUEST_EVENT[] = "event";
-    const char ANNOUNCE_REQUEST_EVENT_COMPLETED[] = "completed";
-    const char ANNOUNCE_REQUEST_EVENT_EMPTY[] = "empty";
-    const char ANNOUNCE_REQUEST_EVENT_STARTED[] = "started";
-    const char ANNOUNCE_REQUEST_EVENT_STOPPED[] = "stopped";
-    const char ANNOUNCE_REQUEST_EVENT_PAUSED[] = "paused";
+    const QString ANNOUNCE_REQUEST_EVENT = u"event"_qs;
+    const QString ANNOUNCE_REQUEST_EVENT_COMPLETED = u"completed"_qs;
+    const QString ANNOUNCE_REQUEST_EVENT_EMPTY = u"empty"_qs;
+    const QString ANNOUNCE_REQUEST_EVENT_STARTED = u"started"_qs;
+    const QString ANNOUNCE_REQUEST_EVENT_STOPPED = u"stopped"_qs;
+    const QString ANNOUNCE_REQUEST_EVENT_PAUSED = u"paused"_qs;
 
     const char ANNOUNCE_RESPONSE_COMPLETE[] = "complete";
     const char ANNOUNCE_RESPONSE_EXTERNAL_IP[] = "external ip";
@@ -140,7 +140,11 @@ namespace BitTorrent
         return !(left == right);
     }
 
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+    std::size_t qHash(const Peer &key, const std::size_t seed)
+#else
     uint qHash(const Peer &key, const uint seed)
+#endif
     {
         return qHash(key.uniqueID(), seed);
     }
@@ -199,12 +203,12 @@ Tracker::Tracker(QObject *parent)
 
 bool Tracker::start()
 {
-    const QHostAddress ip = QHostAddress::Any;
     const int port = Preferences::instance()->getTrackerPort();
 
     if (m_server->isListening())
     {
-        if (m_server->serverPort() == port)
+        if (const int oldPort = m_server->serverPort()
+            ; oldPort == port)
         {
             // Already listening on the right port, just return
             return true;
@@ -214,9 +218,9 @@ bool Tracker::start()
         m_server->close();
     }
 
-    // Listen on the predefined port
+    // Listen on port
+    const QHostAddress ip = QHostAddress::Any;
     const bool listenSuccess = m_server->listen(ip, port);
-
     if (listenSuccess)
     {
         LogMsg(tr("Embedded Tracker: Now listening on IP: %1, port: %2")
@@ -293,32 +297,32 @@ void Tracker::processAnnounceRequest()
     // 1. info_hash
     const auto infoHashIter = queryParams.find(ANNOUNCE_REQUEST_INFO_HASH);
     if (infoHashIter == queryParams.end())
-        throw TrackerError("Missing \"info_hash\" parameter");
+        throw TrackerError(u"Missing \"info_hash\" parameter"_qs);
 
-    const auto torrentID = TorrentID::fromString(infoHashIter->toHex());
+    const auto torrentID = TorrentID::fromString(QString::fromLatin1(infoHashIter->toHex()));
     if (!torrentID.isValid())
-        throw TrackerError("Invalid \"info_hash\" parameter");
+        throw TrackerError(u"Invalid \"info_hash\" parameter"_qs);
 
     announceReq.torrentID = torrentID;
 
     // 2. peer_id
     const auto peerIdIter = queryParams.find(ANNOUNCE_REQUEST_PEER_ID);
     if (peerIdIter == queryParams.end())
-        throw TrackerError("Missing \"peer_id\" parameter");
+        throw TrackerError(u"Missing \"peer_id\" parameter"_qs);
 
     if (peerIdIter->size() > PEER_ID_SIZE)
-        throw TrackerError("Invalid \"peer_id\" parameter");
+        throw TrackerError(u"Invalid \"peer_id\" parameter"_qs);
 
     announceReq.peer.peerId = *peerIdIter;
 
     // 3. port
     const auto portIter = queryParams.find(ANNOUNCE_REQUEST_PORT);
     if (portIter == queryParams.end())
-        throw TrackerError("Missing \"port\" parameter");
+        throw TrackerError(u"Missing \"port\" parameter"_qs);
 
     const ushort portNum = portIter->toUShort();
     if (portNum == 0)
-        throw TrackerError("Invalid \"port\" parameter");
+        throw TrackerError(u"Invalid \"port\" parameter"_qs);
 
     announceReq.peer.port = portNum;
 
@@ -328,7 +332,7 @@ void Tracker::processAnnounceRequest()
     {
         const int num = numWantIter->toInt();
         if (num < 0)
-            throw TrackerError("Invalid \"numwant\" parameter");
+            throw TrackerError(u"Invalid \"numwant\" parameter"_qs);
         announceReq.numwant = num;
     }
 
@@ -355,7 +359,7 @@ void Tracker::processAnnounceRequest()
         : announceReq.socketAddress.toString().toLatin1().constData(),
 
     // 10. event
-    announceReq.event = queryParams.value(ANNOUNCE_REQUEST_EVENT);
+    announceReq.event = QString::fromLatin1(queryParams.value(ANNOUNCE_REQUEST_EVENT));
 
     if (announceReq.event.isEmpty()
         || (announceReq.event == ANNOUNCE_REQUEST_EVENT_EMPTY)
@@ -373,7 +377,7 @@ void Tracker::processAnnounceRequest()
     }
     else
     {
-        throw TrackerError("Invalid \"event\" parameter");
+        throw TrackerError(u"Invalid \"event\" parameter"_qs);
     }
 
     prepareAnnounceResponse(announceReq);
