@@ -34,6 +34,7 @@
 
 #include <libtorrent/entry.hpp>
 
+#include <QtGlobal>
 #include <QHash>
 #include <QObject>
 #include <QSet>
@@ -64,7 +65,11 @@ namespace BitTorrent
 
     bool operator==(const Peer &left, const Peer &right);
     bool operator!=(const Peer &left, const Peer &right);
-    uint qHash(const Peer &key, uint seed);
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+    std::size_t qHash(const Peer &key, std::size_t seed = 0);
+#else
+    uint qHash(const Peer &key, uint seed = 0);
+#endif
 
     // *Basic* Bittorrent tracker implementation
     // [BEP-3] The BitTorrent Protocol Specification
@@ -98,7 +103,7 @@ namespace BitTorrent
         void unregisterPeer(const TrackerAnnounceRequest &announceReq);
         void prepareAnnounceResponse(const TrackerAnnounceRequest &announceReq);
 
-        Http::Server *m_server;
+        Http::Server *m_server = nullptr;
         Http::Request m_request;
         Http::Environment m_env;
 
