@@ -31,6 +31,8 @@
 
 #include <QObject>
 
+#include "base/pathfwd.h"
+
 class QString;
 
 class IconProvider : public QObject
@@ -42,7 +44,7 @@ public:
     static void freeInstance();
     static IconProvider *instance();
 
-    virtual QString getIconPath(const QString &iconId) const;
+    virtual Path getIconPath(const QString &iconId) const;
 
 protected:
     explicit IconProvider(QObject *parent = nullptr);
