@@ -26,8 +26,7 @@
  * exception statement from your version.
  */
 
-#ifndef DNSUPDATER_H
-#define DNSUPDATER_H
+#pragma once
 
 #include <QDateTime>
 #include <QHostAddress>
@@ -44,12 +43,13 @@ namespace Net
     class DNSUpdater : public QObject
     {
         Q_OBJECT
+        Q_DISABLE_COPY_MOVE(DNSUpdater)
 
     public:
         explicit DNSUpdater(QObject *parent = nullptr);
         ~DNSUpdater();
 
-        static QUrl getRegistrationUrl(int service);
+        static QUrl getRegistrationUrl(DNS::Service service);
 
     public slots:
         void updateCredentials();
@@ -68,8 +68,6 @@ namespace Net
             FATAL
         };
 
-        static const int IP_CHECK_INTERVAL_MS = 1800000; // 30 min
-
         QString getUpdateUrl() const;
         void processIPUpdateReply(const QString &reply);
 
@@ -84,5 +82,3 @@ namespace Net
         QString m_password;
     };
 }
-
-#endif // DNSUPDATER_H
