@@ -124,7 +124,7 @@ namespace
         // Spoof HTTP Referer to allow adding torrent link from Torcache/KickAssTorrents
         request.setRawHeader("Referer", request.url().toEncoded().data());
 #ifdef QT_NO_COMPRESS
-        // The macro "QT_NO_COMPRESS" defined in QT will disable the zlib releated features
+        // The macro "QT_NO_COMPRESS" defined in QT will disable the zlib related features
         // and reply data auto-decompression in QT will also be disabled. But we can support
         // gzip encoding and manually decompress the reply data.
         request.setRawHeader("Accept-Encoding", "gzip");
@@ -230,7 +230,7 @@ bool Net::DownloadManager::hasSupportedScheme(const QString &url)
     const QStringList schemes = instance()->m_networkManager.supportedSchemes();
     return std::any_of(schemes.cbegin(), schemes.cend(), [&url](const QString &scheme)
     {
-        return url.startsWith((scheme + QLatin1Char(':')), Qt::CaseInsensitive);
+        return url.startsWith((scheme + u':'), Qt::CaseInsensitive);
     });
 }
 
@@ -297,7 +297,7 @@ void Net::DownloadManager::ignoreSslErrors(QNetworkReply *reply, const QList<QSs
     QStringList errorList;
     for (const QSslError &error : errors)
         errorList += error.errorString();
-    LogMsg(tr("Ignoring SSL error, URL: \"%1\", errors: \"%2\"").arg(reply->url().toString(), errorList.join(". ")), Log::WARNING);
+    LogMsg(tr("Ignoring SSL error, URL: \"%1\", errors: \"%2\"").arg(reply->url().toString(), errorList.join(u". ")), Log::WARNING);
 
     // Ignore all SSL errors
     reply->ignoreSslErrors();
@@ -352,12 +352,12 @@ Net::DownloadRequest &Net::DownloadRequest::saveToFile(const bool value)
     return *this;
 }
 
-QString Net::DownloadRequest::destFileName() const
+Path Net::DownloadRequest::destFileName() const
 {
     return m_destFileName;
 }
 
-Net::DownloadRequest &Net::DownloadRequest::destFileName(const QString &value)
+Net::DownloadRequest &Net::DownloadRequest::destFileName(const Path &value)
 {
     m_destFileName = value;
     return *this;
@@ -368,10 +368,17 @@ Net::ServiceID Net::ServiceID::fromURL(const QUrl &url)
     return {url.host(), url.port(80)};
 }
 
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+std::size_t Net::qHash(const ServiceID &serviceID, const std::size_t seed)
+{
+    return qHashMulti(seed, serviceID.hostName, serviceID.port);
+}
+#else
 uint Net::qHash(const ServiceID &serviceID, const uint seed)
 {
     return ::qHash(serviceID.hostName, seed) ^ ::qHash(serviceID.port);
 }
+#endif
 
 bool Net::operator==(const ServiceID &lhs, const ServiceID &rhs)
 {
