@@ -30,34 +30,21 @@
 
 #include "rss_article.h"
 
-#include <QJsonObject>
 #include <QVariant>
 
+#include "base/global.h"
 #include "rss_feed.h"
 
 using namespace RSS;
 
-namespace
-{
-    QVariantHash articleDataFromJSON(const QJsonObject &jsonObj)
-    {
-        auto varHash = jsonObj.toVariantHash();
-        // JSON object store DateTime as string so we need to convert it
-        varHash[Article::KeyDate] =
-                QDateTime::fromString(jsonObj.value(Article::KeyDate).toString(), Qt::RFC2822Date);
-
-        return varHash;
-    }
-}
-
-const QString Article::KeyId(QStringLiteral("id"));
-const QString Article::KeyDate(QStringLiteral("date"));
-const QString Article::KeyTitle(QStringLiteral("title"));
-const QString Article::KeyAuthor(QStringLiteral("author"));
-const QString Article::KeyDescription(QStringLiteral("description"));
-const QString Article::KeyTorrentURL(QStringLiteral("torrentURL"));
-const QString Article::KeyLink(QStringLiteral("link"));
-const QString Article::KeyIsRead(QStringLiteral("isRead"));
+const QString Article::KeyId = u"id"_qs;
+const QString Article::KeyDate = u"date"_qs;
+const QString Article::KeyTitle = u"title"_qs;
+const QString Article::KeyAuthor = u"author"_qs;
+const QString Article::KeyDescription = u"description"_qs;
+const QString Article::KeyTorrentURL = u"torrentURL"_qs;
+const QString Article::KeyLink = u"link"_qs;
+const QString Article::KeyIsRead = u"isRead"_qs;
 
 Article::Article(Feed *feed, const QVariantHash &varHash)
     : QObject(feed)
@@ -74,11 +61,6 @@ Article::Article(Feed *feed, const QVariantHash &varHash)
 {
 }
 
-Article::Article(Feed *feed, const QJsonObject &jsonObj)
-    : Article(feed, articleDataFromJSON(jsonObj))
-{
-}
-
 QString Article::guid() const
 {
     return m_guid;
@@ -134,15 +116,6 @@ void Article::markAsRead()
     }
 }
 
-QJsonObject Article::toJsonObject() const
-{
-    auto jsonObj = QJsonObject::fromVariantHash(m_data);
-    // JSON object doesn't support DateTime so we need to convert it
-    jsonObj[KeyDate] = m_date.toString(Qt::RFC2822Date);
-
-    return jsonObj;
-}
-
 bool Article::articleDateRecentThan(const Article *article, const QDateTime &date)
 {
     return article->date() > date;
