@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2015, 2017  Vladimir Golovnev <glassez@yandex.ru>
+ * Copyright (C) 2015-2022  Vladimir Golovnev <glassez@yandex.ru>
  * Copyright (C) 2010  Christophe Dumez <chris@qbittorrent.org>
  * Copyright (C) 2010  Arnaud Demaiziere <arnaud@qbittorrent.org>
  *
@@ -34,12 +34,12 @@
 #include <utility>
 #include <vector>
 
-#include <QDir>
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QJsonValue>
 #include <QUrl>
+#include <QVector>
 
 #include "base/asyncfilestorage.h"
 #include "base/global.h"
@@ -47,17 +47,18 @@
 #include "base/net/downloadmanager.h"
 #include "base/profile.h"
 #include "base/utils/fs.h"
+#include "feed_serializer.h"
 #include "rss_article.h"
 #include "rss_parser.h"
 #include "rss_session.h"
 
-const QString KEY_UID(QStringLiteral("uid"));
-const QString KEY_URL(QStringLiteral("url"));
-const QString KEY_TITLE(QStringLiteral("title"));
-const QString KEY_LASTBUILDDATE(QStringLiteral("lastBuildDate"));
-const QString KEY_ISLOADING(QStringLiteral("isLoading"));
-const QString KEY_HASERROR(QStringLiteral("hasError"));
-const QString KEY_ARTICLES(QStringLiteral("articles"));
+const QString KEY_UID = u"uid"_qs;
+const QString KEY_URL = u"url"_qs;
+const QString KEY_TITLE = u"title"_qs;
+const QString KEY_LASTBUILDDATE = u"lastBuildDate"_qs;
+const QString KEY_ISLOADING = u"isLoading"_qs;
+const QString KEY_HASERROR = u"hasError"_qs;
+const QString KEY_ARTICLES = u"articles"_qs;
 
 using namespace RSS;
 
@@ -68,17 +69,21 @@ Feed::Feed(const QUuid &uid, const QString &url, const QString &path, Session *s
     , m_url(url)
 {
     const auto uidHex = QString::fromLatin1(m_uid.toRfc4122().toHex());
-    m_dataFileName = uidHex + QLatin1String(".json");
+    m_dataFileName = Path(uidHex + u".json");
 
     // Move to new file naming scheme (since v4.1.2)
-    const QString legacyFilename
-    {Utils::Fs::toValidFileSystemName(m_url, false, QLatin1String("_"))
-                + QLatin1String(".json")};
-    const QDir storageDir {m_session->dataFileStorage()->storageDir()};
-    if (!QFile::exists(storageDir.absoluteFilePath(m_dataFileName)))
-        QFile::rename(storageDir.absoluteFilePath(legacyFilename), storageDir.absoluteFilePath(m_dataFileName));
+    const QString legacyFilename = Utils::Fs::toValidFileName(m_url, u"_"_qs) + u".json";
+    const Path storageDir = m_session->dataFileStorage()->storageDir();
+    const Path dataFilePath = storageDir / m_dataFileName;
+    if (!dataFilePath.exists())
+        Utils::Fs::renameFile((storageDir / Path(legacyFilename)), dataFilePath);
 
-    m_iconPath = Utils::Fs::toUniformPath(storageDir.absoluteFilePath(uidHex + QLatin1String(".ico")));
+    m_iconPath = storageDir / Path(uidHex + u".ico");
+
+    m_serializer = new Private::FeedSerializer;
+    m_serializer->moveToThread(m_session->workingThread());
+    connect(this, &Feed::destroyed, m_serializer, &Private::FeedSerializer::deleteLater);
+    connect(m_serializer, &Private::FeedSerializer::loadingFinished, this, &Feed::handleArticleLoadFinished);
 
     m_parser = new Private::Parser(m_lastBuildDate);
     m_parser->moveToThread(m_session->workingThread());
@@ -99,6 +104,7 @@ Feed::Feed(const QUuid &uid, const QString &url, const QString &path, Session *s
 
 Feed::~Feed()
 {
+    store();
     emit aboutToBeDestroyed(this);
 }
 
@@ -131,6 +137,12 @@ void Feed::markAsRead()
 
 void Feed::refresh()
 {
+    if (!m_isInitialized)
+    {
+        m_pendingRefresh = true;
+        return;
+    }
+
     if (m_downloadHandler)
         m_downloadHandler->cancel();
 
@@ -139,7 +151,7 @@ void Feed::refresh()
     m_downloadHandler = Net::DownloadManager::instance()->download(m_url);
     connect(m_downloadHandler, &Net::DownloadHandler::finished, this, &Feed::handleDownloadFinished);
 
-    if (!QFile::exists(m_iconPath))
+    if (!m_iconPath.exists())
         downloadIcon();
 
     m_isLoading = true;
@@ -163,7 +175,7 @@ QString Feed::title() const
 
 bool Feed::isLoading() const
 {
-    return m_isLoading;
+    return m_isLoading || !m_isInitialized;
 }
 
 QString Feed::lastBuildDate() const
@@ -210,7 +222,10 @@ void Feed::handleDownloadFinished(const Net::DownloadResult &result)
         LogMsg(tr("RSS feed at '%1' is successfully downloaded. Starting to parse it.")
                 .arg(result.url));
         // Parse the download RSS
-        m_parser->parse(result.data);
+        QMetaObject::invokeMethod(m_parser, [this, data = result.data]()
+        {
+            m_parser->parse(data);
+        });
     }
     else
     {
@@ -262,100 +277,34 @@ void Feed::handleParsingFinished(const RSS::Private::ParsingResult &result)
 
 void Feed::load()
 {
-    QFile file(m_session->dataFileStorage()->storageDir().absoluteFilePath(m_dataFileName));
-
-    if (!file.exists())
+    QMetaObject::invokeMethod(m_serializer
+            , [serializer = m_serializer, url = m_url
+                , path = (m_session->dataFileStorage()->storageDir() / m_dataFileName)]
     {
-        loadArticlesLegacy();
-        m_dirty = true;
-        store(); // convert to new format
-    }
-    else if (file.open(QFile::ReadOnly))
-    {
-        loadArticles(file.readAll());
-        file.close();
-    }
-    else
-    {
-        LogMsg(tr("Couldn't read RSS Session data from %1. Error: %2")
-               .arg(m_dataFileName, file.errorString())
-               , Log::WARNING);
-    }
-}
-
-void Feed::loadArticles(const QByteArray &data)
-{
-    QJsonParseError jsonError;
-    const QJsonDocument jsonDoc = QJsonDocument::fromJson(data, &jsonError);
-    if (jsonError.error != QJsonParseError::NoError)
-    {
-        LogMsg(tr("Couldn't parse RSS Session data. Error: %1").arg(jsonError.errorString())
-               , Log::WARNING);
-        return;
-    }
-
-    if (!jsonDoc.isArray())
-    {
-        LogMsg(tr("Couldn't load RSS Session data. Invalid data format."), Log::WARNING);
-        return;
-    }
-
-    const QJsonArray jsonArr = jsonDoc.array();
-    int i = -1;
-    for (const QJsonValue &jsonVal : jsonArr)
-    {
-        ++i;
-        if (!jsonVal.isObject())
-        {
-            LogMsg(tr("Couldn't load RSS article '%1#%2'. Invalid data format.").arg(m_url).arg(i)
-                   , Log::WARNING);
-            continue;
-        }
-
-        try
-        {
-            auto article = new Article(this, jsonVal.toObject());
-            if (!addArticle(article))
-                delete article;
-        }
-        catch (const RuntimeError &) {}
-    }
-}
-
-void Feed::loadArticlesLegacy()
-{
-    const SettingsPtr qBTRSSFeeds = Profile::instance()->applicationSettings(QStringLiteral("qBittorrent-rss-feeds"));
-    const QVariantHash allOldItems = qBTRSSFeeds->value("old_items").toHash();
-
-    for (const QVariant &var : asConst(allOldItems.value(m_url).toList()))
-    {
-        auto hash = var.toHash();
-        // update legacy keys
-        hash[Article::KeyLink] = hash.take(QLatin1String("news_link"));
-        hash[Article::KeyTorrentURL] = hash.take(QLatin1String("torrent_url"));
-        hash[Article::KeyIsRead] = hash.take(QLatin1String("read"));
-        try
-        {
-            auto article = new Article(this, hash);
-            if (!addArticle(article))
-                delete article;
-        }
-        catch (const RuntimeError &) {}
-    }
+        serializer->load(path, url);
+    });
 }
 
 void Feed::store()
 {
-    if (!m_dirty) return;
+    if (!m_dirty)
+        return;
 
     m_dirty = false;
     m_savingTimer.stop();
 
-    QJsonArray jsonArr;
-    for (Article *article :asConst(m_articles))
-        jsonArr << article->toJsonObject();
+    QVector<QVariantHash> articlesData;
+    articlesData.reserve(m_articles.size());
 
-    m_session->dataFileStorage()->store(m_dataFileName, QJsonDocument(jsonArr).toJson());
+    for (Article *article :asConst(m_articles))
+        articlesData.push_back(article->data());
+
+    QMetaObject::invokeMethod(m_serializer
+            , [articlesData, serializer = m_serializer
+                , path = (m_session->dataFileStorage()->storageDir() / m_dataFileName)]
+    {
+        serializer->store(path, articlesData);
+    });
 }
 
 void Feed::storeDeferred()
@@ -364,18 +313,18 @@ void Feed::storeDeferred()
         m_savingTimer.start(5 * 1000, this);
 }
 
-bool Feed::addArticle(Article *article)
+bool Feed::addArticle(const QVariantHash &articleData)
 {
-    Q_ASSERT(article);
-    Q_ASSERT(!m_articles.contains(article->guid()));
+    Q_ASSERT(!m_articles.contains(articleData.value(Article::KeyId).toString()));
 
     // Insertion sort
     const int maxArticles = m_session->maxArticlesPerFeed();
     const auto lowerBound = std::lower_bound(m_articlesByDate.begin(), m_articlesByDate.end()
-                                       , article->date(), Article::articleDateRecentThan);
+                                       , articleData.value(Article::KeyDate).toDateTime(), Article::articleDateRecentThan);
     if ((lowerBound - m_articlesByDate.begin()) >= maxArticles)
         return false; // we reach max articles
 
+    auto *article = new Article(this, articleData);
     m_articles[article->guid()] = article;
     m_articlesByDate.insert(lowerBound, article);
     if (!article->isRead())
@@ -426,7 +375,7 @@ void Feed::downloadIcon()
     // Download the RSS Feed icon
     // XXX: This works for most sites but it is not perfect
     const QUrl url(m_url);
-    const auto iconUrl = QString::fromLatin1("%1://%2/favicon.ico").arg(url.scheme(), url.host());
+    const auto iconUrl = u"%1://%2/favicon.ico"_qs.arg(url.scheme(), url.host());
     Net::DownloadManager::instance()->download(
             Net::DownloadRequest(iconUrl).saveToFile(true).destFileName(m_iconPath)
                 , this, &Feed::handleIconDownloadFinished);
@@ -492,7 +441,7 @@ int Feed::updateArticles(const QList<QVariantHash> &loadedArticles)
     {
         if (a.second)
         {
-            addArticle(new Article {this, *a.second});
+            addArticle(*a.second);
             ++newArticlesCount;
         }
     });
@@ -500,7 +449,7 @@ int Feed::updateArticles(const QList<QVariantHash> &loadedArticles)
     return newArticlesCount;
 }
 
-QString Feed::iconPath() const
+Path Feed::iconPath() const
 {
     return m_iconPath;
 }
@@ -520,7 +469,12 @@ QJsonValue Feed::toJsonValue(const bool withData) const
 
         QJsonArray jsonArr;
         for (Article *article : asConst(m_articles))
-            jsonArr << article->toJsonObject();
+        {
+            auto articleObj = QJsonObject::fromVariantHash(article->data());
+            // JSON object doesn't support DateTime so we need to convert it
+            articleObj[Article::KeyDate] = article->date().toString(Qt::RFC2822Date);
+            jsonArr.append(articleObj);
+        }
         jsonObj.insert(KEY_ARTICLES, jsonArr);
     }
 
@@ -547,10 +501,56 @@ void Feed::handleArticleRead(Article *article)
     storeDeferred();
 }
 
+void Feed::handleArticleLoadFinished(QVector<QVariantHash> articles)
+{
+    Q_ASSERT(m_articles.isEmpty());
+    Q_ASSERT(m_unreadCount == 0);
+
+    const int maxArticles = m_session->maxArticlesPerFeed();
+    if (articles.size() > maxArticles)
+        articles.resize(maxArticles);
+
+    m_articles.reserve(articles.size());
+    m_articlesByDate.reserve(articles.size());
+
+    for (const QVariantHash &articleData : articles)
+    {
+        const auto articleID = articleData.value(Article::KeyId).toString();
+        // TODO: use [[unlikely]] in C++20
+        if (Q_UNLIKELY(m_articles.contains(articleID)))
+            continue;
+
+        auto *article = new Article(this, articleData);
+        m_articles[articleID] = article;
+        m_articlesByDate.append(article);
+        if (!article->isRead())
+        {
+            ++m_unreadCount;
+            connect(article, &Article::read, this, &Feed::handleArticleRead);
+        }
+
+        emit newArticle(article);
+    }
+
+    if (m_unreadCount > 0)
+        emit unreadCountChanged(this);
+
+    m_isInitialized = true;
+    emit stateChanged(this);
+
+    if (m_pendingRefresh)
+    {
+        m_pendingRefresh = false;
+        refresh();
+    }
+}
+
 void Feed::cleanup()
 {
-    Utils::Fs::forceRemove(m_session->dataFileStorage()->storageDir().absoluteFilePath(m_dataFileName));
-    Utils::Fs::forceRemove(m_iconPath);
+    m_dirty = false;
+    m_savingTimer.stop();
+    Utils::Fs::removeFile(m_session->dataFileStorage()->storageDir() / m_dataFileName);
+    Utils::Fs::removeFile(m_iconPath);
 }
 
 void Feed::timerEvent(QTimerEvent *event)
