@@ -75,6 +75,7 @@
 
 #include "base/3rdparty/expected.hpp"
 #include "base/settingvalue.h"
+#include "base/utils/thread.h"
 
 class QThread;
 
@@ -158,9 +159,9 @@ namespace RSS
         CachedSettingValue<bool> m_storeProcessingEnabled;
         CachedSettingValue<int> m_storeRefreshInterval;
         CachedSettingValue<int> m_storeMaxArticlesPerFeed;
-        QThread *m_workingThread;
-        AsyncFileStorage *m_confFileStorage;
-        AsyncFileStorage *m_dataFileStorage;
+        Utils::Thread::UniquePtr m_workingThread;
+        AsyncFileStorage *m_confFileStorage = nullptr;
+        AsyncFileStorage *m_dataFileStorage = nullptr;
         QTimer m_refreshTimer;
         QHash<QString, Item *> m_itemsByPath;
         QHash<QUuid, Feed *> m_feedsByUID;
