@@ -31,9 +31,9 @@
 #include "bittorrent/infohash.h"
 #include "bittorrent/torrent.h"
 
-const QString TorrentFilter::AnyCategory;
-const InfoHashSet TorrentFilter::AnyHash {{}};
-const QString TorrentFilter::AnyTag;
+const std::optional<QString> TorrentFilter::AnyCategory;
+const std::optional<TorrentIDSet> TorrentFilter::AnyID;
+const std::optional<QString> TorrentFilter::AnyTag;
 
 const TorrentFilter TorrentFilter::DownloadingTorrent(TorrentFilter::Downloading);
 const TorrentFilter TorrentFilter::SeedingTorrent(TorrentFilter::Seeding);
@@ -45,23 +45,27 @@ const TorrentFilter TorrentFilter::InactiveTorrent(TorrentFilter::Inactive);
 const TorrentFilter TorrentFilter::StalledTorrent(TorrentFilter::Stalled);
 const TorrentFilter TorrentFilter::StalledUploadingTorrent(TorrentFilter::StalledUploading);
 const TorrentFilter TorrentFilter::StalledDownloadingTorrent(TorrentFilter::StalledDownloading);
+const TorrentFilter TorrentFilter::CheckingTorrent(TorrentFilter::Checking);
+const TorrentFilter TorrentFilter::MovingTorrent(TorrentFilter::Moving);
 const TorrentFilter TorrentFilter::ErroredTorrent(TorrentFilter::Errored);
 
 using BitTorrent::Torrent;
 
-TorrentFilter::TorrentFilter(const Type type, const InfoHashSet &hashSet, const QString &category, const QString &tag)
+TorrentFilter::TorrentFilter(const Type type, const std::optional<TorrentIDSet> &idSet
+                             , const std::optional<QString> &category, const std::optional<QString> &tag)
     : m_type(type)
     , m_category(category)
     , m_tag(tag)
-    , m_hashSet(hashSet)
+    , m_idSet(idSet)
 {
 }
 
-TorrentFilter::TorrentFilter(const QString &filter, const InfoHashSet &hashSet, const QString &category, const QString &tag)
+TorrentFilter::TorrentFilter(const QString &filter, const std::optional<TorrentIDSet> &idSet
+                             , const std::optional<QString> &category, const std::optional<QString> &tag)
     : m_type(All)
     , m_category(category)
     , m_tag(tag)
-    , m_hashSet(hashSet)
+    , m_idSet(idSet)
 {
     setTypeByName(filter);
 }
@@ -81,50 +85,51 @@ bool TorrentFilter::setTypeByName(const QString &filter)
 {
     Type type = All;
 
-    if (filter == "downloading")
+    if (filter == u"downloading")
         type = Downloading;
-    else if (filter == "seeding")
+    else if (filter == u"seeding")
         type = Seeding;
-    else if (filter == "completed")
+    else if (filter == u"completed")
         type = Completed;
-    else if (filter == "paused")
+    else if (filter == u"paused")
         type = Paused;
-    else if (filter == "resumed")
+    else if (filter == u"resumed")
         type = Resumed;
-    else if (filter == "active")
+    else if (filter == u"active")
         type = Active;
-    else if (filter == "inactive")
+    else if (filter == u"inactive")
         type = Inactive;
-    else if (filter == "stalled")
+    else if (filter == u"stalled")
         type = Stalled;
-    else if (filter == "stalled_uploading")
+    else if (filter == u"stalled_uploading")
         type = StalledUploading;
-    else if (filter == "stalled_downloading")
+    else if (filter == u"stalled_downloading")
         type = StalledDownloading;
-    else if (filter == "errored")
+    else if (filter == u"checking")
+        type = Checking;
+    else if (filter == u"moving")
+        type = Moving;
+    else if (filter == u"errored")
         type = Errored;
 
     return setType(type);
 }
 
-bool TorrentFilter::setHashSet(const InfoHashSet &hashSet)
+bool TorrentFilter::setTorrentIDSet(const std::optional<TorrentIDSet> &idSet)
 {
-    if (m_hashSet != hashSet)
+    if (m_idSet != idSet)
     {
-        m_hashSet = hashSet;
+        m_idSet = idSet;
         return true;
     }
 
     return false;
 }
 
-bool TorrentFilter::setCategory(const QString &category)
+bool TorrentFilter::setCategory(const std::optional<QString> &category)
 {
-    // QString::operator==() doesn't distinguish between empty and null strings.
-    if ((m_category != category)
-            || (m_category.isNull() && !category.isNull())
-            || (!m_category.isNull() && category.isNull()))
-            {
+    if (m_category != category)
+    {
         m_category = category;
         return true;
     }
@@ -132,13 +137,10 @@ bool TorrentFilter::setCategory(const QString &category)
     return false;
 }
 
-bool TorrentFilter::setTag(const QString &tag)
+bool TorrentFilter::setTag(const std::optional<QString> &tag)
 {
-    // QString::operator==() doesn't distinguish between empty and null strings.
-    if ((m_tag != tag)
-        || (m_tag.isNull() && !tag.isNull())
-        || (!m_tag.isNull() && tag.isNull()))
-        {
+    if (m_tag != tag)
+    {
         m_tag = tag;
         return true;
     }
@@ -158,6 +160,7 @@ bool TorrentFilter::matchState(const BitTorrent::Torrent *const torrent) const
     switch (m_type)
     {
     case All:
+    default:
         return true;
     case Downloading:
         return torrent->isDownloading();
@@ -180,32 +183,41 @@ bool TorrentFilter::matchState(const BitTorrent::Torrent *const torrent) const
         return torrent->state() ==  BitTorrent::TorrentState::StalledUploading;
     case StalledDownloading:
         return torrent->state() ==  BitTorrent::TorrentState::StalledDownloading;
+    case Checking:
+        return (torrent->state() == BitTorrent::TorrentState::CheckingUploading)
+                || (torrent->state() == BitTorrent::TorrentState::CheckingDownloading)
+                || (torrent->state() == BitTorrent::TorrentState::CheckingResumeData);
+    case Moving:
+        return torrent->isMoving();
     case Errored:
         return torrent->isErrored();
-    default: // All
-        return true;
     }
 }
 
 bool TorrentFilter::matchHash(const BitTorrent::Torrent *const torrent) const
 {
-    if (m_hashSet == AnyHash) return true;
+    if (!m_idSet)
+        return true;
 
-    return m_hashSet.contains(torrent->hash());
+    return m_idSet->contains(torrent->id());
 }
 
 bool TorrentFilter::matchCategory(const BitTorrent::Torrent *const torrent) const
 {
-    if (m_category.isNull()) return true;
+    if (!m_category)
+        return true;
 
-    return (torrent->belongsToCategory(m_category));
+    return (torrent->belongsToCategory(*m_category));
 }
 
 bool TorrentFilter::matchTag(const BitTorrent::Torrent *const torrent) const
 {
-    // Empty tag is a special value to indicate we're filtering for untagged torrents.
-    if (m_tag.isNull()) return true;
-    if (m_tag.isEmpty()) return torrent->tags().isEmpty();
+    if (!m_tag)
+        return true;
 
-    return (torrent->hasTag(m_tag));
+    // Empty tag is a special value to indicate we're filtering for untagged torrents.
+    if (m_tag->isEmpty())
+        return torrent->tags().isEmpty();
+
+    return torrent->hasTag(*m_tag);
 }
