@@ -34,6 +34,7 @@
 #include <libtorrent/fwd.hpp>
 
 #include "base/3rdparty/expected.hpp"
+#include "base/pathfwd.h"
 
 class QByteArray;
 class QFileDevice;
@@ -75,11 +76,11 @@ namespace Utils::IO
         }
 
     private:
-        QFileDevice *m_device;
+        QFileDevice *m_device = nullptr;
         std::shared_ptr<QByteArray> m_buffer;
-        int m_bufferSize;
+        int m_bufferSize = 0;
     };
 
-    nonstd::expected<void, QString> saveToFile(const QString &path, const QByteArray &data);
-    nonstd::expected<void, QString> saveToFile(const QString &path, const lt::entry &data);
+    nonstd::expected<void, QString> saveToFile(const Path &path, const QByteArray &data);
+    nonstd::expected<void, QString> saveToFile(const Path &path, const lt::entry &data);
 }
