@@ -28,10 +28,15 @@
 
 #include "net.h"
 
+#include <QList>
 #include <QNetworkInterface>
+#include <QPair>
 #include <QSslCertificate>
 #include <QSslKey>
 #include <QString>
+#include <QVector>
+
+#include "base/global.h"
 
 namespace Utils
 {
@@ -42,54 +47,49 @@ namespace Utils
             return !QHostAddress(ip).isNull();
         }
 
-        Subnet parseSubnet(const QString &subnetStr, bool *ok)
+        std::optional<Subnet> parseSubnet(const QString &subnetStr)
         {
-            const Subnet invalid = qMakePair(QHostAddress(), -1);
             const Subnet subnet = QHostAddress::parseSubnet(subnetStr);
-            if (ok)
-                *ok = (subnet != invalid);
+            const Subnet invalid = {QHostAddress(), -1};
+            if (subnet == invalid)
+                return std::nullopt;
             return subnet;
         }
 
-        bool canParseSubnet(const QString &subnetStr)
-        {
-            bool ok = false;
-            parseSubnet(subnetStr, &ok);
-            return ok;
-        }
-
         bool isLoopbackAddress(const QHostAddress &addr)
         {
             return (addr == QHostAddress::LocalHost)
                     || (addr == QHostAddress::LocalHostIPv6)
-                    || (addr == QHostAddress(QLatin1String("::ffff:127.0.0.1")));
+                    || (addr == QHostAddress(u"::ffff:127.0.0.1"_qs));
         }
 
-        bool isIPInRange(const QHostAddress &addr, const QVector<Subnet> &subnets)
+        bool isIPInSubnets(const QHostAddress &addr, const QVector<Subnet> &subnets)
         {
             QHostAddress protocolEquivalentAddress;
             bool addrConversionOk = false;
 
-            if (addr.protocol() == QAbstractSocket::IPv4Protocol) {
+            if (addr.protocol() == QAbstractSocket::IPv4Protocol)
+            {
                 // always succeeds
                 protocolEquivalentAddress = QHostAddress(addr.toIPv6Address());
                 addrConversionOk = true;
             }
-            else {
+            else
+            {
                 // only succeeds when addr is an ipv4-mapped ipv6 address
                 protocolEquivalentAddress = QHostAddress(addr.toIPv4Address(&addrConversionOk));
             }
 
-            for (const Subnet &subnet : subnets)
-                if (addr.isInSubnet(subnet) || (addrConversionOk && protocolEquivalentAddress.isInSubnet(subnet)))
-                    return true;
-
-            return false;
+            return std::any_of(subnets.begin(), subnets.end(), [&](const Subnet &subnet)
+            {
+                return addr.isInSubnet(subnet)
+                    || (addrConversionOk && protocolEquivalentAddress.isInSubnet(subnet));
+            });
         }
 
         QString subnetToString(const Subnet &subnet)
         {
-            return subnet.first.toString() + '/' + QString::number(subnet.second);
+            return subnet.first.toString() + u'/' + QString::number(subnet.second);
         }
 
         QHostAddress canonicalIPv6Addr(const QHostAddress &addr)
@@ -126,9 +126,11 @@ namespace Utils
         QList<QSslCertificate> loadSSLCertificate(const QByteArray &data)
         {
             const QList<QSslCertificate> certs {QSslCertificate::fromData(data)};
-            if (std::any_of(certs.cbegin(), certs.cend(), [](const QSslCertificate &c) { return c.isNull(); }))
-                return {};
-            return certs;
+            const bool hasInvalidCerts = std::any_of(certs.cbegin(), certs.cend(), [](const QSslCertificate &cert)
+            {
+                return cert.isNull();
+            });
+            return hasInvalidCerts ? QList<QSslCertificate>() : certs;
         }
 
         bool isSSLCertificatesValid(const QByteArray &data)
@@ -139,7 +141,7 @@ namespace Utils
         QSslKey loadSSLKey(const QByteArray &data)
         {
             // try different formats
-            QSslKey key {data, QSsl::Rsa};
+            const QSslKey key {data, QSsl::Rsa};
             if (!key.isNull())
                 return key;
             return {data, QSsl::Ec};
