@@ -37,24 +37,24 @@
 #include "uithememanager.h"
 #include "utils.h"
 
-#define SETTINGS_KEY(name) "CookiesDialog/" name
+#define SETTINGS_KEY(name) u"CookiesDialog/" name
 
 CookiesDialog::CookiesDialog(QWidget *parent)
     : QDialog(parent)
     , m_ui(new Ui::CookiesDialog)
     , m_cookiesModel(new CookiesModel(Net::DownloadManager::instance()->allCookies(), this))
-    , m_storeDialogSize(SETTINGS_KEY("Size"))
+    , m_storeDialogSize(SETTINGS_KEY(u"Size"_qs))
 #if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
-    , m_storeViewState("GUI/Qt6/" SETTINGS_KEY("ViewState"))
+    , m_storeViewState("GUI/Qt6/" SETTINGS_KEY(u"ViewState"_qs))
 #else
-    , m_storeViewState(SETTINGS_KEY("CookiesViewState"))
+    , m_storeViewState(SETTINGS_KEY(u"CookiesViewState"_qs))
 #endif
 {
     m_ui->setupUi(this);
 
-    setWindowIcon(UIThemeManager::instance()->getIcon("preferences-web-browser-cookies"));
-    m_ui->buttonAdd->setIcon(UIThemeManager::instance()->getIcon("list-add"));
-    m_ui->buttonDelete->setIcon(UIThemeManager::instance()->getIcon("list-remove"));
+    setWindowIcon(UIThemeManager::instance()->getIcon(u"browser-cookies"_qs));
+    m_ui->buttonAdd->setIcon(UIThemeManager::instance()->getIcon(u"list-add"_qs));
+    m_ui->buttonDelete->setIcon(UIThemeManager::instance()->getIcon(u"list-remove"_qs));
     m_ui->buttonAdd->setIconSize(Utils::Gui::mediumIconSize());
     m_ui->buttonDelete->setIconSize(Utils::Gui::mediumIconSize());
 
@@ -64,7 +64,9 @@ CookiesDialog::CookiesDialog(QWidget *parent)
                     m_cookiesModel->index(0, 0),
                     QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
 
-    Utils::Gui::resize(this, m_storeDialogSize);
+    if (const QSize dialogSize = m_storeDialogSize; dialogSize.isValid())
+        resize(dialogSize);
+
     m_ui->treeView->header()->restoreState(m_storeViewState);
 }
 
