@@ -26,8 +26,7 @@
  * exception statement from your version.
  */
 
-#ifndef DELETIONCONFIRMATIONDIALOG_H
-#define DELETIONCONFIRMATIONDIALOG_H
+#pragma once
 
 #include <QDialog>
 
@@ -41,6 +40,7 @@ namespace Ui
 class DeletionConfirmationDialog : public QDialog
 {
     Q_OBJECT
+    Q_DISABLE_COPY_MOVE(DeletionConfirmationDialog)
 
 public:
     DeletionConfirmationDialog(QWidget *parent, int size, const QString &name, bool defaultDeleteFiles);
@@ -53,7 +53,5 @@ private slots:
     void on_rememberBtn_clicked();
 
 private:
-    Ui::DeletionConfirmationDialog *m_ui;
+    Ui::DeletionConfirmationDialog *m_ui = nullptr;
 };
-
-#endif // DELETIONCONFIRMATIONDIALOG_H
