@@ -32,6 +32,7 @@
 #include <QMenu>
 #include <QPalette>
 
+#include "base/global.h"
 #include "log/logfiltermodel.h"
 #include "log/loglistview.h"
 #include "log/logmodel.h"
@@ -50,26 +51,26 @@ ExecutionLogWidget::ExecutionLogWidget(const Log::MsgTypes types, QWidget *paren
     LogListView *messageView = new LogListView(this);
     messageView->setModel(m_messageFilterModel);
     messageView->setContextMenuPolicy(Qt::CustomContextMenu);
-    connect(messageView, &LogListView::customContextMenuRequested, this, [this, messageView, messageModel](const QPoint &pos)
+    connect(messageView, &LogListView::customContextMenuRequested, this, [this, messageView, messageModel]()
     {
-        displayContextMenu(pos, messageView, messageModel);
+        displayContextMenu(messageView, messageModel);
     });
 
     LogPeerModel *peerModel = new LogPeerModel(this);
     LogListView *peerView = new LogListView(this);
     peerView->setModel(peerModel);
     peerView->setContextMenuPolicy(Qt::CustomContextMenu);
-    connect(peerView, &LogListView::customContextMenuRequested, this, [this, peerView, peerModel](const QPoint &pos)
+    connect(peerView, &LogListView::customContextMenuRequested, this, [this, peerView, peerModel]()
     {
-        displayContextMenu(pos, peerView, peerModel);
+        displayContextMenu(peerView, peerModel);
     });
 
     m_ui->tabGeneral->layout()->addWidget(messageView);
     m_ui->tabBan->layout()->addWidget(peerView);
 
 #ifndef Q_OS_MACOS
-    m_ui->tabConsole->setTabIcon(0, UIThemeManager::instance()->getIcon("view-calendar-journal"));
-    m_ui->tabConsole->setTabIcon(1, UIThemeManager::instance()->getIcon("view-filter"));
+    m_ui->tabConsole->setTabIcon(0, UIThemeManager::instance()->getIcon(u"help-contents"_qs, u"view-calendar-journal"_qs));
+    m_ui->tabConsole->setTabIcon(1, UIThemeManager::instance()->getIcon(u"ip-blocked"_qs, u"view-filter"_qs));
 #endif
 }
 
@@ -83,7 +84,7 @@ void ExecutionLogWidget::setMessageTypes(const Log::MsgTypes types)
     m_messageFilterModel->setMessageTypes(types);
 }
 
-void ExecutionLogWidget::displayContextMenu(const QPoint &pos, const LogListView *view, const BaseLogModel *model) const
+void ExecutionLogWidget::displayContextMenu(const LogListView *view, const BaseLogModel *model) const
 {
     QMenu *menu = new QMenu;
     menu->setAttribute(Qt::WA_DeleteOnClose);
@@ -91,12 +92,12 @@ void ExecutionLogWidget::displayContextMenu(const QPoint &pos, const LogListView
     // only show copy action if any of the row is selected
     if (view->currentIndex().isValid())
     {
-        menu->addAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Copy")
+        menu->addAction(UIThemeManager::instance()->getIcon(u"edit-copy"_qs), tr("Copy")
             , view, &LogListView::copySelection);
     }
 
-    menu->addAction(UIThemeManager::instance()->getIcon("edit-clear"), tr("Clear")
+    menu->addAction(UIThemeManager::instance()->getIcon(u"edit-clear"_qs), tr("Clear")
         , model, &BaseLogModel::reset);
 
-    menu->popup(view->mapToGlobal(pos));
+    menu->popup(QCursor::pos());
 }
