@@ -16,18 +16,19 @@
 #include <QStyle>
 #include <QToolButton>
 
+#include "base/global.h"
 #include "uithememanager.h"
 
 LineEdit::LineEdit(QWidget *parent)
     : QLineEdit(parent)
 {
     m_searchButton = new QToolButton(this);
-    m_searchButton->setIcon(UIThemeManager::instance()->getIcon("edit-find"));
+    m_searchButton->setIcon(UIThemeManager::instance()->getIcon(u"edit-find"_qs));
     m_searchButton->setCursor(Qt::ArrowCursor);
-    m_searchButton->setStyleSheet("QToolButton {border: none; padding: 2px;}");
+    m_searchButton->setStyleSheet(u"QToolButton {border: none; padding: 2px;}"_qs);
 
     // padding between text and widget borders
-    setStyleSheet(QString::fromLatin1("QLineEdit {padding-left: %1px;}").arg(m_searchButton->sizeHint().width()));
+    setStyleSheet(u"QLineEdit {padding-left: %1px;}"_qs.arg(m_searchButton->sizeHint().width()));
 
     setClearButtonEnabled(true);
 
@@ -47,7 +48,8 @@ void LineEdit::resizeEvent(QResizeEvent *e)
 
 void LineEdit::keyPressEvent(QKeyEvent *event)
 {
-    if ((event->modifiers() == Qt::NoModifier) && (event->key() == Qt::Key_Escape)) {
+    if ((event->modifiers() == Qt::NoModifier) && (event->key() == Qt::Key_Escape))
+    {
         clear();
     }
     QLineEdit::keyPressEvent(event);
