@@ -34,11 +34,10 @@
 #include <QSpacerItem>
 
 #include "base/global.h"
-#include "uithememanager.h"
+#include "gui/uithememanager.h"
 
 PropTabBar::PropTabBar(QWidget *parent)
     : QHBoxLayout(parent)
-    , m_currentIndex(-1)
 {
     setAlignment(Qt::AlignLeft | Qt::AlignCenter);
     setSpacing(3);
@@ -46,7 +45,7 @@ PropTabBar::PropTabBar(QWidget *parent)
     // General tab
     QPushButton *mainInfosButton = new QPushButton(
 #ifndef Q_OS_MACOS
-            UIThemeManager::instance()->getIcon("document-properties"),
+            UIThemeManager::instance()->getIcon(u"help-about"_qs, u"document-properties"_qs),
 #endif
             tr("General"), parent);
     mainInfosButton->setShortcut(Qt::ALT + Qt::Key_G);
@@ -55,7 +54,7 @@ PropTabBar::PropTabBar(QWidget *parent)
     // Trackers tab
     QPushButton *trackersButton = new QPushButton(
 #ifndef Q_OS_MACOS
-            UIThemeManager::instance()->getIcon("network-server"),
+            UIThemeManager::instance()->getIcon(u"trackers"_qs, u"network-server"_qs),
 #endif
             tr("Trackers"), parent);
     trackersButton->setShortcut(Qt::ALT + Qt::Key_C);
@@ -64,7 +63,7 @@ PropTabBar::PropTabBar(QWidget *parent)
     // Peers tab
     QPushButton *peersButton = new QPushButton(
 #ifndef Q_OS_MACOS
-            UIThemeManager::instance()->getIcon("edit-find-user"),
+            UIThemeManager::instance()->getIcon(u"peers"_qs),
 #endif
             tr("Peers"), parent);
     peersButton->setShortcut(Qt::ALT + Qt::Key_R);
@@ -73,7 +72,7 @@ PropTabBar::PropTabBar(QWidget *parent)
     // URL seeds tab
     QPushButton *URLSeedsButton = new QPushButton(
 #ifndef Q_OS_MACOS
-            UIThemeManager::instance()->getIcon("network-server"),
+            UIThemeManager::instance()->getIcon(u"network-server"_qs),
 #endif
             tr("HTTP Sources"), parent);
     URLSeedsButton->setShortcut(Qt::ALT + Qt::Key_B);
@@ -82,7 +81,7 @@ PropTabBar::PropTabBar(QWidget *parent)
     // Files tab
     QPushButton *filesButton = new QPushButton(
 #ifndef Q_OS_MACOS
-            UIThemeManager::instance()->getIcon("inode-directory"),
+            UIThemeManager::instance()->getIcon(u"directory"_qs),
 #endif
             tr("Content"), parent);
     filesButton->setShortcut(Qt::ALT + Qt::Key_Z);
@@ -93,18 +92,15 @@ PropTabBar::PropTabBar(QWidget *parent)
     // Speed tab
     QPushButton *speedButton = new QPushButton(
 #ifndef Q_OS_MACOS
-            UIThemeManager::instance()->getIcon("office-chart-line"),
+            UIThemeManager::instance()->getIcon(u"chart-line"_qs),
 #endif
             tr("Speed"), parent);
     speedButton->setShortcut(Qt::ALT + Qt::Key_D);
     addWidget(speedButton);
     m_btnGroup->addButton(speedButton, SpeedTab);
     // SIGNAL/SLOT
-    connect(m_btnGroup, qOverload<int>(&QButtonGroup::buttonClicked)
+    connect(m_btnGroup, &QButtonGroup::idClicked
             , this, &PropTabBar::setCurrentIndex);
-    // Disable buttons focus
-    for (QAbstractButton *btn : asConst(m_btnGroup->buttons()))
-        btn->setFocusPolicy(Qt::NoFocus);
 }
 
 int PropTabBar::currentIndex() const
@@ -117,8 +113,10 @@ void PropTabBar::setCurrentIndex(int index)
     if (index >= m_btnGroup->buttons().size())
         index = 0;
     // If asked to hide or if the currently selected tab is clicked
-    if ((index < 0) || (m_currentIndex == index)) {
-        if (m_currentIndex >= 0) {
+    if ((index < 0) || (m_currentIndex == index))
+    {
+        if (m_currentIndex >= 0)
+        {
           m_btnGroup->button(m_currentIndex)->setDown(false);
           m_currentIndex = -1;
           emit visibilityToggled(false);
@@ -126,10 +124,12 @@ void PropTabBar::setCurrentIndex(int index)
         return;
     }
     // Unselect previous tab
-    if (m_currentIndex >= 0) {
+    if (m_currentIndex >= 0)
+    {
         m_btnGroup->button(m_currentIndex)->setDown(false);
     }
-    else {
+    else
+    {
         // Nothing was selected, show!
         emit visibilityToggled(true);
     }
