@@ -33,6 +33,7 @@
 #include "base/global.h"
 #include "base/rss/rss_article.h"
 #include "base/rss/rss_item.h"
+#include "gui/uithememanager.h"
 
 ArticleListWidget::ArticleListWidget(QWidget *parent)
     : QListWidget(parent)
@@ -46,7 +47,7 @@ ArticleListWidget::ArticleListWidget(QWidget *parent)
 RSS::Article *ArticleListWidget::getRSSArticle(QListWidgetItem *item) const
 {
     Q_ASSERT(item);
-    return reinterpret_cast<RSS::Article *>(item->data(Qt::UserRole).value<quintptr>());
+    return item->data(Qt::UserRole).value<RSS::Article *>();
 }
 
 QListWidgetItem *ArticleListWidget::mapRSSArticle(RSS::Article *rssArticle) const
@@ -64,13 +65,16 @@ void ArticleListWidget::setRSSItem(RSS::Item *rssItem, bool unreadOnly)
 
     m_unreadOnly = unreadOnly;
     m_rssItem = rssItem;
-    if (m_rssItem) {
+    if (m_rssItem)
+    {
         connect(m_rssItem, &RSS::Item::newArticle, this, &ArticleListWidget::handleArticleAdded);
         connect(m_rssItem, &RSS::Item::articleRead, this, &ArticleListWidget::handleArticleRead);
         connect(m_rssItem, &RSS::Item::articleAboutToBeRemoved, this, &ArticleListWidget::handleArticleAboutToBeRemoved);
 
-        for (const auto article : asConst(rssItem->articles())) {
-            if (!(m_unreadOnly && article->isRead())) {
+        for (const auto article : asConst(rssItem->articles()))
+        {
+            if (!(m_unreadOnly && article->isRead()))
+            {
                 auto item = createItem(article);
                 addItem(item);
                 m_rssArticleToListItemMapping.insert(article, item);
@@ -83,7 +87,8 @@ void ArticleListWidget::setRSSItem(RSS::Item *rssItem, bool unreadOnly)
 
 void ArticleListWidget::handleArticleAdded(RSS::Article *rssArticle)
 {
-    if (!(m_unreadOnly && rssArticle->isRead())) {
+    if (!(m_unreadOnly && rssArticle->isRead()))
+    {
         auto item = createItem(rssArticle);
         insertItem(0, item);
         m_rssArticleToListItemMapping.insert(rssArticle, item);
@@ -97,8 +102,10 @@ void ArticleListWidget::handleArticleRead(RSS::Article *rssArticle)
     auto item = mapRSSArticle(rssArticle);
     if (!item) return;
 
-    item->setData(Qt::ForegroundRole, QPalette().color(QPalette::Inactive, QPalette::WindowText));
-    item->setData(Qt::DecorationRole, QIcon(":/icons/sphere.png"));
+    const QColor defaultColor {palette().color(QPalette::Inactive, QPalette::WindowText)};
+    const QBrush foregroundBrush {UIThemeManager::instance()->getColor(u"RSS.ReadArticle"_qs, defaultColor)};
+    item->setData(Qt::ForegroundRole, foregroundBrush);
+    item->setData(Qt::DecorationRole, UIThemeManager::instance()->getIcon(u"loading"_qs, u"sphere"_qs));
 
     checkInvariant();
 }
@@ -117,17 +124,23 @@ void ArticleListWidget::checkInvariant() const
 QListWidgetItem *ArticleListWidget::createItem(RSS::Article *article) const
 {
     Q_ASSERT(article);
-    QListWidgetItem *item = new QListWidgetItem;
+    auto *item = new QListWidgetItem;
 
     item->setData(Qt::DisplayRole, article->title());
-    item->setData(Qt::UserRole, reinterpret_cast<quintptr>(article));
-    if (article->isRead()) {
-        item->setData(Qt::ForegroundRole, QPalette().color(QPalette::Inactive, QPalette::WindowText));
-        item->setData(Qt::DecorationRole, QIcon(":/icons/sphere.png"));
+    item->setData(Qt::UserRole, QVariant::fromValue(article));
+    if (article->isRead())
+    {
+        const QColor defaultColor {palette().color(QPalette::Inactive, QPalette::WindowText)};
+        const QBrush foregroundBrush {UIThemeManager::instance()->getColor(u"RSS.ReadArticle"_qs, defaultColor)};
+        item->setData(Qt::ForegroundRole, foregroundBrush);
+        item->setData(Qt::DecorationRole, UIThemeManager::instance()->getIcon(u"loading"_qs, u"sphere"_qs));
     }
-    else {
-        item->setData(Qt::ForegroundRole, QPalette().color(QPalette::Active, QPalette::Link));
-        item->setData(Qt::DecorationRole, QIcon(":/icons/sphere2.png"));
+    else
+    {
+        const QColor defaultColor {palette().color(QPalette::Active, QPalette::Link)};
+        const QBrush foregroundBrush {UIThemeManager::instance()->getColor(u"RSS.UnreadArticle"_qs, defaultColor)};
+        item->setData(Qt::ForegroundRole, foregroundBrush);
+        item->setData(Qt::DecorationRole, UIThemeManager::instance()->getIcon(u"loading"_qs, u"sphere"_qs));
     }
 
     return item;
