@@ -30,7 +30,6 @@
 
 #include <QApplication>
 #include <QDateTime>
-#include <QDir>
 #include <QDebug>
 #include <QNetworkDiskCache>
 #include <QNetworkReply>
@@ -38,6 +37,8 @@
 #include <QScrollBar>
 #include <QStyle>
 
+#include "base/global.h"
+#include "base/path.h"
 #include "base/profile.h"
 
 HtmlBrowser::HtmlBrowser(QWidget *parent)
@@ -45,7 +46,7 @@ HtmlBrowser::HtmlBrowser(QWidget *parent)
 {
     m_netManager = new QNetworkAccessManager(this);
     m_diskCache = new QNetworkDiskCache(this);
-    m_diskCache->setCacheDirectory(QDir::cleanPath(specialFolderLocation(SpecialFolder::Cache) + "/rss"));
+    m_diskCache->setCacheDirectory((specialFolderLocation(SpecialFolder::Cache) / Path(u"rss"_qs)).data());
     m_diskCache->setMaximumCacheSize(50 * 1024 * 1024);
     qDebug() << "HtmlBrowser  cache path:" << m_diskCache->cacheDirectory() << " max size:" << m_diskCache->maximumCacheSize() / 1024 / 1024 << "MB";
     m_netManager->setCache(m_diskCache);
@@ -63,7 +64,7 @@ QVariant HtmlBrowser::loadResource(int type, const QUrl &name)
     {
         QUrl url(name);
         if (url.scheme().isEmpty())
-            url.setScheme("http");
+            url.setScheme(u"http"_qs);
 
         QIODevice *dev = m_diskCache->data(url);
         if (dev)
@@ -107,7 +108,7 @@ void HtmlBrowser::resourceLoaded(QNetworkReply *reply)
         metaData.setUrl(reply->request().url());
         metaData.setSaveToDisk(true);
         atts[QNetworkRequest::HttpStatusCodeAttribute] = 200;
-        atts[QNetworkRequest::HttpReasonPhraseAttribute] = "Ok";
+        atts[QNetworkRequest::HttpReasonPhraseAttribute] = u"Ok"_qs;
         metaData.setAttributes(atts);
         metaData.setLastModified(QDateTime::currentDateTime());
         metaData.setExpirationDate(QDateTime::currentDateTime().addDays(1));
