@@ -52,15 +52,10 @@ void SearchSortModel::enableNameFilter(const bool enabled)
 void SearchSortModel::setNameFilter(const QString &searchTerm)
 {
     m_searchTerm = searchTerm;
-    if ((searchTerm.length() > 2)
-        && searchTerm.startsWith(QLatin1Char('"')) && searchTerm.endsWith(QLatin1Char('"')))
-        {
+    if ((searchTerm.length() > 2) && searchTerm.startsWith(u'"') && searchTerm.endsWith(u'"'))
         m_searchTermWords = QStringList(m_searchTerm.mid(1, m_searchTerm.length() - 2));
-    }
     else
-    {
-        m_searchTermWords = searchTerm.split(QLatin1Char(' '), Qt::SkipEmptyParts);
-    }
+        m_searchTermWords = searchTerm.split(u' ', Qt::SkipEmptyParts);
 }
 
 void SearchSortModel::setSizeFilter(const qint64 minSize, const qint64 maxSize)
