@@ -68,7 +68,7 @@ QString TorrentCategoryDialog::createCategory(QWidget *parent, const QString &pa
 
     QString newCategoryName = parentCategoryName;
     if (!newCategoryName.isEmpty())
-        newCategoryName += QLatin1Char('/');
+        newCategoryName += u'/';
     newCategoryName += tr("New Category");
 
     TorrentCategoryDialog dialog {parent};
@@ -133,6 +133,9 @@ QString TorrentCategoryDialog::categoryName() const
 void TorrentCategoryDialog::setCategoryName(const QString &categoryName)
 {
     m_ui->textCategoryName->setText(categoryName);
+
+    const int subcategoryNameStart = categoryName.lastIndexOf(u"/") + 1;
+    m_ui->textCategoryName->setSelection(subcategoryNameStart, (categoryName.size() - subcategoryNameStart));
 }
 
 BitTorrent::CategoryOptions TorrentCategoryDialog::categoryOptions() const
@@ -153,7 +156,7 @@ void TorrentCategoryDialog::setCategoryOptions(const BitTorrent::CategoryOptions
     if (categoryOptions.downloadPath)
     {
         m_ui->comboUseDownloadPath->setCurrentIndex(categoryOptions.downloadPath->enabled ? 1 : 2);
-        m_ui->comboDownloadPath->setSelectedPath(categoryOptions.downloadPath->enabled ? categoryOptions.downloadPath->path : QString());
+        m_ui->comboDownloadPath->setSelectedPath(categoryOptions.downloadPath->enabled ? categoryOptions.downloadPath->path : Path());
     }
     else
     {
@@ -164,30 +167,29 @@ void TorrentCategoryDialog::setCategoryOptions(const BitTorrent::CategoryOptions
 
 void TorrentCategoryDialog::categoryNameChanged(const QString &categoryName)
 {
-    const QString categoryPath = Utils::Fs::toValidFileSystemName(categoryName, true);
+    const Path categoryPath = Utils::Fs::toValidPath(categoryName);
     const auto *btSession = BitTorrent::Session::instance();
-    m_ui->comboSavePath->setPlaceholder(Utils::Fs::resolvePath(categoryPath, btSession->savePath()));
+    m_ui->comboSavePath->setPlaceholder(btSession->savePath() / categoryPath);
 
     const int index = m_ui->comboUseDownloadPath->currentIndex();
     const bool useDownloadPath = (index == 1) || ((index == 0) && BitTorrent::Session::instance()->isDownloadPathEnabled());
     if (useDownloadPath)
-        m_ui->comboDownloadPath->setPlaceholder(Utils::Fs::resolvePath(categoryPath, btSession->downloadPath()));
+        m_ui->comboDownloadPath->setPlaceholder(btSession->downloadPath() / categoryPath);
 
     m_ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(!categoryName.isEmpty());
 }
 
 void TorrentCategoryDialog::useDownloadPathChanged(const int index)
 {
-    if (const QString selectedPath = m_ui->comboDownloadPath->selectedPath(); !selectedPath.isEmpty())
+    if (const Path selectedPath = m_ui->comboDownloadPath->selectedPath(); !selectedPath.isEmpty())
         m_lastEnteredDownloadPath = selectedPath;
 
     m_ui->labelDownloadPath->setEnabled(index == 1);
     m_ui->comboDownloadPath->setEnabled(index == 1);
-    m_ui->comboDownloadPath->setSelectedPath((index == 1) ? m_lastEnteredDownloadPath : QString());
+    m_ui->comboDownloadPath->setSelectedPath((index == 1) ? m_lastEnteredDownloadPath : Path());
 
     const QString categoryName = m_ui->textCategoryName->text();
-    const QString categoryPath = Utils::Fs::resolvePath(Utils::Fs::toValidFileSystemName(categoryName, true)
-                                             , BitTorrent::Session::instance()->downloadPath());
+    const Path categoryPath = BitTorrent::Session::instance()->downloadPath() / Utils::Fs::toValidPath(categoryName);
     const bool useDownloadPath = (index == 1) || ((index == 0) && BitTorrent::Session::instance()->isDownloadPathEnabled());
-    m_ui->comboDownloadPath->setPlaceholder(useDownloadPath ? categoryPath : QString());
+    m_ui->comboDownloadPath->setPlaceholder(useDownloadPath ? categoryPath : Path());
 }
