@@ -42,33 +42,31 @@
 #include "ui_torrentcreatordialog.h"
 #include "utils.h"
 
-#define SETTINGS_KEY(name) "TorrentCreator/" name
+#define SETTINGS_KEY(name) u"TorrentCreator/" name
 
-TorrentCreatorDialog::TorrentCreatorDialog(QWidget *parent, const QString &defaultPath)
+TorrentCreatorDialog::TorrentCreatorDialog(QWidget *parent, const Path &defaultPath)
     : QDialog(parent)
     , m_ui(new Ui::TorrentCreatorDialog)
     , m_creatorThread(new BitTorrent::TorrentCreatorThread(this))
-    , m_storeDialogSize(SETTINGS_KEY("Size"))
-    , m_storePieceSize(SETTINGS_KEY("PieceSize"))
-    , m_storePrivateTorrent(SETTINGS_KEY("PrivateTorrent"))
-    , m_storeStartSeeding(SETTINGS_KEY("StartSeeding"))
-    , m_storeIgnoreRatio(SETTINGS_KEY("IgnoreRatio"))
-#if (LIBTORRENT_VERSION_NUM >= 20000)
-    , m_storeTorrentFormat(SETTINGS_KEY("TorrentFormat"))
+    , m_storeDialogSize(SETTINGS_KEY(u"Size"_qs))
+    , m_storePieceSize(SETTINGS_KEY(u"PieceSize"_qs))
+    , m_storePrivateTorrent(SETTINGS_KEY(u"PrivateTorrent"_qs))
+    , m_storeStartSeeding(SETTINGS_KEY(u"StartSeeding"_qs))
+    , m_storeIgnoreRatio(SETTINGS_KEY(u"IgnoreRatio"_qs))
+#ifdef QBT_USES_LIBTORRENT2
+    , m_storeTorrentFormat(SETTINGS_KEY(u"TorrentFormat"_qs))
 #else
-    , m_storeOptimizeAlignment(SETTINGS_KEY("OptimizeAlignment"))
-    , m_paddedFileSizeLimit(SETTINGS_KEY("PaddedFileSizeLimit"))
+    , m_storeOptimizeAlignment(SETTINGS_KEY(u"OptimizeAlignment"_qs))
+    , m_paddedFileSizeLimit(SETTINGS_KEY(u"PaddedFileSizeLimit"_qs))
 #endif
-    , m_storeLastAddPath(SETTINGS_KEY("LastAddPath"))
-    , m_storeTrackerList(SETTINGS_KEY("TrackerList"))
-    , m_storeWebSeedList(SETTINGS_KEY("WebSeedList"))
-    , m_storeComments(SETTINGS_KEY("Comments"))
-    , m_storeLastSavePath(SETTINGS_KEY("LastSavePath"))
-    , m_storeSource(SETTINGS_KEY("Source"))
+    , m_storeLastAddPath(SETTINGS_KEY(u"LastAddPath"_qs))
+    , m_storeTrackerList(SETTINGS_KEY(u"TrackerList"_qs))
+    , m_storeWebSeedList(SETTINGS_KEY(u"WebSeedList"_qs))
+    , m_storeComments(SETTINGS_KEY(u"Comments"_qs))
+    , m_storeLastSavePath(SETTINGS_KEY(u"LastSavePath"_qs))
+    , m_storeSource(SETTINGS_KEY(u"Source"_qs))
 {
     m_ui->setupUi(this);
-    setAttribute(Qt::WA_DeleteOnClose);
-    setModal(false);
 
     m_ui->buttonBox->button(QDialogButtonBox::Ok)->setText(tr("Create Torrent"));
 
@@ -84,13 +82,11 @@ TorrentCreatorDialog::TorrentCreatorDialog(QWidget *parent, const QString &defau
     loadSettings();
     updateInputPath(defaultPath);
 
-#if (LIBTORRENT_VERSION_NUM >= 20000)
+#ifdef QBT_USES_LIBTORRENT2
     m_ui->checkOptimizeAlignment->hide();
 #else
     m_ui->widgetTorrentFormat->hide();
 #endif
-
-    show();
 }
 
 TorrentCreatorDialog::~TorrentCreatorDialog()
@@ -100,24 +96,24 @@ TorrentCreatorDialog::~TorrentCreatorDialog()
     delete m_ui;
 }
 
-void TorrentCreatorDialog::updateInputPath(const QString &path)
+void TorrentCreatorDialog::updateInputPath(const Path &path)
 {
     if (path.isEmpty()) return;
-    m_ui->textInputPath->setText(Utils::Fs::toNativePath(path));
+    m_ui->textInputPath->setText(path.toString());
     updateProgressBar(0);
 }
 
 void TorrentCreatorDialog::onAddFolderButtonClicked()
 {
-    QString oldPath = m_ui->textInputPath->text();
-    QString path = QFileDialog::getExistingDirectory(this, tr("Select folder"), oldPath);
+    const QString oldPath = m_ui->textInputPath->text();
+    const Path path {QFileDialog::getExistingDirectory(this, tr("Select folder"), oldPath)};
     updateInputPath(path);
 }
 
 void TorrentCreatorDialog::onAddFileButtonClicked()
 {
-    QString oldPath = m_ui->textInputPath->text();
-    QString path = QFileDialog::getOpenFileName(this, tr("Select file"), oldPath);
+    const QString oldPath = m_ui->textInputPath->text();
+    const Path path {QFileDialog::getOpenFileName(this, tr("Select file"), oldPath)};
     updateInputPath(path);
 }
 
@@ -127,7 +123,7 @@ int TorrentCreatorDialog::getPieceSize() const
     return pieceSizes[m_ui->comboPieceSize->currentIndex()] * 1024;
 }
 
-#if (LIBTORRENT_VERSION_NUM >= 20000)
+#ifdef QBT_USES_LIBTORRENT2
 BitTorrent::TorrentFormat TorrentCreatorDialog::getTorrentFormat() const
 {
     switch (m_ui->comboTorrentFormat->currentIndex())
@@ -156,63 +152,64 @@ void TorrentCreatorDialog::dropEvent(QDropEvent *event)
     if (event->mimeData()->hasUrls())
     {
         // only take the first one
-        QUrl firstItem = event->mimeData()->urls().first();
-        QString path = (firstItem.scheme().compare("file", Qt::CaseInsensitive) == 0)
-                       ? firstItem.toLocalFile() : firstItem.toString();
+        const QUrl firstItem = event->mimeData()->urls().first();
+        const Path path {
+            (firstItem.scheme().compare(u"file", Qt::CaseInsensitive) == 0)
+                    ? firstItem.toLocalFile() : firstItem.toString()
+        };
         updateInputPath(path);
     }
 }
 
 void TorrentCreatorDialog::dragEnterEvent(QDragEnterEvent *event)
 {
-    if (event->mimeData()->hasFormat("text/plain") || event->mimeData()->hasFormat("text/uri-list"))
+    if (event->mimeData()->hasFormat(u"text/plain"_qs) || event->mimeData()->hasFormat(u"text/uri-list"_qs))
         event->acceptProposedAction();
 }
 
 // Main function that create a .torrent file
 void TorrentCreatorDialog::onCreateButtonClicked()
 {
-    QString input = Utils::Fs::toUniformPath(m_ui->textInputPath->text()).trimmed();
+    const auto inputPath = Utils::Fs::toCanonicalPath(Path(m_ui->textInputPath->text().trimmed()));
 
     // test if readable
-    const QFileInfo fi(input);
-    if (!fi.isReadable())
+    if (!Utils::Fs::isReadable(inputPath))
     {
         QMessageBox::critical(this, tr("Torrent creation failed"), tr("Reason: Path to file/folder is not readable."));
         return;
     }
-    input = fi.canonicalFilePath();
 
     // get save path
-    const QString savePath = m_storeLastSavePath.get(QDir::homePath()) + QLatin1Char('/') + fi.fileName() + QLatin1String(".torrent");
-    QString destination = QFileDialog::getSaveFileName(this, tr("Select where to save the new torrent"), savePath, tr("Torrent Files (*.torrent)"));
-    if (destination.isEmpty())
+    const Path lastSavePath = (m_storeLastSavePath.get(Utils::Fs::homePath()) / Path(inputPath.filename() + u".torrent"));
+    Path destPath {QFileDialog::getSaveFileName(this, tr("Select where to save the new torrent"), lastSavePath.data(), tr("Torrent Files (*.torrent)"))};
+    if (destPath.isEmpty())
         return;
-    if (!destination.endsWith(C_TORRENT_FILE_EXTENSION, Qt::CaseInsensitive))
-        destination += C_TORRENT_FILE_EXTENSION;
-    m_storeLastSavePath = Utils::Fs::branchPath(destination);
+    if (!destPath.hasExtension(TORRENT_FILE_EXTENSION))
+        destPath += TORRENT_FILE_EXTENSION;
+    m_storeLastSavePath = destPath.parentPath();
 
     // Disable dialog & set busy cursor
     setInteractionEnabled(false);
     setCursor(QCursor(Qt::WaitCursor));
 
     const QStringList trackers = m_ui->trackersList->toPlainText().trimmed()
-        .replace(QRegularExpression("\n\n[\n]+"), "\n\n").split('\n');
+        .replace(QRegularExpression(u"\n\n[\n]+"_qs), u"\n\n"_qs).split(u'\n');
     const BitTorrent::TorrentCreatorParams params
     {
         m_ui->checkPrivate->isChecked()
-#if (LIBTORRENT_VERSION_NUM >= 20000)
+#ifdef QBT_USES_LIBTORRENT2
         , getTorrentFormat()
 #else
         , m_ui->checkOptimizeAlignment->isChecked()
         , getPaddedFileSizeLimit()
 #endif
         , getPieceSize()
-        , input, destination
+        , inputPath
+        , destPath
         , m_ui->txtComment->toPlainText()
         , m_ui->lineEditSource->text()
         , trackers
-        , m_ui->URLSeedsList->toPlainText().split('\n', QString::SkipEmptyParts)
+        , m_ui->URLSeedsList->toPlainText().split(u'\n', Qt::SkipEmptyParts)
     };
 
     // run the creator thread
@@ -227,15 +224,15 @@ void TorrentCreatorDialog::handleCreationFailure(const QString &msg)
     setInteractionEnabled(true);
 }
 
-void TorrentCreatorDialog::handleCreationSuccess(const QString &path, const QString &branchPath)
+void TorrentCreatorDialog::handleCreationSuccess(const Path &path, const Path &branchPath)
 {
     // Remove busy cursor
     setCursor(QCursor(Qt::ArrowCursor));
     if (m_ui->checkStartSeeding->isChecked())
     {
         // Create save path temp data
-        const BitTorrent::TorrentInfo info = BitTorrent::TorrentInfo::loadFromFile(Utils::Fs::toNativePath(path));
-        if (!info.isValid())
+        const nonstd::expected<BitTorrent::TorrentInfo, QString> result = BitTorrent::TorrentInfo::loadFromFile(path);
+        if (!result)
         {
             QMessageBox::critical(this, tr("Torrent creation failed"), tr("Reason: Created torrent is invalid. It won't be added to download list."));
             return;
@@ -251,10 +248,10 @@ void TorrentCreatorDialog::handleCreationSuccess(const QString &path, const QStr
         }
         params.useAutoTMM = false;  // otherwise if it is on by default, it will overwrite `savePath` to the default save path
 
-        BitTorrent::Session::instance()->addTorrent(info, params);
+        BitTorrent::Session::instance()->addTorrent(result.value(), params);
     }
     QMessageBox::information(this, tr("Torrent creator")
-        , QString::fromLatin1("%1\n%2").arg(tr("Torrent created:"), Utils::Fs::toNativePath(path)));
+        , u"%1\n%2"_qs.arg(tr("Torrent created:"), path.toString()));
     setInteractionEnabled(true);
 }
 
@@ -265,8 +262,8 @@ void TorrentCreatorDialog::updateProgressBar(int progress)
 
 void TorrentCreatorDialog::updatePiecesCount()
 {
-    const QString path = m_ui->textInputPath->text().trimmed();
-#if (LIBTORRENT_VERSION_NUM >= 20000)
+    const Path path {m_ui->textInputPath->text().trimmed()};
+#ifdef QBT_USES_LIBTORRENT2
     const int count = BitTorrent::TorrentCreatorThread::calculateTotalPieces(
         path, getPieceSize(), getTorrentFormat());
 #else
@@ -291,7 +288,7 @@ void TorrentCreatorDialog::setInteractionEnabled(const bool enabled) const
     m_ui->checkStartSeeding->setEnabled(enabled);
     m_ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(enabled);
     m_ui->checkIgnoreShareLimits->setEnabled(enabled && m_ui->checkStartSeeding->isChecked());
-#if (LIBTORRENT_VERSION_NUM >= 20000)
+#ifdef QBT_USES_LIBTORRENT2
     m_ui->widgetTorrentFormat->setEnabled(enabled);
 #else
     m_ui->checkOptimizeAlignment->setEnabled(enabled);
@@ -301,13 +298,13 @@ void TorrentCreatorDialog::setInteractionEnabled(const bool enabled) const
 
 void TorrentCreatorDialog::saveSettings()
 {
-    m_storeLastAddPath = m_ui->textInputPath->text().trimmed();
+    m_storeLastAddPath = Path(m_ui->textInputPath->text().trimmed());
 
     m_storePieceSize = m_ui->comboPieceSize->currentIndex();
     m_storePrivateTorrent = m_ui->checkPrivate->isChecked();
     m_storeStartSeeding = m_ui->checkStartSeeding->isChecked();
     m_storeIgnoreRatio = m_ui->checkIgnoreShareLimits->isChecked();
-#if (LIBTORRENT_VERSION_NUM >= 20000)
+#ifdef QBT_USES_LIBTORRENT2
     m_storeTorrentFormat = m_ui->comboTorrentFormat->currentIndex();
 #else
     m_storeOptimizeAlignment = m_ui->checkOptimizeAlignment->isChecked();
@@ -324,14 +321,14 @@ void TorrentCreatorDialog::saveSettings()
 
 void TorrentCreatorDialog::loadSettings()
 {
-    m_ui->textInputPath->setText(m_storeLastAddPath.get(QDir::homePath()));
+    m_ui->textInputPath->setText(m_storeLastAddPath.get(Utils::Fs::homePath()).toString());
 
     m_ui->comboPieceSize->setCurrentIndex(m_storePieceSize);
     m_ui->checkPrivate->setChecked(m_storePrivateTorrent);
     m_ui->checkStartSeeding->setChecked(m_storeStartSeeding);
     m_ui->checkIgnoreShareLimits->setChecked(m_storeIgnoreRatio);
     m_ui->checkIgnoreShareLimits->setEnabled(m_ui->checkStartSeeding->isChecked());
-#if (LIBTORRENT_VERSION_NUM >= 20000)
+#ifdef QBT_USES_LIBTORRENT2
     m_ui->comboTorrentFormat->setCurrentIndex(m_storeTorrentFormat.get(1));
 #else
     m_ui->checkOptimizeAlignment->setChecked(m_storeOptimizeAlignment.get(true));
@@ -343,5 +340,6 @@ void TorrentCreatorDialog::loadSettings()
     m_ui->txtComment->setPlainText(m_storeComments);
     m_ui->lineEditSource->setText(m_storeSource);
 
-    Utils::Gui::resize(this, m_storeDialogSize);
+    if (const QSize dialogSize = m_storeDialogSize; dialogSize.isValid())
+        resize(dialogSize);
 }
