@@ -39,16 +39,15 @@
 #include <QRegularExpression>
 #include <QSet>
 #include <QShortcut>
-#include <QTableView>
 #include <QVector>
 #include <QWheelEvent>
 
-#include "base/bittorrent/common.h"
 #include "base/bittorrent/session.h"
 #include "base/bittorrent/torrent.h"
 #include "base/bittorrent/trackerentry.h"
 #include "base/global.h"
 #include "base/logger.h"
+#include "base/path.h"
 #include "base/preferences.h"
 #include "base/torrentfilter.h"
 #include "base/utils/compare.h"
@@ -91,12 +90,9 @@ namespace
         if (!torrent->hasMetadata())
             return false;
 
-        for (const QString &filePath : asConst(torrent->filePaths()))
+        for (const Path &filePath : asConst(torrent->filePaths()))
         {
-            QString fileName = Utils::Fs::fileName(filePath);
-            if (fileName.endsWith(QB_EXT))
-                fileName.chop(QB_EXT.length());
-            if (Utils::Misc::isPreviewable(fileName))
+            if (Utils::Misc::isPreviewable(filePath))
                 return true;
         }
 
@@ -156,6 +152,7 @@ TransferListWidget::TransferListWidget(QWidget *parent, MainWindow *mainWindow)
 #if defined(Q_OS_MACOS)
     setAttribute(Qt::WA_MacShowFocusRect, false);
 #endif
+    header()->setFirstSectionMovable(true);
     header()->setStretchLastSection(false);
     header()->setTextElideMode(Qt::ElideRight);
 
@@ -174,6 +171,9 @@ TransferListWidget::TransferListWidget(QWidget *parent, MainWindow *mainWindow)
         setColumnHidden(TransferListModel::TR_AMOUNT_LEFT, true);
         setColumnHidden(TransferListModel::TR_TIME_ELAPSED, true);
         setColumnHidden(TransferListModel::TR_SAVE_PATH, true);
+        setColumnHidden(TransferListModel::TR_DOWNLOAD_PATH, true);
+        setColumnHidden(TransferListModel::TR_INFOHASH_V1, true);
+        setColumnHidden(TransferListModel::TR_INFOHASH_V2, true);
         setColumnHidden(TransferListModel::TR_COMPLETED, true);
         setColumnHidden(TransferListModel::TR_RATIO_LIMIT, true);
         setColumnHidden(TransferListModel::TR_SEEN_COMPLETE_DATE, true);
@@ -198,8 +198,10 @@ TransferListWidget::TransferListWidget(QWidget *parent, MainWindow *mainWindow)
     //end up being size 0 when the new version is launched with
     //a conf file from the previous version.
     for (int i = 0; i < TransferListModel::NB_COLUMNS; ++i)
+    {
         if ((columnWidth(i) <= 0) && (!isColumnHidden(i)))
             resizeColumnToContents(i);
+    }
 
     setContextMenuPolicy(Qt::CustomContextMenu);
 
@@ -207,7 +209,7 @@ TransferListWidget::TransferListWidget(QWidget *parent, MainWindow *mainWindow)
     connect(this, &QAbstractItemView::doubleClicked, this, &TransferListWidget::torrentDoubleClicked);
     connect(this, &QWidget::customContextMenuRequested, this, &TransferListWidget::displayListMenu);
     header()->setContextMenuPolicy(Qt::CustomContextMenu);
-    connect(header(), &QWidget::customContextMenuRequested, this, &TransferListWidget::displayDLHoSMenu);
+    connect(header(), &QWidget::customContextMenuRequested, this, &TransferListWidget::displayColumnHeaderMenu);
     connect(header(), &QHeaderView::sectionMoved, this, &TransferListWidget::saveSettings);
     connect(header(), &QHeaderView::sectionResized, this, &TransferListWidget::saveSettings);
     connect(header(), &QHeaderView::sortIndicatorChanged, this, &TransferListWidget::saveSettings);
@@ -224,13 +226,8 @@ TransferListWidget::TransferListWidget(QWidget *parent, MainWindow *mainWindow)
     connect(doubleClickHotkeyEnter, &QShortcut::activated, this, &TransferListWidget::torrentDoubleClicked);
     const auto *recheckHotkey = new QShortcut(Qt::CTRL + Qt::Key_R, this, nullptr, nullptr, Qt::WidgetShortcut);
     connect(recheckHotkey, &QShortcut::activated, this, &TransferListWidget::recheckSelectedTorrents);
-
-    // This hack fixes reordering of first column with Qt5.
-    // https://github.com/qtproject/qtbase/commit/e0fc088c0c8bc61dbcaf5928b24986cd61a22777
-    QTableView unused;
-    unused.setVerticalHeader(header());
-    header()->setParent(this);
-    unused.setVerticalHeader(new QHeaderView(Qt::Horizontal));
+    const auto *forceStartHotkey = new QShortcut(Qt::CTRL + Qt::Key_M, this, nullptr, nullptr, Qt::WidgetShortcut);
+    connect(forceStartHotkey, &QShortcut::activated, this, &TransferListWidget::forceStartSelectedTorrents);
 }
 
 TransferListWidget::~TransferListWidget()
@@ -244,7 +241,7 @@ TransferListModel *TransferListWidget::getSourceModel() const
     return m_listModel;
 }
 
-void TransferListWidget::previewFile(const QString &filePath)
+void TransferListWidget::previewFile(const Path &filePath)
 {
     Utils::Gui::openPath(filePath);
 }
@@ -337,12 +334,11 @@ void TransferListWidget::setSelectedTorrentsLocation()
     if (torrents.isEmpty())
         return;
 
-    const QString oldLocation = torrents[0]->savePath();
+    const Path oldLocation = torrents[0]->savePath();
 
-    auto fileDialog = new QFileDialog(this, tr("Choose save path"), oldLocation);
+    auto fileDialog = new QFileDialog(this, tr("Choose save path"), oldLocation.data());
     fileDialog->setAttribute(Qt::WA_DeleteOnClose);
     fileDialog->setFileMode(QFileDialog::Directory);
-    fileDialog->setModal(true);
     fileDialog->setOptions(QFileDialog::DontConfirmOverwrite | QFileDialog::ShowDirsOnly | QFileDialog::HideNameFilterDetails);
     connect(fileDialog, &QDialog::accepted, this, [this, fileDialog]()
     {
@@ -350,8 +346,8 @@ void TransferListWidget::setSelectedTorrentsLocation()
         if (torrents.isEmpty())
             return;
 
-        const QString newLocation = fileDialog->selectedFiles().constFirst();
-        if (newLocation.isEmpty() || !QDir(newLocation).exists())
+        const Path newLocation {fileDialog->selectedFiles().constFirst()};
+        if (!newLocation.exists())
             return;
 
         // Actually move storage
@@ -362,17 +358,35 @@ void TransferListWidget::setSelectedTorrentsLocation()
         }
     });
 
-    fileDialog->show();
+    fileDialog->open();
 }
 
 void TransferListWidget::pauseAllTorrents()
 {
+    // Show confirmation if user would really like to Pause All
+    const QMessageBox::StandardButton ret =
+            QMessageBox::question(this, tr("Confirm pause")
+                                  , tr("Would you like to pause all torrents?")
+                                  , (QMessageBox::Yes | QMessageBox::No));
+
+    if (ret != QMessageBox::Yes)
+        return;
+
     for (BitTorrent::Torrent *const torrent : asConst(BitTorrent::Session::instance()->torrents()))
         torrent->pause();
 }
 
 void TransferListWidget::resumeAllTorrents()
 {
+    // Show confirmation if user would really like to Resume All
+    const QMessageBox::StandardButton ret =
+            QMessageBox::question(this, tr("Confirm resume")
+                                  , tr("Would you like to resume all torrents?")
+                                  , (QMessageBox::Yes | QMessageBox::No));
+
+    if (ret != QMessageBox::Yes)
+        return;
+
     for (BitTorrent::Torrent *const torrent : asConst(BitTorrent::Session::instance()->torrents()))
         torrent->resume();
 }
@@ -497,7 +511,7 @@ void TransferListWidget::copySelectedMagnetURIs() const
     for (BitTorrent::Torrent *const torrent : asConst(getSelectedTorrents()))
         magnetUris << torrent->createMagnetURI();
 
-    qApp->clipboard()->setText(magnetUris.join('\n'));
+    qApp->clipboard()->setText(magnetUris.join(u'\n'));
 }
 
 void TransferListWidget::copySelectedNames() const
@@ -506,7 +520,7 @@ void TransferListWidget::copySelectedNames() const
     for (BitTorrent::Torrent *const torrent : asConst(getSelectedTorrents()))
         torrentNames << torrent->name();
 
-    qApp->clipboard()->setText(torrentNames.join('\n'));
+    qApp->clipboard()->setText(torrentNames.join(u'\n'));
 }
 
 void TransferListWidget::copySelectedInfohashes(const CopyInfohashPolicy policy) const
@@ -532,7 +546,7 @@ void TransferListWidget::copySelectedInfohashes(const CopyInfohashPolicy policy)
         break;
     }
 
-    qApp->clipboard()->setText(infoHashes.join('\n'));
+    qApp->clipboard()->setText(infoHashes.join(u'\n'));
 }
 
 void TransferListWidget::copySelectedIDs() const
@@ -541,7 +555,7 @@ void TransferListWidget::copySelectedIDs() const
     for (BitTorrent::Torrent *const torrent : asConst(getSelectedTorrents()))
         torrentIDs << torrent->id().toString();
 
-    qApp->clipboard()->setText(torrentIDs.join('\n'));
+    qApp->clipboard()->setText(torrentIDs.join(u'\n'));
 }
 
 void TransferListWidget::hideQueuePosColumn(bool hide)
@@ -553,28 +567,28 @@ void TransferListWidget::hideQueuePosColumn(bool hide)
 
 void TransferListWidget::openSelectedTorrentsFolder() const
 {
-    QSet<QString> pathsList;
+    QSet<Path> paths;
 #ifdef Q_OS_MACOS
     // On macOS you expect both the files and folders to be opened in their parent
     // folders prehilighted for opening, so we use a custom method.
     for (BitTorrent::Torrent *const torrent : asConst(getSelectedTorrents()))
     {
-        const QString contentPath = QDir(torrent->actualStorageLocation()).absoluteFilePath(torrent->contentPath());
-        pathsList.insert(contentPath);
+        const Path contentPath = torrent->contentPath();
+        paths.insert(contentPath);
     }
-    MacUtils::openFiles(pathsList);
+    MacUtils::openFiles(PathList(paths.cbegin(), paths.cend()));
 #else
     for (BitTorrent::Torrent *const torrent : asConst(getSelectedTorrents()))
     {
-        const QString contentPath = torrent->contentPath();
-        if (!pathsList.contains(contentPath))
+        const Path contentPath = torrent->contentPath();
+        if (!paths.contains(contentPath))
         {
             if (torrent->filesCount() == 1)
                 Utils::Gui::openFolderSelect(contentPath);
             else
                 Utils::Gui::openPath(contentPath);
         }
-        pathsList.insert(contentPath);
+        paths.insert(contentPath);
     }
 #endif // Q_OS_MACOS
 }
@@ -626,48 +640,59 @@ void TransferListWidget::reannounceSelectedTorrents()
         torrent->forceReannounce();
 }
 
+int TransferListWidget::visibleColumnsCount() const
+{
+    int count = 0;
+    for (int i = 0, iMax = header()->count(); i < iMax; ++i)
+    {
+        if (!isColumnHidden(i))
+            ++count;
+    }
+
+    return count;
+}
+
 // hide/show columns menu
-void TransferListWidget::displayDLHoSMenu(const QPoint&)
+void TransferListWidget::displayColumnHeaderMenu()
 {
     auto menu = new QMenu(this);
     menu->setAttribute(Qt::WA_DeleteOnClose);
     menu->setTitle(tr("Column visibility"));
+    menu->setToolTipsVisible(true);
 
-    for (int i = 0; i < m_listModel->columnCount(); ++i)
+    for (int i = 0; i < TransferListModel::NB_COLUMNS; ++i)
     {
         if (!BitTorrent::Session::instance()->isQueueingSystemEnabled() && (i == TransferListModel::TR_QUEUE_POSITION))
             continue;
 
-        QAction *myAct = menu->addAction(m_listModel->headerData(i, Qt::Horizontal, Qt::DisplayRole).toString());
-        myAct->setCheckable(true);
-        myAct->setChecked(!isColumnHidden(i));
-        myAct->setData(i);
+        const auto columnName = m_listModel->headerData(i, Qt::Horizontal, Qt::DisplayRole).toString();
+        QAction *action = menu->addAction(columnName, this, [this, i](const bool checked)
+        {
+            if (!checked && (visibleColumnsCount() <= 1))
+                return;
+
+            setColumnHidden(i, !checked);
+
+            if (checked && (columnWidth(i) <= 5))
+                resizeColumnToContents(i);
+
+            saveSettings();
+        });
+        action->setCheckable(true);
+        action->setChecked(!isColumnHidden(i));
     }
 
-    connect(menu, &QMenu::triggered, this, [this](const QAction *action)
+    menu->addSeparator();
+    QAction *resizeAction = menu->addAction(tr("Resize columns"), this, [this]()
     {
-        int visibleCols = 0;
-        for (int i = 0; i < TransferListModel::NB_COLUMNS; ++i)
+        for (int i = 0, count = header()->count(); i < count; ++i)
         {
             if (!isColumnHidden(i))
-                ++visibleCols;
-
-            if (visibleCols > 1)
-                break;
+                resizeColumnToContents(i);
         }
-
-        const int col = action->data().toInt();
-
-        if (!isColumnHidden(col) && visibleCols == 1)
-            return;
-
-        setColumnHidden(col, !isColumnHidden(col));
-
-        if (!isColumnHidden(col) && columnWidth(col) <= 5)
-            resizeColumnToContents(col);
-
         saveSettings();
     });
+    resizeAction->setToolTip(tr("Resize all non-hidden columns to the size of their contents"));
 
     menu->popup(QCursor::pos());
 }
@@ -756,6 +781,57 @@ void TransferListWidget::editTorrentTrackers()
     trackerDialog->open();
 }
 
+void TransferListWidget::exportTorrent()
+{
+    if (getSelectedTorrents().isEmpty())
+        return;
+
+    auto fileDialog = new QFileDialog(this, tr("Choose folder to save exported .torrent files"));
+    fileDialog->setAttribute(Qt::WA_DeleteOnClose);
+    fileDialog->setFileMode(QFileDialog::Directory);
+    fileDialog->setOptions(QFileDialog::ShowDirsOnly);
+    connect(fileDialog, &QFileDialog::fileSelected, this, [this](const QString &dir)
+    {
+        const QVector<BitTorrent::Torrent *> torrents = getSelectedTorrents();
+        if (torrents.isEmpty())
+            return;
+
+        const Path savePath {dir};
+        if (!savePath.exists())
+            return;
+
+        const QString errorMsg = tr("Export .torrent file failed. Torrent: \"%1\". Save path: \"%2\". Reason: \"%3\"");
+
+        bool hasError = false;
+        for (const BitTorrent::Torrent *torrent : torrents)
+        {
+            const Path filePath = savePath / Path(torrent->name() + u".torrent");
+            if (filePath.exists())
+            {
+                LogMsg(errorMsg.arg(torrent->name(), filePath.toString(), tr("A file with the same name already exists")) , Log::WARNING);
+                hasError = true;
+                continue;
+            }
+
+            const nonstd::expected<void, QString> result = torrent->exportToFile(filePath);
+            if (!result)
+            {
+                LogMsg(errorMsg.arg(torrent->name(), filePath.toString(), result.error()) , Log::WARNING);
+                hasError = true;
+                continue;
+            }
+        }
+
+        if (hasError)
+        {
+            QMessageBox::warning(this, tr("Export .torrent file error")
+                , tr("Errors occurred when exporting .torrent files. Check execution log for details."));
+        }
+    });
+
+    fileDialog->open();
+}
+
 void TransferListWidget::confirmRemoveAllTagsForSelection()
 {
     QMessageBox::StandardButton response = QMessageBox::question(
@@ -774,10 +850,10 @@ QStringList TransferListWidget::askTagsForSelection(const QString &dialogTitle)
         bool ok = false;
         invalid = false;
         const QString tagsInput = AutoExpandableDialog::getText(
-            this, dialogTitle, tr("Comma-separated tags:"), QLineEdit::Normal, "", &ok).trimmed();
+            this, dialogTitle, tr("Comma-separated tags:"), QLineEdit::Normal, {}, &ok).trimmed();
         if (!ok || tagsInput.isEmpty())
             return {};
-        tags = tagsInput.split(',', Qt::SkipEmptyParts);
+        tags = tagsInput.split(u',', Qt::SkipEmptyParts);
         for (QString &tag : tags)
         {
             tag = tag.trimmed();
@@ -816,7 +892,7 @@ void TransferListWidget::renameSelectedTorrent()
     QString name = AutoExpandableDialog::getText(this, tr("Rename"), tr("New name:"), QLineEdit::Normal, torrent->name(), &ok);
     if (ok && !name.isEmpty())
     {
-        name.replace(QRegularExpression("\r?\n|\r"), " ");
+        name.replace(QRegularExpression(u"\r?\n|\r"_qs), u" "_qs);
         // Rename the torrent
         m_listModel->setData(mi, name, Qt::DisplayRole);
     }
@@ -843,7 +919,7 @@ void TransferListWidget::clearSelectionTags()
     applyToSelectedTorrents([](BitTorrent::Torrent *const torrent) { torrent->removeAllTags(); });
 }
 
-void TransferListWidget::displayListMenu(const QPoint &)
+void TransferListWidget::displayListMenu()
 {
     const QModelIndexList selectedIndexes = selectionModel()->selectedRows();
     if (selectedIndexes.isEmpty()) return;
@@ -854,47 +930,47 @@ void TransferListWidget::displayListMenu(const QPoint &)
 
     // Create actions
 
-    auto *actionStart = new QAction(UIThemeManager::instance()->getIcon("media-playback-start"), tr("Resume", "Resume/start the torrent"), listMenu);
+    auto *actionStart = new QAction(UIThemeManager::instance()->getIcon(u"torrent-start"_qs, u"media-playback-start"_qs), tr("&Resume", "Resume/start the torrent"), listMenu);
     connect(actionStart, &QAction::triggered, this, &TransferListWidget::startSelectedTorrents);
-    auto *actionPause = new QAction(UIThemeManager::instance()->getIcon("media-playback-pause"), tr("Pause", "Pause the torrent"), listMenu);
+    auto *actionPause = new QAction(UIThemeManager::instance()->getIcon(u"torrent-stop"_qs, u"media-playback-pause"_qs), tr("&Pause", "Pause the torrent"), listMenu);
     connect(actionPause, &QAction::triggered, this, &TransferListWidget::pauseSelectedTorrents);
-    auto *actionForceStart = new QAction(UIThemeManager::instance()->getIcon("media-seek-forward"), tr("Force Resume", "Force Resume/start the torrent"), listMenu);
+    auto *actionForceStart = new QAction(UIThemeManager::instance()->getIcon(u"torrent-start-forced"_qs, u"media-playback-start"_qs), tr("Force Resu&me", "Force Resume/start the torrent"), listMenu);
     connect(actionForceStart, &QAction::triggered, this, &TransferListWidget::forceStartSelectedTorrents);
-    auto *actionDelete = new QAction(UIThemeManager::instance()->getIcon("list-remove"), tr("Delete", "Delete the torrent"), listMenu);
+    auto *actionDelete = new QAction(UIThemeManager::instance()->getIcon(u"list-remove"_qs), tr("&Remove", "Remove the torrent"), listMenu);
     connect(actionDelete, &QAction::triggered, this, &TransferListWidget::softDeleteSelectedTorrents);
-    auto *actionPreviewFile = new QAction(UIThemeManager::instance()->getIcon("view-preview"), tr("Preview file..."), listMenu);
+    auto *actionPreviewFile = new QAction(UIThemeManager::instance()->getIcon(u"view-preview"_qs), tr("Pre&view file..."), listMenu);
     connect(actionPreviewFile, &QAction::triggered, this, &TransferListWidget::previewSelectedTorrents);
-    auto *actionTorrentOptions = new QAction(UIThemeManager::instance()->getIcon("configure"), tr("Torrent options..."), listMenu);
+    auto *actionTorrentOptions = new QAction(UIThemeManager::instance()->getIcon(u"configure"_qs), tr("Torrent &options..."), listMenu);
     connect(actionTorrentOptions, &QAction::triggered, this, &TransferListWidget::setTorrentOptions);
-    auto *actionOpenDestinationFolder = new QAction(UIThemeManager::instance()->getIcon("inode-directory"), tr("Open destination folder"), listMenu);
+    auto *actionOpenDestinationFolder = new QAction(UIThemeManager::instance()->getIcon(u"directory"_qs), tr("Open destination &folder"), listMenu);
     connect(actionOpenDestinationFolder, &QAction::triggered, this, &TransferListWidget::openSelectedTorrentsFolder);
-    auto *actionIncreaseQueuePos = new QAction(UIThemeManager::instance()->getIcon("go-up"), tr("Move up", "i.e. move up in the queue"), listMenu);
+    auto *actionIncreaseQueuePos = new QAction(UIThemeManager::instance()->getIcon(u"go-up"_qs), tr("Move &up", "i.e. move up in the queue"), listMenu);
     connect(actionIncreaseQueuePos, &QAction::triggered, this, &TransferListWidget::increaseQueuePosSelectedTorrents);
-    auto *actionDecreaseQueuePos = new QAction(UIThemeManager::instance()->getIcon("go-down"), tr("Move down", "i.e. Move down in the queue"), listMenu);
+    auto *actionDecreaseQueuePos = new QAction(UIThemeManager::instance()->getIcon(u"go-down"_qs), tr("Move &down", "i.e. Move down in the queue"), listMenu);
     connect(actionDecreaseQueuePos, &QAction::triggered, this, &TransferListWidget::decreaseQueuePosSelectedTorrents);
-    auto *actionTopQueuePos = new QAction(UIThemeManager::instance()->getIcon("go-top"), tr("Move to top", "i.e. Move to top of the queue"), listMenu);
+    auto *actionTopQueuePos = new QAction(UIThemeManager::instance()->getIcon(u"go-top"_qs), tr("Move to &top", "i.e. Move to top of the queue"), listMenu);
     connect(actionTopQueuePos, &QAction::triggered, this, &TransferListWidget::topQueuePosSelectedTorrents);
-    auto *actionBottomQueuePos = new QAction(UIThemeManager::instance()->getIcon("go-bottom"), tr("Move to bottom", "i.e. Move to bottom of the queue"), listMenu);
+    auto *actionBottomQueuePos = new QAction(UIThemeManager::instance()->getIcon(u"go-bottom"_qs), tr("Move to &bottom", "i.e. Move to bottom of the queue"), listMenu);
     connect(actionBottomQueuePos, &QAction::triggered, this, &TransferListWidget::bottomQueuePosSelectedTorrents);
-    auto *actionSetTorrentPath = new QAction(UIThemeManager::instance()->getIcon("inode-directory"), tr("Set location..."), listMenu);
+    auto *actionSetTorrentPath = new QAction(UIThemeManager::instance()->getIcon(u"set-location"_qs, u"inode-directory"_qs), tr("Set loc&ation..."), listMenu);
     connect(actionSetTorrentPath, &QAction::triggered, this, &TransferListWidget::setSelectedTorrentsLocation);
-    auto *actionForceRecheck = new QAction(UIThemeManager::instance()->getIcon("document-edit-verify"), tr("Force recheck"), listMenu);
+    auto *actionForceRecheck = new QAction(UIThemeManager::instance()->getIcon(u"force-recheck"_qs, u"document-edit-verify"_qs), tr("Force rec&heck"), listMenu);
     connect(actionForceRecheck, &QAction::triggered, this, &TransferListWidget::recheckSelectedTorrents);
-    auto *actionForceReannounce = new QAction(UIThemeManager::instance()->getIcon("document-edit-verify"), tr("Force reannounce"), listMenu);
+    auto *actionForceReannounce = new QAction(UIThemeManager::instance()->getIcon(u"reannounce"_qs, u"document-edit-verify"_qs), tr("Force r&eannounce"), listMenu);
     connect(actionForceReannounce, &QAction::triggered, this, &TransferListWidget::reannounceSelectedTorrents);
-    auto *actionCopyMagnetLink = new QAction(UIThemeManager::instance()->getIcon("kt-magnet"), tr("Magnet link"), listMenu);
+    auto *actionCopyMagnetLink = new QAction(UIThemeManager::instance()->getIcon(u"torrent-magnet"_qs, u"kt-magnet"_qs), tr("&Magnet link"), listMenu);
     connect(actionCopyMagnetLink, &QAction::triggered, this, &TransferListWidget::copySelectedMagnetURIs);
-    auto *actionCopyID = new QAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Torrent ID"), listMenu);
+    auto *actionCopyID = new QAction(UIThemeManager::instance()->getIcon(u"help-about"_qs, u"edit-copy"_qs), tr("Torrent &ID"), listMenu);
     connect(actionCopyID, &QAction::triggered, this, &TransferListWidget::copySelectedIDs);
-    auto *actionCopyName = new QAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Name"), listMenu);
+    auto *actionCopyName = new QAction(UIThemeManager::instance()->getIcon(u"name"_qs, u"edit-copy"_qs), tr("&Name"), listMenu);
     connect(actionCopyName, &QAction::triggered, this, &TransferListWidget::copySelectedNames);
-    auto *actionCopyHash1 = new QAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Info hash v1"), listMenu);
+    auto *actionCopyHash1 = new QAction(UIThemeManager::instance()->getIcon(u"hash"_qs, u"edit-copy"_qs), tr("Info &hash v1"), listMenu);
     connect(actionCopyHash1, &QAction::triggered, this, [this]() { copySelectedInfohashes(CopyInfohashPolicy::Version1); });
-    auto *actionCopyHash2 = new QAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Info hash v2"), listMenu);
+    auto *actionCopyHash2 = new QAction(UIThemeManager::instance()->getIcon(u"hash"_qs, u"edit-copy"_qs), tr("Info h&ash v2"), listMenu);
     connect(actionCopyHash2, &QAction::triggered, this, [this]() { copySelectedInfohashes(CopyInfohashPolicy::Version2); });
     auto *actionSuperSeedingMode = new TriStateAction(tr("Super seeding mode"), listMenu);
     connect(actionSuperSeedingMode, &QAction::triggered, this, &TransferListWidget::setSelectedTorrentsSuperSeeding);
-    auto *actionRename = new QAction(UIThemeManager::instance()->getIcon("edit-rename"), tr("Rename..."), listMenu);
+    auto *actionRename = new QAction(UIThemeManager::instance()->getIcon(u"edit-rename"_qs), tr("Re&name..."), listMenu);
     connect(actionRename, &QAction::triggered, this, &TransferListWidget::renameSelectedTorrent);
     auto *actionSequentialDownload = new TriStateAction(tr("Download in sequential order"), listMenu);
     connect(actionSequentialDownload, &QAction::triggered, this, &TransferListWidget::setSelectedTorrentsSequentialDownload);
@@ -903,8 +979,10 @@ void TransferListWidget::displayListMenu(const QPoint &)
     auto *actionAutoTMM = new TriStateAction(tr("Automatic Torrent Management"), listMenu);
     actionAutoTMM->setToolTip(tr("Automatic mode means that various torrent properties (e.g. save path) will be decided by the associated category"));
     connect(actionAutoTMM, &QAction::triggered, this, &TransferListWidget::setSelectedAutoTMMEnabled);
-    auto *actionEditTracker = new QAction(UIThemeManager::instance()->getIcon("edit-rename"), tr("Edit trackers..."), listMenu);
+    auto *actionEditTracker = new QAction(UIThemeManager::instance()->getIcon(u"edit-rename"_qs), tr("Edit trac&kers..."), listMenu);
     connect(actionEditTracker, &QAction::triggered, this, &TransferListWidget::editTorrentTrackers);
+    auto *actionExportTorrent = new QAction(UIThemeManager::instance()->getIcon(u"edit-copy"_qs), tr("E&xport .torrent..."), listMenu);
+    connect(actionExportTorrent, &QAction::triggered, this, &TransferListWidget::exportTorrent);
     // End of actions
 
     // Enable/disable pause/start action given the DL state
@@ -922,6 +1000,7 @@ void TransferListWidget::displayListMenu(const QPoint &)
     TagSet tagsInAny;
     TagSet tagsInAll;
     bool hasInfohashV1 = false, hasInfohashV2 = false;
+    bool oneCanForceReannounce = false;
 
     for (const QModelIndex &index : selectedIndexes)
     {
@@ -985,7 +1064,8 @@ void TransferListWidget::displayListMenu(const QPoint &)
         else
             needsStart = true;
 
-        if (torrent->isPaused())
+        const bool isPaused = torrent->isPaused();
+        if (isPaused)
             needsStart = true;
         else
             needsPause = true;
@@ -1008,10 +1088,22 @@ void TransferListWidget::displayListMenu(const QPoint &)
 
         first = false;
 
+        const bool rechecking = torrent->isChecking();
+        if (rechecking)
+        {
+            needsStart = true;
+            needsPause = true;
+        }
+
+        const bool queued = (BitTorrent::Session::instance()->isQueueingSystemEnabled() && torrent->isQueued());
+
+        if (!isPaused && !rechecking && !queued)
+            oneCanForceReannounce = true;
+
         if (oneHasMetadata && oneNotSeed && !allSameSequentialDownloadMode
             && !allSamePrioFirstlast && !allSameSuperSeeding && !allSameCategory
             && needsStart && needsForce && needsPause && needsPreview && !allSameAutoTMM
-            && hasInfohashV1 && hasInfohashV2)
+            && hasInfohashV1 && hasInfohashV2 && oneCanForceReannounce)
         {
             break;
         }
@@ -1035,18 +1127,18 @@ void TransferListWidget::displayListMenu(const QPoint &)
     QStringList categories = BitTorrent::Session::instance()->categories();
     std::sort(categories.begin(), categories.end(), Utils::Compare::NaturalLessThan<Qt::CaseInsensitive>());
 
-    QMenu *categoryMenu = listMenu->addMenu(UIThemeManager::instance()->getIcon("view-categories"), tr("Category"));
+    QMenu *categoryMenu = listMenu->addMenu(UIThemeManager::instance()->getIcon(u"view-categories"_qs), tr("Categor&y"));
 
-    categoryMenu->addAction(UIThemeManager::instance()->getIcon("list-add"), tr("New...", "New category...")
+    categoryMenu->addAction(UIThemeManager::instance()->getIcon(u"list-add"_qs), tr("&New...", "New category...")
         , this, &TransferListWidget::askNewCategoryForSelection);
-    categoryMenu->addAction(UIThemeManager::instance()->getIcon("edit-clear"), tr("Reset", "Reset category")
-        , this, [this]() { setSelectionCategory(""); });
+    categoryMenu->addAction(UIThemeManager::instance()->getIcon(u"edit-clear"_qs), tr("&Reset", "Reset category")
+        , this, [this]() { setSelectionCategory(u""_qs); });
     categoryMenu->addSeparator();
 
     for (const QString &category : asConst(categories))
     {
-        const QString escapedCategory = QString(category).replace('&', "&&");  // avoid '&' becomes accelerator key
-        QAction *categoryAction = categoryMenu->addAction(UIThemeManager::instance()->getIcon("inode-directory"), escapedCategory
+        const QString escapedCategory = QString(category).replace(u'&', u"&&"_qs);  // avoid '&' becomes accelerator key
+        QAction *categoryAction = categoryMenu->addAction(UIThemeManager::instance()->getIcon(u"view-categories"_qs), escapedCategory
             , this, [this, category]() { setSelectionCategory(category); });
 
         if (allSameCategory && (category == firstCategory))
@@ -1060,11 +1152,11 @@ void TransferListWidget::displayListMenu(const QPoint &)
     QStringList tags(BitTorrent::Session::instance()->tags().values());
     std::sort(tags.begin(), tags.end(), Utils::Compare::NaturalLessThan<Qt::CaseInsensitive>());
 
-    QMenu *tagsMenu = listMenu->addMenu(UIThemeManager::instance()->getIcon("view-categories"), tr("Tags"));
+    QMenu *tagsMenu = listMenu->addMenu(UIThemeManager::instance()->getIcon(u"tags"_qs, u"view-categories"_qs), tr("Ta&gs"));
 
-    tagsMenu->addAction(UIThemeManager::instance()->getIcon("list-add"), tr("Add...", "Add / assign multiple tags...")
+    tagsMenu->addAction(UIThemeManager::instance()->getIcon(u"list-add"_qs), tr("&Add...", "Add / assign multiple tags...")
         , this, &TransferListWidget::askAddTagsForSelection);
-    tagsMenu->addAction(UIThemeManager::instance()->getIcon("edit-clear"), tr("Remove All", "Remove all tags")
+    tagsMenu->addAction(UIThemeManager::instance()->getIcon(u"edit-clear"_qs), tr("&Remove All", "Remove all tags")
         , this, [this]()
     {
         if (Preferences::instance()->confirmRemoveAllTags())
@@ -1133,24 +1225,27 @@ void TransferListWidget::displayListMenu(const QPoint &)
     if (addedPreviewAction)
         listMenu->addSeparator();
     if (oneHasMetadata)
-    {
         listMenu->addAction(actionForceRecheck);
-        listMenu->addAction(actionForceReannounce);
-        listMenu->addSeparator();
-    }
+    // We can not force reannounce torrents that are paused/errored/checking/missing files/queued.
+    // We may already have the tracker list from magnet url. So we can force reannounce torrents without metadata anyway.
+    listMenu->addAction(actionForceReannounce);
+    actionForceReannounce->setEnabled(oneCanForceReannounce);
+    if (!oneCanForceReannounce)
+        actionForceReannounce->setToolTip(tr("Can not force reannounce if torrent is Paused/Queued/Errored/Checking"));
+    listMenu->addSeparator();
     listMenu->addAction(actionOpenDestinationFolder);
     if (BitTorrent::Session::instance()->isQueueingSystemEnabled() && oneNotSeed)
     {
         listMenu->addSeparator();
-        QMenu *queueMenu = listMenu->addMenu(tr("Queue"));
+        QMenu *queueMenu = listMenu->addMenu(
+            UIThemeManager::instance()->getIcon(u"queued"_qs), tr("&Queue"));
         queueMenu->addAction(actionTopQueuePos);
         queueMenu->addAction(actionIncreaseQueuePos);
         queueMenu->addAction(actionDecreaseQueuePos);
         queueMenu->addAction(actionBottomQueuePos);
     }
 
-    QMenu *copySubMenu = listMenu->addMenu(
-        UIThemeManager::instance()->getIcon("edit-copy"), tr("Copy"));
+    QMenu *copySubMenu = listMenu->addMenu(UIThemeManager::instance()->getIcon(u"edit-copy"_qs), tr("&Copy"));
     copySubMenu->addAction(actionCopyName);
     copySubMenu->addAction(actionCopyHash1);
     actionCopyHash1->setEnabled(hasInfohashV1);
@@ -1159,6 +1254,9 @@ void TransferListWidget::displayListMenu(const QPoint &)
     copySubMenu->addAction(actionCopyMagnetLink);
     copySubMenu->addAction(actionCopyID);
 
+    actionExportTorrent->setToolTip(tr("Exported torrent is not necessarily the same as the imported"));
+    listMenu->addAction(actionExportTorrent);
+
     listMenu->popup(QCursor::pos());
 }
 
@@ -1169,8 +1267,8 @@ void TransferListWidget::currentChanged(const QModelIndex &current, const QModel
     if (current.isValid())
     {
         torrent = m_listModel->torrentHandle(mapToSource(current));
-        // Scroll Fix
-        scrollTo(current);
+        // Fix scrolling to the lowermost visible torrent
+        QMetaObject::invokeMethod(this, [this, current] { scrollTo(current); }, Qt::QueuedConnection);
     }
     emit currentTorrentChanged(torrent);
 }
@@ -1235,9 +1333,9 @@ void TransferListWidget::wheelEvent(QWheelEvent *event)
     {
         // Shift + scroll = horizontal scroll
         event->accept();
-        QWheelEvent scrollHEvent(event->position(), event->globalPosition()
+        QWheelEvent scrollHEvent {event->position(), event->globalPosition()
             , event->pixelDelta(), event->angleDelta().transposed(), event->buttons()
-            , event->modifiers(), event->phase(), event->inverted(), event->source());
+            , event->modifiers(), event->phase(), event->inverted(), event->source()};
         QTreeView::wheelEvent(&scrollHEvent);
         return;
     }
