@@ -26,26 +26,35 @@
  * exception statement from your version.
  */
 
-#ifndef TRANSFERLISTWIDGET_H
-#define TRANSFERLISTWIDGET_H
+#pragma once
 
 #include <functional>
+
+#include <QtContainerFwd>
 #include <QTreeView>
-#include <QVector>
+
+#include "base/bittorrent/infohash.h"
 
 class MainWindow;
-class TransferListDelegate;
+class Path;
 class TransferListModel;
 class TransferListSortModel;
 
 namespace BitTorrent
 {
-    class TorrentHandle;
+    class Torrent;
 }
 
+enum class CopyInfohashPolicy
+{
+    Version1,
+    Version2
+};
+
 class TransferListWidget final : public QTreeView
 {
     Q_OBJECT
+    Q_DISABLE_COPY_MOVE(TransferListWidget)
 
 public:
     TransferListWidget(QWidget *parent, MainWindow *mainWindow);
@@ -75,58 +84,54 @@ public slots:
     void bottomQueuePosSelectedTorrents();
     void copySelectedMagnetURIs() const;
     void copySelectedNames() const;
-    void copySelectedHashes() const;
+    void copySelectedInfohashes(CopyInfohashPolicy policy) const;
+    void copySelectedIDs() const;
     void openSelectedTorrentsFolder() const;
     void recheckSelectedTorrents();
     void reannounceSelectedTorrents();
-    void setDlLimitSelectedTorrents();
-    void setUpLimitSelectedTorrents();
-    void setMaxRatioSelectedTorrents();
+    void setTorrentOptions();
     void previewSelectedTorrents();
     void hideQueuePosColumn(bool hide);
-    void displayDLHoSMenu(const QPoint&);
     void applyNameFilter(const QString &name);
     void applyStatusFilter(int f);
     void applyCategoryFilter(const QString &category);
     void applyTagFilter(const QString &tag);
     void applyTrackerFilterAll();
-    void applyTrackerFilter(const QStringList &hashes);
-    void previewFile(const QString &filePath);
+    void applyTrackerFilter(const QSet<BitTorrent::TorrentID> &torrentIDs);
+    void previewFile(const Path &filePath);
     void renameSelectedTorrent();
 
-protected:
-    QModelIndex mapToSource(const QModelIndex &index) const;
-    QModelIndex mapFromSource(const QModelIndex &index) const;
-    bool loadSettings();
-    QVector<BitTorrent::TorrentHandle *> getSelectedTorrents() const;
+signals:
+    void currentTorrentChanged(BitTorrent::Torrent *const torrent);
 
-protected slots:
+private slots:
     void torrentDoubleClicked();
-    void displayListMenu(const QPoint &);
+    void displayListMenu();
+    void displayColumnHeaderMenu();
     void currentChanged(const QModelIndex &current, const QModelIndex&) override;
     void setSelectedTorrentsSuperSeeding(bool enabled) const;
     void setSelectedTorrentsSequentialDownload(bool enabled) const;
     void setSelectedFirstLastPiecePrio(bool enabled) const;
-    void setSelectedAutoTMMEnabled(bool enabled) const;
+    void setSelectedAutoTMMEnabled(bool enabled);
     void askNewCategoryForSelection();
     void saveSettings();
 
-signals:
-    void currentTorrentChanged(BitTorrent::TorrentHandle *const torrent);
-
 private:
     void wheelEvent(QWheelEvent *event) override;
+    QModelIndex mapToSource(const QModelIndex &index) const;
+    QModelIndex mapFromSource(const QModelIndex &index) const;
+    bool loadSettings();
+    QVector<BitTorrent::Torrent *> getSelectedTorrents() const;
     void askAddTagsForSelection();
     void editTorrentTrackers();
+    void exportTorrent();
     void confirmRemoveAllTagsForSelection();
     QStringList askTagsForSelection(const QString &dialogTitle);
-    void applyToSelectedTorrents(const std::function<void (BitTorrent::TorrentHandle *const)> &fn);
-    QVector<BitTorrent::TorrentHandle *> getVisibleTorrents() const;
+    void applyToSelectedTorrents(const std::function<void (BitTorrent::Torrent *const)> &fn);
+    QVector<BitTorrent::Torrent *> getVisibleTorrents() const;
+    int visibleColumnsCount() const;
 
-    TransferListDelegate *m_listDelegate;
-    TransferListModel *m_listModel;
-    TransferListSortModel *m_sortFilterModel;
-    MainWindow *m_mainWindow;
+    TransferListModel *m_listModel = nullptr;
+    TransferListSortModel *m_sortFilterModel = nullptr;
+    MainWindow *m_mainWindow = nullptr;
 };
-
-#endif // TRANSFERLISTWIDGET_H
