@@ -37,7 +37,7 @@
 #include "ui_watchedfolderoptionsdialog.h"
 #include "utils.h"
 
-#define SETTINGS_KEY(name) "WatchedFolderOptionsDialog/" name
+#define SETTINGS_KEY(name) u"WatchedFolderOptionsDialog/" name
 
 WatchedFolderOptionsDialog::WatchedFolderOptionsDialog(
         const TorrentFilesWatcher::WatchedFolderOptions &watchedFolderOptions, QWidget *parent)
@@ -45,18 +45,18 @@ WatchedFolderOptionsDialog::WatchedFolderOptionsDialog(
     , m_ui {new Ui::WatchedFolderOptionsDialog}
     , m_savePath {watchedFolderOptions.addTorrentParams.savePath}
     , m_downloadPath {watchedFolderOptions.addTorrentParams.downloadPath}
-    , m_storeDialogSize {SETTINGS_KEY("DialogSize")}
+    , m_storeDialogSize {SETTINGS_KEY(u"DialogSize"_qs)}
 {
     m_ui->setupUi(this);
 
     m_ui->savePath->setMode(FileSystemPathEdit::Mode::DirectorySave);
     m_ui->savePath->setDialogCaption(tr("Choose save path"));
 
-    const auto *session = BitTorrent::Session::instance();
-
     m_ui->downloadPath->setMode(FileSystemPathEdit::Mode::DirectorySave);
     m_ui->downloadPath->setDialogCaption(tr("Choose save path"));
-    m_ui->groupBoxDownloadPath->setChecked(watchedFolderOptions.addTorrentParams.useDownloadPath.value_or(session->isDownloadPathEnabled()));
+
+    const auto *session = BitTorrent::Session::instance();
+    m_useDownloadPath = watchedFolderOptions.addTorrentParams.useDownloadPath.value_or(session->isDownloadPathEnabled());
 
     connect(m_ui->comboTTM, qOverload<int>(&QComboBox::currentIndexChanged), this, &WatchedFolderOptionsDialog::onTMMChanged);
     connect(m_ui->categoryComboBox, qOverload<int>(&QComboBox::currentIndexChanged), this, &WatchedFolderOptionsDialog::onCategoryChanged);
@@ -77,7 +77,7 @@ WatchedFolderOptionsDialog::WatchedFolderOptionsDialog(
 
     if (!torrentParams.category.isEmpty())
         m_ui->categoryComboBox->addItem(torrentParams.category);
-    m_ui->categoryComboBox->addItem("");
+    m_ui->categoryComboBox->addItem(u""_qs);
 
     for (const QString &category : asConst(categories))
     {
@@ -87,7 +87,11 @@ WatchedFolderOptionsDialog::WatchedFolderOptionsDialog(
 
     loadState();
 
-    m_ui->buttonBox->button(QDialogButtonBox::Ok)->setFocus();
+    // Default focus
+    if (m_ui->comboTTM->currentIndex() == 0) // 0 is Manual mode
+        m_ui->savePath->setFocus();
+    else
+        m_ui->categoryComboBox->setFocus();
 }
 
 WatchedFolderOptionsDialog::~WatchedFolderOptionsDialog()
@@ -121,7 +125,8 @@ TorrentFilesWatcher::WatchedFolderOptions WatchedFolderOptionsDialog::watchedFol
 
 void WatchedFolderOptionsDialog::loadState()
 {
-    Utils::Gui::resize(this, m_storeDialogSize);
+    if (const QSize dialogSize = m_storeDialogSize; dialogSize.isValid())
+        resize(dialogSize);
 }
 
 void WatchedFolderOptionsDialog::saveState()
@@ -138,13 +143,13 @@ void WatchedFolderOptionsDialog::onCategoryChanged(const int index)
         const auto *btSession = BitTorrent::Session::instance();
         const QString categoryName = m_ui->categoryComboBox->currentText();
 
-        const QString savePath = btSession->categorySavePath(categoryName);
-        m_ui->savePath->setSelectedPath(Utils::Fs::toNativePath(savePath));
+        const Path savePath = btSession->categorySavePath(categoryName);
+        m_ui->savePath->setSelectedPath(savePath);
 
-        const QString finishedSavePath = btSession->categoryDownloadPath(categoryName);
-        m_ui->downloadPath->setSelectedPath(Utils::Fs::toNativePath(finishedSavePath));
+        const Path downloadPath = btSession->categoryDownloadPath(categoryName);
+        m_ui->downloadPath->setSelectedPath(downloadPath);
 
-        m_ui->groupBoxDownloadPath->setChecked(!finishedSavePath.isEmpty());
+        m_ui->groupBoxDownloadPath->setChecked(!downloadPath.isEmpty());
     }
 }
 
@@ -152,11 +157,11 @@ void WatchedFolderOptionsDialog::populateSavePaths()
 {
     const auto *btSession = BitTorrent::Session::instance();
 
-    const QString defaultSavePath {btSession->savePath()};
+    const Path defaultSavePath {btSession->savePath()};
     m_ui->savePath->setSelectedPath(!m_savePath.isEmpty() ? m_savePath : defaultSavePath);
 
-    const QString defaultFinishedSavePath {btSession->downloadPath()};
-    m_ui->downloadPath->setSelectedPath(!m_downloadPath.isEmpty() ? m_downloadPath : defaultFinishedSavePath);
+    const Path defaultDownloadPath {btSession->downloadPath()};
+    m_ui->downloadPath->setSelectedPath(!m_downloadPath.isEmpty() ? m_downloadPath : defaultDownloadPath);
 
     m_ui->groupBoxDownloadPath->setChecked(m_useDownloadPath);
 }
@@ -178,15 +183,15 @@ void WatchedFolderOptionsDialog::onTMMChanged(const int index)
 
         m_ui->savePath->blockSignals(true);
         m_savePath = m_ui->savePath->selectedPath();
-        const QString savePath = btSession->categorySavePath(m_ui->categoryComboBox->currentText());
+        const Path savePath = btSession->categorySavePath(m_ui->categoryComboBox->currentText());
         m_ui->savePath->setSelectedPath(savePath);
 
         m_ui->downloadPath->blockSignals(true);
         m_downloadPath = m_ui->downloadPath->selectedPath();
-        const QString finishedSavePath = btSession->categoryDownloadPath(m_ui->categoryComboBox->currentText());
-        m_ui->downloadPath->setSelectedPath(finishedSavePath);
+        const Path downloadPath = btSession->categoryDownloadPath(m_ui->categoryComboBox->currentText());
+        m_ui->downloadPath->setSelectedPath(downloadPath);
 
         m_useDownloadPath = m_ui->groupBoxDownloadPath->isChecked();
-        m_ui->groupBoxDownloadPath->setChecked(!finishedSavePath.isEmpty());
+        m_ui->groupBoxDownloadPath->setChecked(!downloadPath.isEmpty());
     }
 }
