@@ -30,24 +30,33 @@
 
 #include <QString>
 
+#include "base/global.h"
 #include "base/logger.h"
 #include "base/preferences.h"
 #include "base/utils/password.h"
 #include "apierror.h"
 #include "isessionmanager.h"
 
+AuthController::AuthController(ISessionManager *sessionManager, IApplication *app, QObject *parent)
+    : APIController(app, parent)
+    , m_sessionManager {sessionManager}
+{
+}
+
 void AuthController::loginAction()
 {
-    if (sessionManager()->session()) {
-        setResult(QLatin1String("Ok."));
+    if (m_sessionManager->session())
+    {
+        setResult(u"Ok."_qs);
         return;
     }
 
-    const QString clientAddr {sessionManager()->clientId()};
-    const QString usernameFromWeb {params()["username"]};
-    const QString passwordFromWeb {params()["password"]};
+    const QString clientAddr {m_sessionManager->clientId()};
+    const QString usernameFromWeb {params()[u"username"_qs]};
+    const QString passwordFromWeb {params()[u"password"_qs]};
 
-    if (isBanned()) {
+    if (isBanned())
+    {
         LogMsg(tr("WebAPI login failure. Reason: IP has been banned, IP: %1, username: %2")
                 .arg(clientAddr, usernameFromWeb)
             , Log::WARNING);
@@ -62,17 +71,19 @@ void AuthController::loginAction()
     const bool usernameEqual = Utils::Password::slowEquals(usernameFromWeb.toUtf8(), username.toUtf8());
     const bool passwordEqual = Utils::Password::PBKDF2::verify(secret, passwordFromWeb);
 
-    if (usernameEqual && passwordEqual) {
+    if (usernameEqual && passwordEqual)
+    {
         m_clientFailedLogins.remove(clientAddr);
 
-        sessionManager()->sessionStart();
-        setResult(QLatin1String("Ok."));
+        m_sessionManager->sessionStart();
+        setResult(u"Ok."_qs);
         LogMsg(tr("WebAPI login success. IP: %1").arg(clientAddr));
     }
-    else {
+    else
+    {
         if (Preferences::instance()->getWebUIMaxAuthFailCount() > 0)
             increaseFailedAttempts();
-        setResult(QLatin1String("Fails."));
+        setResult(u"Fails."_qs);
         LogMsg(tr("WebAPI login failure. Reason: invalid credentials, attempt count: %1, IP: %2, username: %3")
                 .arg(QString::number(failedAttemptsCount()), clientAddr, usernameFromWeb)
             , Log::WARNING);
@@ -81,17 +92,18 @@ void AuthController::loginAction()
 
 void AuthController::logoutAction() const
 {
-    sessionManager()->sessionEnd();
+    m_sessionManager->sessionEnd();
 }
 
 bool AuthController::isBanned() const
 {
-    const auto failedLoginIter = m_clientFailedLogins.find(sessionManager()->clientId());
+    const auto failedLoginIter = m_clientFailedLogins.find(m_sessionManager->clientId());
     if (failedLoginIter == m_clientFailedLogins.end())
         return false;
 
     bool isBanned = (failedLoginIter->banTimer.remainingTime() >= 0);
-    if (isBanned && failedLoginIter->banTimer.hasExpired()) {
+    if (isBanned && failedLoginIter->banTimer.hasExpired())
+    {
         m_clientFailedLogins.erase(failedLoginIter);
         isBanned = false;
     }
@@ -101,17 +113,18 @@ bool AuthController::isBanned() const
 
 int AuthController::failedAttemptsCount() const
 {
-    return m_clientFailedLogins.value(sessionManager()->clientId()).failedAttemptsCount;
+    return m_clientFailedLogins.value(m_sessionManager->clientId()).failedAttemptsCount;
 }
 
 void AuthController::increaseFailedAttempts()
 {
     Q_ASSERT(Preferences::instance()->getWebUIMaxAuthFailCount() > 0);
 
-    FailedLogin &failedLogin = m_clientFailedLogins[sessionManager()->clientId()];
+    FailedLogin &failedLogin = m_clientFailedLogins[m_sessionManager->clientId()];
     ++failedLogin.failedAttemptsCount;
 
-    if (failedLogin.failedAttemptsCount >= Preferences::instance()->getWebUIMaxAuthFailCount()) {
+    if (failedLogin.failedAttemptsCount >= Preferences::instance()->getWebUIMaxAuthFailCount())
+    {
         // Max number of failed attempts reached
         // Start ban period
         failedLogin.banTimer.setRemainingTime(Preferences::instance()->getWebUIBanDuration());
