@@ -35,13 +35,15 @@
 
 class QString;
 
+struct ISessionManager;
+
 class AuthController : public APIController
 {
     Q_OBJECT
     Q_DISABLE_COPY_MOVE(AuthController)
 
 public:
-    using APIController::APIController;
+    explicit AuthController(ISessionManager *sessionManager, IApplication *app, QObject *parent = nullptr);
 
 private slots:
     void loginAction();
@@ -52,6 +54,8 @@ private:
     int failedAttemptsCount() const;
     void increaseFailedAttempts();
 
+    ISessionManager *m_sessionManager = nullptr;
+
     struct FailedLogin
     {
         int failedAttemptsCount = 0;
