@@ -34,6 +34,7 @@
 #include "base/bittorrent/infohash.h"
 #include "base/bittorrent/torrent.h"
 #include "base/bittorrent/trackerentry.h"
+#include "base/path.h"
 #include "base/tagset.h"
 #include "base/utils/fs.h"
 
@@ -44,43 +45,43 @@ namespace
         switch (state)
         {
         case BitTorrent::TorrentState::Error:
-            return QLatin1String("error");
+            return u"error"_qs;
         case BitTorrent::TorrentState::MissingFiles:
-            return QLatin1String("missingFiles");
+            return u"missingFiles"_qs;
         case BitTorrent::TorrentState::Uploading:
-            return QLatin1String("uploading");
+            return u"uploading"_qs;
         case BitTorrent::TorrentState::PausedUploading:
-            return QLatin1String("pausedUP");
+            return u"pausedUP"_qs;
         case BitTorrent::TorrentState::QueuedUploading:
-            return QLatin1String("queuedUP");
+            return u"queuedUP"_qs;
         case BitTorrent::TorrentState::StalledUploading:
-            return QLatin1String("stalledUP");
+            return u"stalledUP"_qs;
         case BitTorrent::TorrentState::CheckingUploading:
-            return QLatin1String("checkingUP");
+            return u"checkingUP"_qs;
         case BitTorrent::TorrentState::ForcedUploading:
-            return QLatin1String("forcedUP");
+            return u"forcedUP"_qs;
         case BitTorrent::TorrentState::Downloading:
-            return QLatin1String("downloading");
+            return u"downloading"_qs;
         case BitTorrent::TorrentState::DownloadingMetadata:
-            return QLatin1String("metaDL");
+            return u"metaDL"_qs;
         case BitTorrent::TorrentState::ForcedDownloadingMetadata:
-            return QLatin1String("forcedMetaDL");
+            return u"forcedMetaDL"_qs;
         case BitTorrent::TorrentState::PausedDownloading:
-            return QLatin1String("pausedDL");
+            return u"pausedDL"_qs;
         case BitTorrent::TorrentState::QueuedDownloading:
-            return QLatin1String("queuedDL");
+            return u"queuedDL"_qs;
         case BitTorrent::TorrentState::StalledDownloading:
-            return QLatin1String("stalledDL");
+            return u"stalledDL"_qs;
         case BitTorrent::TorrentState::CheckingDownloading:
-            return QLatin1String("checkingDL");
+            return u"checkingDL"_qs;
         case BitTorrent::TorrentState::ForcedDownloading:
-            return QLatin1String("forcedDL");
+            return u"forcedDL"_qs;
         case BitTorrent::TorrentState::CheckingResumeData:
-            return QLatin1String("checkingResumeData");
+            return u"checkingResumeData"_qs;
         case BitTorrent::TorrentState::Moving:
-            return QLatin1String("moving");
+            return u"moving"_qs;
         default:
-            return QLatin1String("unknown");
+            return u"unknown"_qs;
         }
     }
 }
@@ -127,12 +128,12 @@ QVariantMap serialize(const BitTorrent::Torrent &torrent)
         {KEY_TORRENT_FIRST_LAST_PIECE_PRIO, torrent.hasFirstLastPiecePriority()},
 
         {KEY_TORRENT_CATEGORY, torrent.category()},
-        {KEY_TORRENT_TAGS, torrent.tags().join(QLatin1String(", "))},
+        {KEY_TORRENT_TAGS, torrent.tags().join(u", "_qs)},
         {KEY_TORRENT_SUPER_SEEDING, torrent.superSeeding()},
         {KEY_TORRENT_FORCE_START, torrent.isForced()},
-        {KEY_TORRENT_SAVE_PATH, Utils::Fs::toNativePath(torrent.savePath())},
-        {KEY_TORRENT_DOWNLOAD_PATH, Utils::Fs::toNativePath(torrent.downloadPath())},
-        {KEY_TORRENT_CONTENT_PATH, Utils::Fs::toNativePath(torrent.contentPath())},
+        {KEY_TORRENT_SAVE_PATH, torrent.savePath().toString()},
+        {KEY_TORRENT_DOWNLOAD_PATH, torrent.downloadPath().toString()},
+        {KEY_TORRENT_CONTENT_PATH, torrent.contentPath().toString()},
         {KEY_TORRENT_ADDED_ON, torrent.addedTime().toSecsSinceEpoch()},
         {KEY_TORRENT_COMPLETION_ON, torrent.completedTime().toSecsSinceEpoch()},
         {KEY_TORRENT_TRACKER, torrent.currentTracker()},
@@ -153,7 +154,7 @@ QVariantMap serialize(const BitTorrent::Torrent &torrent)
         {KEY_TORRENT_LAST_SEEN_COMPLETE_TIME, torrent.lastSeenComplete().toSecsSinceEpoch()},
         {KEY_TORRENT_AUTO_TORRENT_MANAGEMENT, torrent.isAutoTMMEnabled()},
         {KEY_TORRENT_TIME_ACTIVE, torrent.activeTime()},
-        {KEY_TORRENT_SEEDING_TIME, torrent.seedingTime()},
+        {KEY_TORRENT_SEEDING_TIME, torrent.finishedTime()},
         {KEY_TORRENT_LAST_ACTIVITY_TIME, getLastActivityTime()},
         {KEY_TORRENT_AVAILABILITY, torrent.distributedCopies()},
 
