@@ -32,13 +32,16 @@
 
 #include <QJsonObject>
 #include <QMetaObject>
-#include <QThread>
+#include <QThreadPool>
 
+#include "base/bittorrent/cachestatus.h"
 #include "base/bittorrent/infohash.h"
 #include "base/bittorrent/peeraddress.h"
 #include "base/bittorrent/peerinfo.h"
 #include "base/bittorrent/session.h"
+#include "base/bittorrent/sessionstatus.h"
 #include "base/bittorrent/torrent.h"
+#include "base/bittorrent/torrentinfo.h"
 #include "base/bittorrent/trackerentry.h"
 #include "base/global.h"
 #include "base/net/geoipmanager.h"
@@ -46,7 +49,6 @@
 #include "base/utils/string.h"
 #include "apierror.h"
 #include "freediskspacechecker.h"
-#include "isessionmanager.h"
 #include "serialize/serialize_torrent.h"
 
 namespace
@@ -54,58 +56,59 @@ namespace
     const int FREEDISKSPACE_CHECK_TIMEOUT = 30000;
 
     // Sync main data keys
-    const char KEY_SYNC_MAINDATA_QUEUEING[] = "queueing";
-    const char KEY_SYNC_MAINDATA_REFRESH_INTERVAL[] = "refresh_interval";
-    const char KEY_SYNC_MAINDATA_USE_ALT_SPEED_LIMITS[] = "use_alt_speed_limits";
+    const QString KEY_SYNC_MAINDATA_QUEUEING = u"queueing"_qs;
+    const QString KEY_SYNC_MAINDATA_REFRESH_INTERVAL = u"refresh_interval"_qs;
+    const QString KEY_SYNC_MAINDATA_USE_ALT_SPEED_LIMITS = u"use_alt_speed_limits"_qs;
 
     // Sync torrent peers keys
-    const char KEY_SYNC_TORRENT_PEERS_SHOW_FLAGS[] = "show_flags";
+    const QString KEY_SYNC_TORRENT_PEERS_SHOW_FLAGS = u"show_flags"_qs;
 
     // Peer keys
-    const char KEY_PEER_CLIENT[] = "client";
-    const char KEY_PEER_CONNECTION_TYPE[] = "connection";
-    const char KEY_PEER_COUNTRY[] = "country";
-    const char KEY_PEER_COUNTRY_CODE[] = "country_code";
-    const char KEY_PEER_DOWN_SPEED[] = "dl_speed";
-    const char KEY_PEER_FILES[] = "files";
-    const char KEY_PEER_FLAGS[] = "flags";
-    const char KEY_PEER_FLAGS_DESCRIPTION[] = "flags_desc";
-    const char KEY_PEER_IP[] = "ip";
-    const char KEY_PEER_PORT[] = "port";
-    const char KEY_PEER_PROGRESS[] = "progress";
-    const char KEY_PEER_RELEVANCE[] = "relevance";
-    const char KEY_PEER_TOT_DOWN[] = "downloaded";
-    const char KEY_PEER_TOT_UP[] = "uploaded";
-    const char KEY_PEER_UP_SPEED[] = "up_speed";
+    const QString KEY_PEER_CLIENT = u"client"_qs;
+    const QString KEY_PEER_ID_CLIENT = u"peer_id_client"_qs;
+    const QString KEY_PEER_CONNECTION_TYPE = u"connection"_qs;
+    const QString KEY_PEER_COUNTRY = u"country"_qs;
+    const QString KEY_PEER_COUNTRY_CODE = u"country_code"_qs;
+    const QString KEY_PEER_DOWN_SPEED = u"dl_speed"_qs;
+    const QString KEY_PEER_FILES = u"files"_qs;
+    const QString KEY_PEER_FLAGS = u"flags"_qs;
+    const QString KEY_PEER_FLAGS_DESCRIPTION = u"flags_desc"_qs;
+    const QString KEY_PEER_IP = u"ip"_qs;
+    const QString KEY_PEER_PORT = u"port"_qs;
+    const QString KEY_PEER_PROGRESS = u"progress"_qs;
+    const QString KEY_PEER_RELEVANCE = u"relevance"_qs;
+    const QString KEY_PEER_TOT_DOWN = u"downloaded"_qs;
+    const QString KEY_PEER_TOT_UP = u"uploaded"_qs;
+    const QString KEY_PEER_UP_SPEED = u"up_speed"_qs;
 
     // TransferInfo keys
-    const char KEY_TRANSFER_CONNECTION_STATUS[] = "connection_status";
-    const char KEY_TRANSFER_DHT_NODES[] = "dht_nodes";
-    const char KEY_TRANSFER_DLDATA[] = "dl_info_data";
-    const char KEY_TRANSFER_DLRATELIMIT[] = "dl_rate_limit";
-    const char KEY_TRANSFER_DLSPEED[] = "dl_info_speed";
-    const char KEY_TRANSFER_FREESPACEONDISK[] = "free_space_on_disk";
-    const char KEY_TRANSFER_UPDATA[] = "up_info_data";
-    const char KEY_TRANSFER_UPRATELIMIT[] = "up_rate_limit";
-    const char KEY_TRANSFER_UPSPEED[] = "up_info_speed";
+    const QString KEY_TRANSFER_CONNECTION_STATUS = u"connection_status"_qs;
+    const QString KEY_TRANSFER_DHT_NODES = u"dht_nodes"_qs;
+    const QString KEY_TRANSFER_DLDATA = u"dl_info_data"_qs;
+    const QString KEY_TRANSFER_DLRATELIMIT = u"dl_rate_limit"_qs;
+    const QString KEY_TRANSFER_DLSPEED = u"dl_info_speed"_qs;
+    const QString KEY_TRANSFER_FREESPACEONDISK = u"free_space_on_disk"_qs;
+    const QString KEY_TRANSFER_UPDATA = u"up_info_data"_qs;
+    const QString KEY_TRANSFER_UPRATELIMIT = u"up_rate_limit"_qs;
+    const QString KEY_TRANSFER_UPSPEED = u"up_info_speed"_qs;
 
     // Statistics keys
-    const char KEY_TRANSFER_ALLTIME_DL[] = "alltime_dl";
-    const char KEY_TRANSFER_ALLTIME_UL[] = "alltime_ul";
-    const char KEY_TRANSFER_AVERAGE_TIME_QUEUE[] = "average_time_queue";
-    const char KEY_TRANSFER_GLOBAL_RATIO[] = "global_ratio";
-    const char KEY_TRANSFER_QUEUED_IO_JOBS[] = "queued_io_jobs";
-    const char KEY_TRANSFER_READ_CACHE_HITS[] = "read_cache_hits";
-    const char KEY_TRANSFER_READ_CACHE_OVERLOAD[] = "read_cache_overload";
-    const char KEY_TRANSFER_TOTAL_BUFFERS_SIZE[] = "total_buffers_size";
-    const char KEY_TRANSFER_TOTAL_PEER_CONNECTIONS[] = "total_peer_connections";
-    const char KEY_TRANSFER_TOTAL_QUEUED_SIZE[] = "total_queued_size";
-    const char KEY_TRANSFER_TOTAL_WASTE_SESSION[] = "total_wasted_session";
-    const char KEY_TRANSFER_WRITE_CACHE_OVERLOAD[] = "write_cache_overload";
+    const QString KEY_TRANSFER_ALLTIME_DL = u"alltime_dl"_qs;
+    const QString KEY_TRANSFER_ALLTIME_UL = u"alltime_ul"_qs;
+    const QString KEY_TRANSFER_AVERAGE_TIME_QUEUE = u"average_time_queue"_qs;
+    const QString KEY_TRANSFER_GLOBAL_RATIO = u"global_ratio"_qs;
+    const QString KEY_TRANSFER_QUEUED_IO_JOBS = u"queued_io_jobs"_qs;
+    const QString KEY_TRANSFER_READ_CACHE_HITS = u"read_cache_hits"_qs;
+    const QString KEY_TRANSFER_READ_CACHE_OVERLOAD = u"read_cache_overload"_qs;
+    const QString KEY_TRANSFER_TOTAL_BUFFERS_SIZE = u"total_buffers_size"_qs;
+    const QString KEY_TRANSFER_TOTAL_PEER_CONNECTIONS = u"total_peer_connections"_qs;
+    const QString KEY_TRANSFER_TOTAL_QUEUED_SIZE = u"total_queued_size"_qs;
+    const QString KEY_TRANSFER_TOTAL_WASTE_SESSION = u"total_wasted_session"_qs;
+    const QString KEY_TRANSFER_WRITE_CACHE_OVERLOAD = u"write_cache_overload"_qs;
 
-    const char KEY_FULL_UPDATE[] = "full_update";
-    const char KEY_RESPONSE_ID[] = "rid";
-    const char KEY_SUFFIX_REMOVED[] = "_removed";
+    const QString KEY_FULL_UPDATE = u"full_update"_qs;
+    const QString KEY_RESPONSE_ID = u"rid"_qs;
+    const QString KEY_SUFFIX_REMOVED = u"_removed"_qs;
 
     void processMap(const QVariantMap &prevData, const QVariantMap &data, QVariantMap &syncData);
     void processHash(QVariantHash prevData, const QVariantHash &data, QVariantMap &syncData, QVariantList &removedItems);
@@ -126,24 +129,24 @@ namespace
         map[KEY_TRANSFER_DLRATELIMIT] = session->downloadSpeedLimit();
         map[KEY_TRANSFER_UPRATELIMIT] = session->uploadSpeedLimit();
 
-        const quint64 atd = session->getAlltimeDL();
-        const quint64 atu = session->getAlltimeUL();
+        const qint64 atd = sessionStatus.allTimeDownload;
+        const qint64 atu = sessionStatus.allTimeUpload;
         map[KEY_TRANSFER_ALLTIME_DL] = atd;
         map[KEY_TRANSFER_ALLTIME_UL] = atu;
         map[KEY_TRANSFER_TOTAL_WASTE_SESSION] = sessionStatus.totalWasted;
-        map[KEY_TRANSFER_GLOBAL_RATIO] = ((atd > 0) && (atu > 0)) ? Utils::String::fromDouble(static_cast<qreal>(atu) / atd, 2) : "-";
+        map[KEY_TRANSFER_GLOBAL_RATIO] = ((atd > 0) && (atu > 0)) ? Utils::String::fromDouble(static_cast<qreal>(atu) / atd, 2) : u"-"_qs;
         map[KEY_TRANSFER_TOTAL_PEER_CONNECTIONS] = sessionStatus.peersCount;
 
         const qreal readRatio = cacheStatus.readRatio;  // TODO: remove when LIBTORRENT_VERSION_NUM >= 20000
-        map[KEY_TRANSFER_READ_CACHE_HITS] = (readRatio > 0) ? Utils::String::fromDouble(100 * readRatio, 2) : "0";
+        map[KEY_TRANSFER_READ_CACHE_HITS] = (readRatio > 0) ? Utils::String::fromDouble(100 * readRatio, 2) : u"0"_qs;
         map[KEY_TRANSFER_TOTAL_BUFFERS_SIZE] = cacheStatus.totalUsedBuffers * 16 * 1024;
 
         map[KEY_TRANSFER_WRITE_CACHE_OVERLOAD] = ((sessionStatus.diskWriteQueue > 0) && (sessionStatus.peersCount > 0))
             ? Utils::String::fromDouble((100. * sessionStatus.diskWriteQueue / sessionStatus.peersCount), 2)
-            : QLatin1String("0");
+            : u"0"_qs;
         map[KEY_TRANSFER_READ_CACHE_OVERLOAD] = ((sessionStatus.diskReadQueue > 0) && (sessionStatus.peersCount > 0))
             ? Utils::String::fromDouble((100. * sessionStatus.diskReadQueue / sessionStatus.peersCount), 2)
-            : QLatin1String("0");
+            : u"0"_qs;
 
         map[KEY_TRANSFER_QUEUED_IO_JOBS] = cacheStatus.jobQueueLength;
         map[KEY_TRANSFER_AVERAGE_TIME_QUEUE] = cacheStatus.averageJobTime;
@@ -151,8 +154,8 @@ namespace
 
         map[KEY_TRANSFER_DHT_NODES] = sessionStatus.dhtNodes;
         map[KEY_TRANSFER_CONNECTION_STATUS] = session->isListening()
-            ? (sessionStatus.hasIncomingConnections ? "connected" : "firewalled")
-            : "disconnected";
+            ? (sessionStatus.hasIncomingConnections ? u"connected"_qs : u"firewalled"_qs)
+            : u"disconnected"_qs;
 
         return map;
     }
@@ -173,7 +176,7 @@ namespace
             switch (static_cast<QMetaType::Type>(value.type()))
             {
             case QMetaType::QVariantMap:
-            {
+                {
                     QVariantMap map;
                     processMap(prevData[key].toMap(), value.toMap(), map);
                     if (!map.isEmpty())
@@ -181,7 +184,7 @@ namespace
                 }
                 break;
             case QMetaType::QVariantHash:
-            {
+                {
                     QVariantMap map;
                     processHash(prevData[key].toHash(), value.toHash(), map, removedItems);
                     if (!map.isEmpty())
@@ -191,7 +194,7 @@ namespace
                 }
                 break;
             case QMetaType::QVariantList:
-            {
+                {
                     QVariantList list;
                     processList(prevData[key].toList(), value.toList(), list, removedItems);
                     if (!list.isEmpty())
@@ -215,8 +218,8 @@ namespace
                 break;
             default:
                 Q_ASSERT_X(false, "processMap"
-                           , QString("Unexpected type: %1")
-                           .arg(QMetaType::typeName(static_cast<QMetaType::Type>(value.type())))
+                           , u"Unexpected type: %1"_qs
+                           .arg(QString::fromLatin1(QMetaType::typeName(static_cast<QMetaType::Type>(value.type()))))
                            .toUtf8().constData());
             }
         }
@@ -366,27 +369,13 @@ namespace
     }
 }
 
-SyncController::SyncController(ISessionManager *sessionManager, QObject *parent)
-    : APIController(sessionManager, parent)
+SyncController::SyncController(IApplication *app, QObject *parent)
+    : APIController(app, parent)
 {
-    m_freeDiskSpaceThread = new QThread(this);
-    m_freeDiskSpaceChecker = new FreeDiskSpaceChecker();
-    m_freeDiskSpaceChecker->moveToThread(m_freeDiskSpaceThread);
-
-    connect(m_freeDiskSpaceThread, &QThread::finished, m_freeDiskSpaceChecker, &QObject::deleteLater);
-    connect(m_freeDiskSpaceChecker, &FreeDiskSpaceChecker::checked, this, &SyncController::freeDiskSpaceSizeUpdated);
-
-    m_freeDiskSpaceThread->start();
     invokeChecker();
     m_freeDiskSpaceElapsedTimer.start();
 }
 
-SyncController::~SyncController()
-{
-    m_freeDiskSpaceThread->quit();
-    m_freeDiskSpaceThread->wait();
-}
-
 // The function returns the changed data from the server to synchronize with the web client.
 // Return value is map in JSON format.
 // Map contain the key:
@@ -456,9 +445,6 @@ void SyncController::maindataAction()
 
     QVariantMap data;
 
-    QVariantMap lastResponse = sessionManager()->session()->getData(QLatin1String("syncMainDataLastResponse")).toMap();
-    QVariantMap lastAcceptedResponse = sessionManager()->session()->getData(QLatin1String("syncMainDataLastAcceptedResponse")).toMap();
-
     QVariantHash torrents;
     QHash<QString, QStringList> trackers;
     for (const BitTorrent::Torrent *torrent : asConst(session->torrents()))
@@ -470,8 +456,8 @@ void SyncController::maindataAction()
 
         // Calculated last activity time can differ from actual value by up to 10 seconds (this is a libtorrent issue).
         // So we don't need unnecessary updates of last activity time in response.
-        const auto iterTorrents = lastResponse.find("torrents");
-        if (iterTorrents != lastResponse.end())
+        const auto iterTorrents = m_lastMaindataResponse.find(u"torrents"_qs);
+        if (iterTorrents != m_lastMaindataResponse.end())
         {
             const QVariantHash lastResponseTorrents = iterTorrents->toHash();
             const auto iterID = lastResponseTorrents.find(torrentID.toString());
@@ -495,7 +481,7 @@ void SyncController::maindataAction()
 
         torrents[torrentID.toString()] = map;
     }
-    data["torrents"] = torrents;
+    data[u"torrents"_qs] = torrents;
 
     QVariantHash categories;
     const QStringList categoriesList = session->categories();
@@ -503,37 +489,34 @@ void SyncController::maindataAction()
     {
         const BitTorrent::CategoryOptions categoryOptions = session->categoryOptions(categoryName);
         QJsonObject category = categoryOptions.toJSON();
-        // adjust it to be compatible with exisitng WebAPI
-        category[QLatin1String("savePath")] = category.take(QLatin1String("save_path"));
-        category.insert(QLatin1String("name"), categoryName);
+        // adjust it to be compatible with existing WebAPI
+        category[u"savePath"_qs] = category.take(u"save_path"_qs);
+        category.insert(u"name"_qs, categoryName);
         categories[categoryName] = category.toVariantMap();
     }
-    data["categories"] = categories;
+    data[u"categories"_qs] = categories;
 
     QVariantList tags;
     for (const QString &tag : asConst(session->tags()))
         tags << tag;
-    data["tags"] = tags;
+    data[u"tags"_qs] = tags;
 
     QVariantHash trackersHash;
     for (auto i = trackers.constBegin(); i != trackers.constEnd(); ++i)
     {
         trackersHash[i.key()] = i.value();
     }
-    data["trackers"] = trackersHash;
+    data[u"trackers"_qs] = trackersHash;
 
     QVariantMap serverState = getTransferInfo();
     serverState[KEY_TRANSFER_FREESPACEONDISK] = getFreeDiskSpace();
     serverState[KEY_SYNC_MAINDATA_QUEUEING] = session->isQueueingSystemEnabled();
     serverState[KEY_SYNC_MAINDATA_USE_ALT_SPEED_LIMITS] = session->isAltGlobalSpeedLimitEnabled();
     serverState[KEY_SYNC_MAINDATA_REFRESH_INTERVAL] = session->refreshInterval();
-    data["server_state"] = serverState;
+    data[u"server_state"_qs] = serverState;
 
-    const int acceptedResponseId {params()["rid"].toInt()};
-    setResult(QJsonObject::fromVariantMap(generateSyncData(acceptedResponseId, data, lastAcceptedResponse, lastResponse)));
-
-    sessionManager()->session()->setData(QLatin1String("syncMainDataLastResponse"), lastResponse);
-    sessionManager()->session()->setData(QLatin1String("syncMainDataLastAcceptedResponse"), lastAcceptedResponse);
+    const int acceptedResponseId = params()[u"rid"_qs].toInt();
+    setResult(QJsonObject::fromVariantMap(generateSyncData(acceptedResponseId, data, m_lastAcceptedMaindataResponse, m_lastMaindataResponse)));
 }
 
 // GET param:
@@ -541,11 +524,8 @@ void SyncController::maindataAction()
 //   - rid (int): last response id
 void SyncController::torrentPeersAction()
 {
-    auto lastResponse = sessionManager()->session()->getData(QLatin1String("syncTorrentPeersLastResponse")).toMap();
-    auto lastAcceptedResponse = sessionManager()->session()->getData(QLatin1String("syncTorrentPeersLastAcceptedResponse")).toMap();
-
-    const auto id = BitTorrent::TorrentID::fromString(params()["hash"]);
-    const BitTorrent::Torrent *torrent = BitTorrent::Session::instance()->findTorrent(id);
+    const auto id = BitTorrent::TorrentID::fromString(params()[u"hash"_qs]);
+    const BitTorrent::Torrent *torrent = BitTorrent::Session::instance()->getTorrent(id);
     if (!torrent)
         throw APIError(APIErrorType::NotFound);
 
@@ -567,6 +547,7 @@ void SyncController::torrentPeersAction()
             {KEY_PEER_IP, pi.address().ip.toString()},
             {KEY_PEER_PORT, pi.address().port},
             {KEY_PEER_CLIENT, pi.client()},
+            {KEY_PEER_ID_CLIENT, pi.peerIdClient()},
             {KEY_PEER_PROGRESS, pi.progress()},
             {KEY_PEER_DOWN_SPEED, pi.payloadDownSpeed()},
             {KEY_PEER_UP_SPEED, pi.payloadUpSpeed()},
@@ -579,7 +560,14 @@ void SyncController::torrentPeersAction()
         };
 
         if (torrent->hasMetadata())
-            peer.insert(KEY_PEER_FILES, torrent->info().filesForPiece(pi.downloadingPieceIndex()).join('\n'));
+        {
+            const PathList filePaths = torrent->info().filesForPiece(pi.downloadingPieceIndex());
+            QStringList filesForPiece;
+            filesForPiece.reserve(filePaths.size());
+            for (const Path &filePath : filePaths)
+                filesForPiece.append(filePath.toString());
+            peer.insert(KEY_PEER_FILES, filesForPiece.join(u'\n'));
+        }
 
         if (resolvePeerCountries)
         {
@@ -589,32 +577,36 @@ void SyncController::torrentPeersAction()
 
         peers[pi.address().toString()] = peer;
     }
-    data["peers"] = peers;
+    data[u"peers"_qs] = peers;
 
-    const int acceptedResponseId {params()["rid"].toInt()};
-    setResult(QJsonObject::fromVariantMap(generateSyncData(acceptedResponseId, data, lastAcceptedResponse, lastResponse)));
-
-    sessionManager()->session()->setData(QLatin1String("syncTorrentPeersLastResponse"), lastResponse);
-    sessionManager()->session()->setData(QLatin1String("syncTorrentPeersLastAcceptedResponse"), lastAcceptedResponse);
+    const int acceptedResponseId = params()[u"rid"_qs].toInt();
+    setResult(QJsonObject::fromVariantMap(generateSyncData(acceptedResponseId, data, m_lastAcceptedPeersResponse, m_lastPeersResponse)));
 }
 
 qint64 SyncController::getFreeDiskSpace()
 {
     if (m_freeDiskSpaceElapsedTimer.hasExpired(FREEDISKSPACE_CHECK_TIMEOUT))
-    {
         invokeChecker();
-        m_freeDiskSpaceElapsedTimer.restart();
-    }
 
     return m_freeDiskSpace;
 }
 
-void SyncController::freeDiskSpaceSizeUpdated(qint64 freeSpaceSize)
+void SyncController::invokeChecker()
 {
-    m_freeDiskSpace = freeSpaceSize;
-}
+    if (m_isFreeDiskSpaceCheckerRunning)
+        return;
 
-void SyncController::invokeChecker() const
-{
-    QMetaObject::invokeMethod(m_freeDiskSpaceChecker, &FreeDiskSpaceChecker::check, Qt::QueuedConnection);
+    auto *freeDiskSpaceChecker = new FreeDiskSpaceChecker;
+    connect(freeDiskSpaceChecker, &FreeDiskSpaceChecker::checked, this, [this](const qint64 freeSpaceSize)
+    {
+        m_freeDiskSpace = freeSpaceSize;
+        m_isFreeDiskSpaceCheckerRunning = false;
+        m_freeDiskSpaceElapsedTimer.restart();
+    });
+    connect(freeDiskSpaceChecker, &FreeDiskSpaceChecker::checked, freeDiskSpaceChecker, &QObject::deleteLater);
+    m_isFreeDiskSpaceCheckerRunning = true;
+    QThreadPool::globalInstance()->start([freeDiskSpaceChecker]
+    {
+        freeDiskSpaceChecker->check();
+    });
 }
