@@ -1289,11 +1289,11 @@ function registerMagnetHandler() {
         return;
     }
 
-    const hashParams = getHashParamsFromUrl();
-    hashParams.download = '';
-
-    const templateHashString = Object.toQueryString(hashParams).replace('download=', 'download=%s');
+    const hashString = location.hash ? location.hash.replace(/^#/, '') : '';
+    const hashParams = new URLSearchParams(hashString);
+    hashParams.set('download', '');
 
+    const templateHashString = hashParams.toString().replace('download=', 'download=%s');
     const templateUrl = location.origin + location.pathname
         + location.search + '#' + templateHashString;
 
@@ -1313,11 +1313,6 @@ function handleDownloadParam() {
     showDownloadPage([url]);
 }
 
-function getHashParamsFromUrl() {
-    const hashString = location.hash ? location.hash.replace(/^#/, '') : '';
-    return (hashString.length > 0) ? String.parseQueryString(hashString) : {};
-}
-
 function closeWindows() {
     MochaUI.closeAll();
 }
